#!/bin/sh
# *****************************************************************************
# ***                    Programm TkPlot                                    ***
# *****************************************************************************

# Original-Version vom 14.04.1999 (Institut)

# H. Ritter
# Institut fuer Kristallographie
# der Universitaet Tuebingen
# Charlottenstrasse 33
# D-72070 Tuebingen
# Germany
# Tel: +49(0)7071-2976389
# E-mail: harald.ritter@uni-tuebingen.de

# Vernderungen und Erweiterungen ab Oktober 1999 durch
# U. Amann
# ebenfalls Institut fr Kristallographie der Uni Tbingen

# ab 01.08.2001 in version control system RCS auf pkibm8
# eingetragen 

# $Id: tkplot.tcl,v 1.13 2001/01/23 11:05:04 uamann Exp $
# $Log: tkplot.tcl,v $
# Revision 1.13  2001/01/23 11:05:04  uamann
# weitere .gauge Aufrufe entfernt
#
# Revision 1.12  2001/01/23 11:00:15  uamann
# Gauge-Fortschrittsbalken anzeigen rausgenommen,
# da sie einen Groteil der Rechenzeit beanspruchen
#
# Revision 1.11  2001/01/22 17:40:17  uamann
# Ausschnitt-vergrerung gendert:
# rechte Maustaste drcken und halten == erste Ecke,
# rechte Maustaste loslassen == zweite Ecke
#
# Revision 1.10  2001/01/22 17:36:04  uamann
# Problem aus revision 1.9 bereinigt,
#
# Revision 1.9  2001/01/22 17:31:24  uamann
# bei Plotfiles, bei denen $step nicht mit der tatschlichen
# Differenz der 2Theta-Werte bereinstimmt klappt das
# Berechnenn der Positionen der hkl-Werte nicht, deshalb
# wurde als Standard fr hkl-Werte "below mark" gesetzt,
# evtl. sollte die Position nur berechnet werden, wenn auch Marks gesetzt
# werden sollen
#
# Revision 1.8  2001/01/22 16:32:23  uamann
# Starten von simref26 ist nun wieder mglich
#




# the next line restarts using wish \
exec wish8.0 "$0" "$@"


# ----------------------------------------------------------------------
#  EXAMPLE: simple notice dialog
# ----------------------------------------------------------------------
#  Effective Tcl/Tk Programming
#    Mark Harrison, DSC Communications Corp.
#    Michael McLennan, Bell Labs Innovations for Lucent Technologies
#    Addison-Wesley Professional Computing Series
# ======================================================================
#  Copyright (c) 1996-1997  Lucent Technologies Inc. and Mark Harrison
# ======================================================================

option add *Notice*dismiss.text "Dismiss" widgetDefault

proc notice_show {mesg {icon "info"}} {
    set top [dialog_create Notice]

    set x [expr [winfo rootx .]+50]
    set y [expr [winfo rooty .]+50]
    wm geometry $top "+$x+$y"

    set info [dialog_info $top]
    label $info.icon -bitmap $icon
    pack $info.icon -side left -padx 8 -pady 8
    label $info.mesg -text $mesg -wraplength 4i
    pack $info.mesg -side right -expand yes -fill both -padx 8 -pady 8

    set cntls [dialog_controls $top]
    button $cntls.dismiss -command "destroy $top"
    pack $cntls.dismiss -pady 4

    focus $cntls.dismiss
    return $top
}

# ----------------------------------------------------------------------
#  EXAMPLE: procedures to create dialogs
# ----------------------------------------------------------------------
#  Effective Tcl/Tk Programming
#    Mark Harrison, DSC Communications Corp.
#    Michael McLennan, Bell Labs Innovations for Lucent Technologies
#    Addison-Wesley Professional Computing Series
# ======================================================================
#  Copyright (c) 1996-1997  Lucent Technologies Inc. and Mark Harrison
# ======================================================================

proc dialog_create {class {win "auto"}} {
    if {$win == "auto"} {
        set count 0
        set win ".dialog[incr count]"
        while {[winfo exists $win]} {
            set win ".dialog[incr count]"
        }
    }
    toplevel $win -class $class

    frame $win.info
    pack $win.info -expand yes -fill both -padx 4 -pady 4

    frame $win.sep -height 2 -borderwidth 1 -relief sunken
    pack $win.sep -fill x -pady 4

    frame $win.controls
    pack $win.controls -fill x -padx 4 -pady 4

    wm title $win $class
    wm group $win .

    after idle [format {
        update idletasks
        wm minsize %s [winfo reqwidth %s] [winfo reqheight %s]
    } $win $win $win]

    return $win
}

proc dialog_info {win} {
    return "$win.info"
}

proc dialog_controls {win} {
    return "$win.controls"
}

proc dialog_wait {win varName} {
    dialog_safeguard $win

    set x [expr [winfo rootx .]+50]
    set y [expr [winfo rooty .]+50]
    wm geometry $win "+$x+$y"

    wm deiconify $win
    grab set $win

    vwait $varName

    grab release $win
    wm withdraw $win
}

bind modalDialog <ButtonPress> {
    wm deiconify %W
    raise %W
}
proc dialog_safeguard {win} {
    if {[lsearch [bindtags $win] modalDialog] < 0} {
        bindtags $win [linsert [bindtags $win] 0 modalDialog]
    }
}

# ----------------------------------------------------------------------
#  EXAMPLE: simple printer dialog
# ----------------------------------------------------------------------
#  Effective Tcl/Tk Programming
#    Mark Harrison, DSC Communications Corp.
#    Michael McLennan, Bell Labs Innovations for Lucent Technologies
#    Addison-Wesley Professional Computing Series
# ======================================================================
#  Copyright (c) 1996-1997  Lucent Technologies Inc. and Mark Harrison
# ======================================================================

proc printer_create {top} {
    global prInfo

    set top [dialog_create Printer $top]

    set info [dialog_info $top]
    radiobutton $info.printer \
        -text "Send to printer with command: " \
        -variable prInfo($top-where) -value "printer"
    entry $info.printerCmd
    radiobutton $info.file \
        -text "Save output in file: " \
        -variable prInfo($top-where) -value "file"
    entry $info.fileName

    grid $info.printer    -row 0 -sticky w
    grid $info.printerCmd -row 1 -sticky ew
    grid rowconfigure $info 2 -minsize 6
    grid $info.file       -row 3 -sticky w
    grid $info.fileName   -row 4 -sticky ew

    $info.printerCmd insert 0 "lpr"
    $info.fileName insert 0 "output.ps"
    $info.printer invoke

    bind $info.printerCmd <FocusIn> "$info.printer invoke"
    bind $info.fileName <FocusIn> "$info.file invoke"

    set cntls [dialog_controls $top]
    button $cntls.ok -command "set prInfo($top-status) 1"
    pack $cntls.ok -side left -expand yes
    focus $cntls.ok

    button $cntls.cancel -command "set prInfo($top-status) 0"
    pack $cntls.cancel -side left -expand yes

    wm protocol $top WM_DELETE_WINDOW "$cntls.cancel invoke"
    wm withdraw $top

    return $top
}

proc printer_print {top cmd {ok "Print"} {cancel "Cancel"}} {
    global prInfo

    set cntls [dialog_controls $top]
    $cntls.ok configure -text $ok
    $cntls.cancel configure -text $cancel

    dialog_wait $top prInfo($top-status)

    if {$prInfo($top-status)} {
        switch $prInfo($top-where) {
            printer {
                set info [dialog_info $top]

                set print [format {
                    exec %s << [%s]
                } [$info.printerCmd get] $cmd]

                if {[catch $print result] != 0} {
                    notice_show $result error
                } elseif {[string trim $result] != ""} {
                    notice_show $result info
                } else {
                    notice_show "Document printed" info
                }
            }
            file {
                set info [dialog_info $top]
                set fname [$info.fileName get]

                set print [format {
                    set fid [open %s w]
                    puts -nonewline $fid [%s]
                    close $fid
                } $fname $cmd]

                if {[catch $print result] != 0} {
                    notice_show $result error
                } else {
                    notice_show "Output saved in file \"$fname\"" info
                }
            }
        }
    }
}

#==============================================================================
#              A c h s e n b e s c h r i f t u n g
#==============================================================================

proc axisCalc {rmin rmax} {
    global numTics beginTics stepTics

# Relativer Fehler fuer die if-Abfragen:

    set eps 0.00001

# Bestimmung der max. Zehnerpotenz km der Betraege von rmin, rmax

    set a1 [expr abs($rmin)]
    set a2 [expr abs($rmax)]
    if {$a2 > $a1} {
	set rm $a2
    } else {
	set rm $a1
    }
    set logrm [expr log10($rm)]
    set rdlogrm [expr round($logrm)]
    set fel [expr abs($logrm - $rdlogrm)]
    set gre [expr abs($logrm * $eps)]
    if {$fel <= $gre} {
	set km $rdlogrm
    } else {
	if {$rm < 1.0} {
	    set km [expr int($logrm) - 1]
	} else {
	    set km [expr int($logrm)]
	}
    } 

# Erste Markierungsposition beginTics, 
# die Abstaende der Markierungen stepTics bestimmen:

    set i 0
    set end 0
    while {$end == 0} {
	set teiler(1) [expr pow(10.,double($km + 1 - $i))]
	set teiler(2) [expr $teiler(1)/double(2.0)]
	set teiler(3) [expr $teiler(1)/double(5.0)]
	incr i
	for {set n 1} {$n <= 3} {incr n} {
	    set d1 [expr int($rmin/double($teiler($n)))]
	    set d2 [expr int($rmax/double($teiler($n)))]
	    set iz [expr abs($d2-$d1)]
	    if {$iz > 4} {
		set end 1
		break
	    } else {
		set numTics $iz
		set stepTics $teiler($n)
		set beginTics [expr $d1 * $stepTics]
            }
	}
    }

    if {$rmin < [expr -1. * $eps] || [expr abs($beginTics-$rmin)] <= $eps} {
	set $beginTics [expr $beginTics - $stepTics]
	incr numTics
    } elseif {$beginTics < $rmin && [expr abs($rmin)] > $eps} {
	set beginTics [expr $beginTics + $stepTics]
    }

}

#==============================================================================
#                           Diagramm neu zeichnen
#==============================================================================

proc diagram_redraw {type} {
    global dinA4 fileSpec

    .diagram delete all
    if {[info exists connect1] == 1} {unset connect1}
    update

    set x [winfo width .diagram]
    set y [winfo height .diagram]
    
    if {$x != 0. && $y != 0.} {
	if {[expr double($x) / double($y)] >= $dinA4} {
	    set breite [expr $y * $dinA4]
	    set hoehe $y
	} else {
	    set breite $x
	    set hoehe [expr double($x) / $dinA4]
	}
	if {$type == "data"} {
	    if {$fileSpec == "FOR" || $fileSpec == "ERG"} {
		diagram_col $breite $hoehe
	    } else {
		diagram_data $breite $hoehe
	    }
	} else {
	    diagram_W20 $breite $hoehe
        }
     }

}

#==============================================================================
#                 Mit W20 gefittetes Diagramm zeichnen
#==============================================================================

proc diagram_W20 {breite hoehe} {
    global w20symbolShape w20symbolSize w20symbolFillColor \
	w20symbolColor w20lineWidth w20lineDraw w20lineColor diagramColor \
	plotTitle y_obs y_calc y_diff y_back twoTheta_w20 ndata \
	xmin xmax ymin ymax twoThPos numRef axesColor axesWidth \
	diagramWidth diagramHeight xAxisLabel yAxisLabel phsZahl refPhase \
	beginTics stepTics numTics ref_h ref_k ref_l

### Einstellungen:

    set titleSize 14
    set titleFont "-*-times-medium-i-normal--$titleSize-140-*"

    set xNumSize 12

    set xBeschSize 14
    set xBeschFont "-*-times-medium-i-normal--$xBeschSize-140-*"

# Dimensionierung:

    set h1 30
    set h3 60
    set h2 [expr $hoehe - $h1 -$h3]
    set h21 [expr $h2 * double(11)/15]
    set h22 [expr $h2 * double(1)/15]
    set h23 [expr $h2 * double(3)/15]

    set b1 80
    set b3 20
    set b2 [expr $breite - $b1 - $b3]

    set diagramWidth $b2
    set diagramHeight $h21

# Faktor fuer die Umrechnung der Werte in Pixel:
 
    set dx [expr $b2 / double($xmax(W20) - $xmin(W20))]
    set dy [expr $h21 / double($ymax(W20) - $ymin(W20))]

### Tics:

    set t [expr double($b2)/70]
    set ts [expr double($b2)/100]
 
# Tics und Beschriftung der X-Achse:

    axisCalc $xmin(W20) $xmax(W20)
    set beginXTics $beginTics
    set stepXTics $stepTics
    set numXTics $numTics

# Large Tics:

    for {set i 1} {$i <= $numXTics} {incr i} {
	set zahl [expr $beginXTics + ($i-1) * $stepXTics]
	set x1 [expr $b1 + $dx * ($zahl - $xmin(W20))]
	set x2 $x1
	set y1 [expr $h1 + $h2]
	set y2 [expr $h1 + $h2 - $t]
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
	set y1 $h1
	set y2 [expr $h1 + $t]
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
    }

# Small Tics:

    for {set i 0} {$i <= $numXTics} {incr i} {
	set zahl [expr $beginXTics + ($i-0.5) * $stepXTics]
	set x1 [expr $b1 + $dx * ($zahl - $xmin(W20))]
	if {$x1 >= $b1 && $x1 <= [expr $b1 + $b2]} { 
	    set x2 $x1
	    set y1 [expr $h1 + $h2]
	    set y2 [expr $h1 + $h2 - $ts]
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
	    set y1 $h1
	    set y2 [expr $h1 + $ts]
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
	}
    }

# Tics der Y-Achse:

    axisCalc $ymin(W20) $ymax(W20)
    set beginYTics $beginTics
    set stepYTics $stepTics
    set numYTics $numTics

# Large Tics:

    for {set i 1} {$i <= $numYTics} {incr i} {
	set x1 $b1
	set x2 [expr $b1 + $t]
	set zahl [expr $beginYTics + ($i-1) * $stepYTics]
	set y1 [expr $h1 + $h21 - $dy * ($zahl-$ymin(W20))]
	set y2 $y1
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
	set x1 [expr $b1 + $b2]
	set x2 [expr $x1 - $t]
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
    }

# Small Tics:

    for {set i 0} {$i <= $numYTics} {incr i} {
	set zahl [expr $beginYTics + ($i-0.5) * $stepYTics]
	set y1 [expr $h1 + $h21 - $dy * ($zahl-$ymin(W20))]
	if {$y1 >= $h1 && $y1 <= [expr $h1 + $h21]} { 
	    set x1 $b1
	    set x2 [expr $x1 + $ts]
	    set y2 $y1
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
	    set x1 [expr $b1 + $b2]
	    set x2 [expr $x1 - $ts]
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
        }
    }

### Symbole fuer beobachtete Werte (Messwerte):

    if {$w20symbolShape(1) == "Circle"} {
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 - $dy * ($y_obs($i)-$ymin(W20))]
	    circleDraw .diagram $x $y \
		    $w20symbolSize(1) $w20symbolColor(1) $w20symbolFillColor(1)
	}
    } elseif {$w20symbolShape(1) == "Square"} {
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 - $dy * ($y_obs($i)-$ymin(W20))]
	    squareDraw .diagram $x $y \
		    $w20symbolSize(1) $w20symbolColor(1) $w20symbolFillColor(1)
	}
    } elseif {$w20symbolShape(1) == "Up Triangle"} {
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 - $dy * ($y_obs($i)-$ymin(W20))]
	    upTriangleDraw .diagram $x $y \
		    $w20symbolSize(1) $w20symbolColor(1) $w20symbolFillColor(1)
	}
    } elseif {$w20symbolShape(1) == "Down Triangle"} {
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 - $dy * ($y_obs($i)-$ymin(W20))]
	    downTriangleDraw .diagram $x $y \
		    $w20symbolSize(1) $w20symbolColor(1) $w20symbolFillColor(1)
	}
    } elseif {$w20symbolShape(1) == "Diamond"} {
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 - $dy * ($y_obs($i)-$ymin(W20))]
	    diamondDraw .diagram $x $y \
		    $w20symbolSize(1) $w20symbolColor(1) $w20symbolFillColor(1)
	}
    } elseif {$w20symbolShape(1) == "Cross (+)"} {
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 - $dy * ($y_obs($i)-$ymin(W20))]
	    plusDraw .diagram $x $y $w20symbolSize(1) $w20symbolFillColor(1)
	}
    } elseif {$w20symbolShape(1) == "Cross (x)"} {
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 - $dy * ($y_obs($i)-$ymin(W20))]
	    crossDraw .diagram $x $y $w20symbolSize(1) $w20symbolFillColor(1)
	}
    }

# Messwerte als Linie:

    if {$w20lineDraw(1) == 1} {
	set werte {}
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 - $dy * ($y_obs($i)-$ymin(W20))]
	    lappend werte $x
	    lappend werte $y
	}
	if {$ndata > 1} {
	    eval .diagram create line $werte -width $w20lineWidth(1) \
		-fill $w20lineColor(1)
	}
    }

# Berechnete Werte als Linie:

    if {$w20lineDraw(2) == 1} {
	set werte {}
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 - $dy * ($y_calc($i)-$ymin(W20))]
	    lappend werte $x
	    lappend werte $y
	}
	if {$ndata > 1} {
	    eval .diagram create line $werte -width $w20lineWidth(2) \
		-fill $w20lineColor(2)
	}
    }

# Untergrundwerte als Linie:

    if {$w20lineDraw(4) == 1} {
	set werte {}
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 - $dy * ($y_back($i)-$ymin(W20))]
	    lappend werte $x
	    lappend werte $y
	}
	if {$ndata > 1} {
	    eval .diagram create line $werte -width $w20lineWidth(4) \
		-fill $w20lineColor(4)
	}
    }

# Null-Linie:

    if {$ymin(W20) <= 0.0} {
	set x1 $b1
	set y1 [expr $h1 + $h21]
	set x2 [expr $b1 + $b2]
	set y2 $y1
	.diagram create line $x1 $y1 $x2 $y2 -width 1
    }

# Reflexmarkierungen:

    if {[expr $xmax(W20)-$xmin(W20)] != 0.} {
	for {set i 1} {$i <= $numRef} {incr i} {
	    set q 0.25
	    set s [expr 0.8 * $h22 / double($phsZahl + ($phsZahl-1) * $q)]
	    set x1 [expr $b1 + $dx * ($twoThPos($i)-$xmin(W20))]
	    set y1 [expr $h1 + $h21 + 0.2 * $h22 + \
		($refPhase($i) - 1) * (1 + $q) * $s]
	    set x2 $x1
	    set y2 [expr $y1 + $s]
#            if {$phsZahl==1} {set tic_farbe "black"}
#            if {$phsZahl==2} {set tic_farbe "blue"}
#            if {$phsZahl==3} {set tic_farbe "brown"}
#            if {$phsZahl==4} {set tic_farbe "red"}
#            if {$phsZahl>4} {set tic_farbe "orange"}
#
#	    .diagram create line $x1 $y1 $x2 $y2 -width 1 -fill $tic_farbe 
#	    set hkl_x [expr $x2]
#	    set hkl_y [expr $y2]
#	    .diagram create text $hkl_x $hkl_y -anchor sw -text "$ref_hkl($i)" 
	}
    }

# Differenzwerte als Linie:

    if {$w20lineDraw(3) == 1} {
	set werte {}
	for {set i 1} {$i <= $ndata} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta_w20($i)-$xmin(W20))]
	    set y [expr $h1 + $h21 + $h22 + 0.5 * $h23 - $dy * $y_diff($i)]
	    lappend werte $x
	    lappend werte $y
	}
	if {$ndata > 1} {
	    eval .diagram create line $werte -width $w20lineWidth(3) \
		-fill $w20lineColor(3)
	}
	set x1 $b1
	set y1 [expr $h1 + $h21 + $h22 + 0.5 * $h23]
	set x2 [expr $b1 + $b2]
	set y2 $y1
	.diagram create line $x1 $y1 $x2 $y2 -width 1
    }

### Abdecken:

    set w [winfo width .diagram]
    set h [winfo height .diagram]
    .diagram create rectangle 0 0 $b1 $h \
	    -fill $diagramColor -outline $diagramColor
    .diagram create rectangle $b1 0 [expr $b1+$b2] $h1 \
	    -fill $diagramColor -outline $diagramColor
    .diagram create rectangle [expr $b1+$b2] 0 $w $h \
	    -fill $diagramColor -outline $diagramColor
    .diagram create rectangle $b1 [expr $h1+$h2] [expr $b1+$b2] $h \
	    -fill $diagramColor -outline $diagramColor

# Grosses Rechteck (Achsen):

    .diagram create rectangle $b1 $h1 [expr $b1+$b2] [expr $h1+$h2] \
	    -width $axesWidth -outline $axesColor

# Titel:

    set titleXpos [expr $b1 + 20]
    set titleYpos 21
    .diagram create text $titleXpos $titleYpos -anchor sw \
	    -text $plotTitle

# Beschriftung (X-Achse):

    for {set i 1} {$i <= $numXTics} {incr i} {
	set zahl [expr $beginXTics + ($i-1) * $stepXTics]
	set x [expr $b1 + $dx * ($zahl - $xmin(W20))]
        set y [expr $h1 + $h2 + double($xNumSize)/2.0]
	.diagram create text $x $y -anchor n -text "$zahl"
    }
    set x [expr double($breite) * 250/380]
    set y [expr $h1 + $h2 + 45]
    .diagram create text $x $y -anchor sw -text $xAxisLabel

# Beschriftung (Y-Achse):

    for {set i 1} {$i <= $numYTics} {incr i} {
	set x [expr $b1 - 7] 
	set zahl [expr $beginYTics + ($i-1) * $stepYTics]
	set y [expr $h1 + $h21 - $dy * ($zahl-$ymin(W20))]
	.diagram create text $x $y -anchor e -text "$zahl"
    }

}

#==============================================================================
#                 Messdaten ins Diagramm einzeichnen
#==============================================================================

proc diagram_data {breite hoehe} {
    global symbolShape symbolSize rebfakt symbolFillColor symbolColor lineWidth \
	lineDraw lineColor diagramColor fileSpec plotTitle \
	twoTheta ycalc yobs ydiff ysigma yback izahl xmin xmax ymin ymax \
	countNr unt_Theta unt_Wert unt_Zahl \
	diagramWidth diagramHeight axesWidth axesColor xAxisLabel yAxisLabel \
	backgrDraw backgrColor backgrWidth markRef markRefColor markRefWidth \
	ref_Zahl ref_Theta ref_Fwhm refPhase phsZahl \
	numTics beginTics stepTics ref_h ref_k ref_l refPhaseColor hkl_size\
	hklDraw refAlpha hkl_alpha1 hkl_alpha2 hkl_pos step bla

### Einstellungen:

    set titleSize 14
    set titleFont "-*-times-medium-i-normal--$titleSize-140-*"

    set xNumSize 12

    set xBeschSize 14
    set xBeschFont "-*-times-medium-i-normal--$xBeschSize-140-*"

# Dimensionierung:

    set h1 30
    set h3 60
    set h2 [expr $hoehe - $h1 -$h3]
    if {$fileSpec == "SIM"} {
	set h21 [expr $h2 * double(11)/15]
	set h22 [expr $h2 * double(1)/15]
	set h23 [expr $h2 * double(3)/15]
    } else {
	if {$markRef == 1} {
	    set h21 [expr $h2 * double(13)/15]
	    set h22 [expr $h2 * double(1)/15]
	    set h23 [expr $h2 * double(1)/15]
	} else {
	    set h21 $h2
	    set h22 0
	    set h33 0
        }        
    }

    set b1 80
    set b3 20
    set b2 [expr $breite - $b1 - $b3]

    set diagramWidth $b2
    set diagramHeight $h21

# Faktor fuer die Umrechnung der Werte in Pixel:

    set dx [expr $b2 / double($xmax(data) - $xmin(data))]
    set dy [expr $h21 / double($ymax(data) - $ymin(data))]

### Tics:

    set t [expr double($b2)/70]
    set ts [expr double($b2)/100]
 
# Tics und Beschriftung der X-Achse:

    axisCalc $xmin(data) $xmax(data)
    set beginXTics $beginTics
    set stepXTics $stepTics
    set numXTics $numTics

# Large tics:

    for {set i 1} {$i <= $numXTics} {incr i} {
	set zahl [expr $beginXTics + ($i-1) * $stepXTics]
	set x1 [expr $b1 + $dx * ($zahl - $xmin(data))]
	set x2 $x1
	set y1 [expr $h1 + $h2]
	set y2 [expr $h1 + $h2 - $t]
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
	set y1 $h1
	set y2 [expr $h1 + $t]
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
    }

# Small tics:

    for {set i 0} {$i <= $numXTics} {incr i} {
	set zahl [expr $beginXTics + ($i-0.5) * $stepXTics]
	set x1 [expr $b1 + $dx * ($zahl - $xmin(data))]
	if {$x1 >= $b1 && $x1 <= [expr $b1 + $b2]} { 
	    set x2 $x1
	    set y1 [expr $h1 + $h2]
	    set y2 [expr $h1 + $h2 - $ts]
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
	    set y1 $h1
	    set y2 [expr $h1 + $ts]
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
	}
    }

# Tics der Y-Achse:

    axisCalc $ymin(data) $ymax(data)
    set beginYTics $beginTics
    set stepYTics $stepTics
    set numYTics $numTics

# Large Tics:

    for {set i 1} {$i <= $numYTics} {incr i} {
	set x1 $b1
	set x2 [expr $b1 + $t]
	set zahl [expr $beginYTics + ($i-1) * $stepYTics]
	set y1 [expr $h1 + $h21 - $dy * ($zahl-$ymin(data))]
	set y2 $y1
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
	set x1 [expr $b1 + $b2]  
	set x2 [expr $x1 - $t]
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
    }

# Small Tics:

    for {set i 0} {$i <= $numYTics} {incr i} {
	set zahl [expr $beginYTics + ($i-0.5) * $stepYTics]
	set y1 [expr $h1 + $h21 - $dy * ($zahl-$ymin(data))]
	if {$y1 >= $h1 && $y1 <= [expr $h1 + $h21]} { 
	    set x1 $b1
	    set x2 [expr $x1 + $ts]
	    set y2 $y1
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
	    set x1 [expr $b1 + $b2]
	    set x2 [expr $x1 - $ts]
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
        }
    }

### Symbole fuer beobachtete Werte (Messwerte):

    if {$symbolShape(1) == "Circle"} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    circleDraw .diagram $x $y \
		    $symbolSize(1) $symbolColor(1) $symbolFillColor(1)
	}
    } elseif {$symbolShape(1) == "Square"} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    squareDraw .diagram $x $y \
		    $symbolSize(1) $symbolColor(1) $symbolFillColor(1)
	}
    } elseif {$symbolShape(1) == "Up Triangle"} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    upTriangleDraw .diagram $x $y \
		    $symbolSize(1) $symbolColor(1) $symbolFillColor(1)
	}
    } elseif {$symbolShape(1) == "Down Triangle"} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    downTriangleDraw .diagram $x $y \
		    $symbolSize(1) $symbolColor(1) $symbolFillColor(1)
	}
    } elseif {$symbolShape(1) == "Diamond"} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    diamondDraw .diagram $x $y \
		    $symbolSize(1) $symbolColor(1) $symbolFillColor(1)
	}
    } elseif {$symbolShape(1) == "Cross (+)"} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    plusDraw .diagram $x $y $symbolSize(1) $symbolFillColor(1)
	}
    } elseif {$symbolShape(1) == "Cross (x)"} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    crossDraw .diagram $x $y $symbolSize(1) $symbolFillColor(1)
	}
    } elseif {$symbolShape(1) == "Rebmannflche"} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    set rebabwx [expr $dx * $rebfakt(1)]
	    set rebabwy [expr $dy * sqrt($yobs($countNr,$i))]
	    rectDraw .diagram $x $y $rebabwx $rebabwy \
		    $symbolSize(1) $symbolFillColor(1) $symbolFillColor(1)
	}
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    circleDraw .diagram $x $y \
		    1 $symbolColor(1) $symbolColor(1)
	}

    }
# Messwerte als Linie:

    if {$lineDraw(1) == 1} {
	set werte {}
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    lappend werte $x
	    lappend werte $y
	}
	if {$izahl > 1} {
	    eval .diagram create line $werte -width $lineWidth(1) \
		-fill $lineColor(1)
	}
    }

# Berechnete Werte als Linie:

    if {$fileSpec == "SIM" && $lineDraw(2) == 1} {
	set werte {}
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($ycalc($i)-$ymin(data))]
	    lappend werte $x
	    lappend werte $y
	}
	if {$izahl > 1} {
	    eval .diagram create line $werte -width $lineWidth(2) \
		-fill $lineColor(2)
	}
    }

# Untergrundwerte als Linie:

    if {$fileSpec == "DAT" || $fileSpec == "SIM"} {
	if {$lineDraw(4) == 1} {
	    set werte {}
	    for {set i 1} {$i <= $izahl} {incr i} {
		set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
		set y [expr $h1 + $h21 - $dy * ($yback($i)-$ymin(data))]
		lappend werte $x
		lappend werte $y
	    }
	    if {$izahl > 1} {
		eval .diagram create line $werte -width $lineWidth(4) \
		    -fill $lineColor(4)
	    }
	}
    }

# Fehlerbalken der y_sigma-Werte:

    if {$fileSpec == "DAT" && $lineDraw(5)} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($yobs($countNr,$i)-$ymin(data))]
	    set stdd [expr $dy * ($ysigma($i)-$ymin(data))]
	    set y1 [expr $y + $stdd]
	    set y2 [expr $y - $stdd]
	    set xl [expr $x - 3]
	    set xr [expr $x + 3]
	    .diagram create line $x $y1 $x $y2 -width $lineWidth(5)
	    .diagram create line $xl $y1 $xr $y1 -width $lineWidth(5)
	    .diagram create line $xl $y2 $xr $y2 -width $lineWidth(5)
	}
    }

# Untergrund-Daten aus einer Daten zeichnen:

    if {$fileSpec != "SIM" && $backgrDraw == 1} {
	set werte {}
	for {set i 1} {$i <= $unt_Zahl} {incr i} {
	    set x [expr $b1 + $dx * ($unt_Theta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 - $dy * ($unt_Wert($i)-$ymin(data))]
	    lappend werte $x
	    lappend werte $y
	}
	if {$unt_Zahl > 1} {
	    eval .diagram create line $werte -width $backgrWidth \
		-fill $backgrColor
	}
    }

# Null-Linie:

    if {$ymin(data) <= 0.0} {
	set x1 $b1
	set y1 [expr $h1 + $h21]
	set x2 [expr $b1 + $b2]
	set y2 $y1
	.diagram create line $x1 $y1 $x2 $y2 -width 1
    }

# Reflexmarkierungen:

    if {$fileSpec == "SIM" || $markRef == 1} {
	if {[expr $xmax(data)-$xmin(data)] != 0.} {
	    for {set i 1} {$i <= $ref_Zahl} {incr i} {
		set q 0.25
		set s [expr 0.8 * $h22 / double($phsZahl + ($phsZahl-1) * $q)]
		set x1 [expr $b1 + $dx * ($ref_Theta($i)-$xmin(data))]
		set y1 [expr $h1 + $h21 + 0.2 * $h22 + \
			($refPhase($i) - 1) * (1 + $q) * $s]
		set x2 $x1
		set y2 [expr $y1 + $s]
#		eval .diagram create line $x1 $y1 $x2 $y2 \
#		    -width $markRefWidth -fill $markRefColor
		if {$refPhaseColor($refPhase($i))!="Off"} {
		    eval .diagram create line $x1 $y1 $x2 $y2 -width 1 \
			    -fill $refPhaseColor($refPhase($i))
		}
#                .diagram create text $hkl_x $hkl_y -anchor sw -text "$ref_hkl($i)"
	        if {$hklDraw == 1 && $refPhaseColor($refPhase($i))!="Off"} {
		    set hkl_x [expr $x1]
		    if {$hkl_pos == "below mark" } { set hkl_y [expr $y2+$hkl_size+$hkl_size+5]}
		    if {$hkl_pos == "above peak"} {
			set bin [expr round(($ref_Theta($i)-$twoTheta(1))/$step)]
			set intensitaet $ycalc($bin)
			if {$yobs($countNr,$bin) > $intensitaet} {
			    set intensitaet $yobs($countNr,$bin)
			}
			set hkl_y [expr $h1 + $h21 - $dy * ($intensitaet-$ymin(data)) \
				-3  + ((3 * $hkl_size)+2)*($refPhase($i)-$phsZahl)]
		    }
		    if {$hkl_alpha1==1 && $refAlpha($i)==1 } {
			set h_y [expr $hkl_y-$hkl_size-$hkl_size]
			set k_y [expr $hkl_y-$hkl_size]
			set l_y [expr $hkl_y]
			.diagram create text $hkl_x $h_y -anchor sw -text "$ref_h($i)" \
				-fill $refPhaseColor($refPhase($i)) -font "hkl_font"
			.diagram create text $hkl_x $k_y -anchor sw -text "$ref_k($i)" \
				-fill $refPhaseColor($refPhase($i)) -font "hkl_font"
			.diagram create text $hkl_x $l_y -anchor sw -text "$ref_l($i)" \
				-fill $refPhaseColor($refPhase($i)) -font "hkl_font"
		    }
		    if {$hkl_alpha2==1 && $refAlpha($i)==2 } {
			set h_y [expr $y1]
			set k_y [expr $y1+$hkl_size]
			set l_y [expr $y1+$hkl_size+$hkl_size]
			.diagram create text $hkl_x $h_y -anchor sw -text "$ref_h($i)" \
				-fill $refPhaseColor($refPhase($i)) -font "hkl_font"
			.diagram create text $hkl_x $k_y -anchor sw -text "$ref_k($i)" \
				-fill $refPhaseColor($refPhase($i)) -font "hkl_font"
			.diagram create text $hkl_x $l_y -anchor sw -text "$ref_l($i)" \
				-fill $refPhaseColor($refPhase($i)) -font "hkl_font"
		    }
		}
            }
	}
    }

# Differenzwerte als Linie:

    if {$fileSpec == "SIM" && $lineDraw(3) == 1} {
	set werte {}
	for {set i 1} {$i <= $izahl} {incr i} {
	    set x [expr $b1 + $dx * ($twoTheta($i)-$xmin(data))]
	    set y [expr $h1 + $h21 + $h22 + 0.5 * $h23 - $dy * $ydiff($i)]
	    lappend werte $x
	    lappend werte $y
	}
	if {$izahl > 1} {
	    eval .diagram create line $werte -width $lineWidth(3) \
		-fill $lineColor(3)
	}
	set x1 $b1
	set y1 [expr $h1 + $h21 + $h22 + 0.5 * $h23]
	set x2 [expr $b1 + $b2]
	set y2 $y1
	.diagram create line $x1 $y1 $x2 $y2 -width 1
    }

### Abdecken:

    set w [winfo width .diagram]
    set h [winfo height .diagram]
    .diagram create rectangle 0 0 $b1 $h \
	    -fill $diagramColor -outline $diagramColor
    .diagram create rectangle $b1 0 [expr $b1+$b2] $h1 \
	    -fill $diagramColor -outline $diagramColor
    .diagram create rectangle [expr $b1+$b2] 0 $w $h \
	    -fill $diagramColor -outline $diagramColor
    .diagram create rectangle $b1 [expr $h1+$h2] [expr $b1+$b2] $h \
	    -fill $diagramColor -outline $diagramColor

# Grosses Rechteck (Achsen):

    .diagram create rectangle $b1 $h1 [expr $b1+$b2] [expr $h1+$h2] \
	    -width $axesWidth -outline $axesColor

# Titel:

    set titleXpos [expr $b1 + 20]
    set titleYpos 21
    .diagram create text $titleXpos $titleYpos -anchor sw \
	    -text $plotTitle

# Beschriftung (X-Achse):

    for {set i 1} {$i <= $numXTics} {incr i} {
	set zahl [expr $beginXTics + ($i-1) * $stepXTics]
	set x [expr $b1 + $dx * ($zahl-$xmin(data))]
        set y [expr $h1 + $h2 + double($xNumSize)/2.0]
	.diagram create text $x $y -anchor n -text "$zahl"
    }
    set x [expr double($breite) * 250/380]
    set y [expr $h1 + $h2 + 45]
    .diagram create text $x $y -anchor sw -text $xAxisLabel

# Beschriftung (Y-Achse):

    for {set i 1} {$i <= $numYTics} {incr i} {
	set x [expr $b1 - 7] 
	set zahl [expr $beginYTics + ($i-1) * $stepYTics]
	set y [expr $h1 + $h21 - $dy * ($zahl-$ymin(data))]
	.diagram create text $x $y -anchor e -text "$zahl"
    }

}

#==============================================================================
#                          Symbole zeichnen
#==============================================================================

proc circleDraw {w x y size color fcolor} {
    set s [expr $size / 2.0]
    set x1 [expr $x - $s]
    set y1 [expr $y - $s]
    set x2 [expr $x + $s]
    set y2 [expr $y + $s]
    $w create oval $x1 $y1 $x2 $y2 -fill $fcolor -outline $color 
}

proc squareDraw {w x y size color fcolor} {
    set s [expr $size / 2.0]
    set x1 [expr $x - $s]
    set y1 [expr $y - $s]
    set x2 [expr $x + $s]
    set y2 [expr $y + $s]
    $w create rectangle $x1 $y1 $x2 $y2 -fill $fcolor -outline $color
} 
proc rectDraw {w x y rebx reby size color fcolor} {
    set s [expr $size / 2.0]
    set x1 [expr $x - ($rebx)]
    set y1 [expr $y - ($reby / 2.0)]
    set x2 [expr $x + ($rebx)]
    set y2 [expr $y + ($reby / 2.0)]
    $w create rectangle $x1 $y1 $x2 $y2 -fill $fcolor -outline $fcolor
} 

proc crossDraw {w x y size fcolor} {
    set s [expr $size / sqrt(2.0) / 2.0]
    set x1 [expr $x - $s]
    set y1 [expr $y - $s]
    set x2 [expr $x + $s]
    set y2 [expr $y + $s]
    $w create line $x1 $y1 $x2 $y2 -width 1 -fill $fcolor
    set x1 [expr $x - $s]
    set y1 [expr $y + $s]
    set x2 [expr $x + $s]
    set y2 [expr $y - $s]
    $w create line $x1 $y1 $x2 $y2 -width 1 -fill $fcolor
}

proc diamondDraw {w x y size color fcolor} {
    set s [expr $size / 2.0]
    set sq3 [expr $s * sqrt(3.0)]
    set x1 [expr $x - $s]
    set y1 $y
    set x2 $x
    set y2 [expr $y - $sq3]
    set x3 [expr $x + $s]
    set y3 $y
    set x4 $x
    set y4 [expr $y + $sq3]
    $w create polygon $x1 $y1 $x2 $y2 $x3 $y3 $x4 $y4 $x1 $y1 \
	    -fill $fcolor -outline $color
}

proc upTriangleDraw {w x y size color fcolor} {
    set s [expr $size / 2.0]
    set db [expr sqrt(3.0) / 3.0 * $s]
    set da [expr 2.0 * $db] 
    set x1 [expr $x - $s]
    set y1 [expr $y + $db]
    set x2 $x
    set y2 [expr $y - $da]
    set x3 [expr $x + $s]
    set y3 $y1
    $w create polygon $x1 $y1 $x2 $y2 $x3 $y3 $x1 $y1 \
	    -fill $fcolor -outline $color
}

proc downTriangleDraw {w x y size color fcolor} {
    set s [expr $size / 2.0]
    set db [expr sqrt(3.0) / 3.0 * $s]
    set da [expr 2.0 * $db]
    set x1 [expr $x - $s]
    set y1 [expr $y - $db]
    set x2 $x
    set y2 [expr $y + $da]
    set x3 [expr $x + $s]
    set y3 $y1
    $w create polygon $x1 $y1 $x2 $y2 $x3 $y3 $x1 $y1 \
	    -fill $fcolor -outline $color

}

proc plusDraw {w  x y size fcolor} {
    set s [expr $size / 2.0]
    set x1 [expr $x - $s]
    set y1 $y
    set x2 [expr $x + $s]
    set y2 $y
    $w create line $x1 $y1 $x2 $y2 -width 1 -fill $fcolor
    set x1 $x
    set y1 [expr $y - $s]
    set x2 $x
    set y2 [expr $y + $s]
    $w create line $x1 $y1 $x2 $y2 -width 1 -fill $fcolor
}

#==============================================================================
#                 Datei im Spaltenformat zeichnen
#==============================================================================

proc diagram_col {breite hoehe} {
    global symbolShape symbolSize symbolFillColor \
	symbolColor lineWidth lineDraw lineColor diagramColor \
	plotTitle daten izahl absNr xmin xmax ymin ymax colZahl \
	diagramWidth diagramHeight axesWidth axesColor errorBarWidth \
	xErrorBars yErrorBars xAxisLabel yAxisLabel \
	beginTics stepTics numTics

### Einstellungen:

    set titleSize 14
    set titleFont "-*-times-medium-i-normal--$titleSize-140-*"

    set xNumSize 12

    set xBeschSize 14
    set xBeschFont "-*-times-medium-i-normal--$xBeschSize-140-*"

# Dimensionierung:

    set h1 30
    set h3 60
    set h2 [expr $hoehe - $h1 -$h3]
    set b1 80
    set b3 20
    set b2 [expr $breite - $b1 - $b3]

    set diagramWidth $b2
    set diagramHeight $h2

# Faktor fuer die Umrechnung der Werte in Pixel:

    set dx [expr $b2 / double($xmax(data) - $xmin(data))]
    set dy [expr $h2 / double($ymax(data) - $ymin(data))]

### Tics:

    set t [expr double($b2)/70]
    set ts [expr double($b2)/100]
 
# Tics und Beschriftung der X-Achse:

    axisCalc $xmin(data) $xmax(data)
    set beginXTics $beginTics
    set stepXTics $stepTics
    set numXTics $numTics

# Large Tics:

    for {set i 1} {$i <= $numXTics} {incr i} {
	set zahl [expr $beginXTics + ($i-1) * $stepXTics]
	set x1 [expr $b1 + $dx * ($zahl - $xmin(data))]
	set x2 $x1
	set y1 [expr $h1 + $h2]
	set y2 [expr $h1 + $h2 - $t]
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
	set y1 $h1
	set y2 [expr $h1 + $t]
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
    }

# Small tics:

    for {set i 0} {$i <= $numXTics} {incr i} {
	set zahl [expr $beginXTics + ($i-0.5) * $stepXTics]
	set x1 [expr $b1 + $dx * ($zahl - $xmin(data))]
	if {$x1 >= $b1 && $x1 <= [expr $b1 + $b2]} { 
	    set x2 $x1
	    set y1 [expr $h1 + $h2]
	    set y2 [expr $h1 + $h2 - $ts]
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
	    set y1 $h1
	    set y2 [expr $h1 + $ts]
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
	}
    }

# Tics der Y-Achse:

    axisCalc $ymin(data) $ymax(data)
    set beginYTics $beginTics
    set stepYTics $stepTics
    set numYTics $numTics

# Large Tics:

    for {set i 1} {$i <= $numYTics} {incr i} {
	set x1 $b1
	set x2 [expr $b1 + $t]
	set zahl [expr $beginYTics + ($i-1) * $stepYTics]
	set y1 [expr $h1 + $h2 - $dy * ($zahl - $ymin(data))]
	set y2 $y1
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
	set x1 [expr $b1 + $b2]  
	set x2 [expr $x1 - $t]
	.diagram create line $x1 $y1 $x2 $y2 -width $axesWidth -fill $axesColor
    }

# Small Tics:

    for {set i 0} {$i <= $numYTics} {incr i} {
	set zahl [expr $beginYTics + ($i-0.5) * $stepYTics]
	set y1 [expr $h1 + $h2 - $dy * ($zahl - $ymin(data))]
	if {$y1 >= $h1 && $y1 <= [expr $h1 + $h2]} { 
	    set x1 $b1
	    set x2 [expr $x1 + $ts]
	    set y2 $y1
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
	    set x1 [expr $b1 + $b2]
	    set x2 [expr $x1 - $ts]
	    .diagram create line $x1 $y1 $x2 $y2 -width $axesWidth \
		-fill $axesColor
        }
    }

### Messwerte zeichnen:

    for {set n 1} {$n <= $colZahl} {incr n} {

### Symbole fuer beobachtete Werte (Messwerte):

	if {$symbolShape($n) == "Circle"} {
	    for {set i 1} {$i <= $izahl} {incr i} {
		set x [expr $b1 + $dx * ($daten($absNr,$i)-$xmin(data))]
		set y [expr $h1 + $h2 - $dy * ($daten($n,$i)-$ymin(data))]
		circleDraw .diagram $x $y \
		    $symbolSize($n) $symbolColor($n) $symbolFillColor($n)
	    }
	} elseif {$symbolShape($n) == "Square"} {
	    for {set i 1} {$i <= $izahl} {incr i} {
		set x [expr $b1 + $dx * ($daten($absNr,$i)-$xmin(data))]
		set y [expr $h1 + $h2 - $dy * ($daten($n,$i)-$ymin(data))]
		squareDraw .diagram $x $y \
		    $symbolSize($n) $symbolColor($n) $symbolFillColor($n)
	    }
	} elseif {$symbolShape($n) == "Up Triangle"} {
	    for {set i 1} {$i <= $izahl} {incr i} {
		set x [expr $b1 + $dx * ($daten($absNr,$i)-$xmin(data))]
		set y [expr $h1 + $h2 - $dy * ($daten($n,$i)-$ymin(data))]
		upTriangleDraw .diagram $x $y \
		    $symbolSize($n) $symbolColor($n) $symbolFillColor($n)
	    }
	} elseif {$symbolShape($n) == "Down Triangle"} {
	    for {set i 1} {$i <= $izahl} {incr i} {
		set x [expr $b1 + $dx * ($daten($absNr,$i)-$xmin(data))]
		set y [expr $h1 + $h2 - $dy * ($daten($n,$i)-$ymin(data))]
		downTriangleDraw .diagram $x $y \
		    $symbolSize($n) $symbolColor($n) $symbolFillColor($n)
	    }
	} elseif {$symbolShape($n) == "Diamond"} {
	    for {set i 1} {$i <= $izahl} {incr i} {
		set x [expr $b1 + $dx * ($daten($absNr,$i)-$xmin(data))]
		set y [expr $h1 + $h2 - $dy * ($daten($n,$i)-$ymin(data))]
		diamondDraw .diagram $x $y \
		    $symbolSize($n) $symbolColor($n) $symbolFillColor($n)
	    }
	} elseif {$symbolShape($n) == "Cross (+)"} {
	    for {set i 1} {$i <= $izahl} {incr i} {
		set x [expr $b1 + $dx * ($daten($absNr,$i)-$xmin(data))]
		set y [expr $h1 + $h2 - $dy * ($daten($n,$i)-$ymin(data))]
		plusDraw .diagram $x $y $symbolSize($n) $symbolFillColor($n)
	    }
	} elseif {$symbolShape($n) == "Cross (x)"} {
	    for {set i 1} {$i <= $izahl} {incr i} {
		set x [expr $b1 + $dx * ($daten($absNr,$i)-$xmin(data))]
		set y [expr $h1 + $h2 - $dy * ($daten($n,$i)-$ymin(data))]
		crossDraw .diagram $x $y $symbolSize($n) $symbolFillColor($n)
	    }
	}

# Y-Werte mit einer Linie verbinden:

	if {$lineDraw($n) == 1} {
	    set werte {}
	    for {set i 1} {$i <= $izahl} {incr i} {
		set x [expr $b1 + $dx * ($daten($absNr,$i)-$xmin(data))]
		set y [expr $h1 + $h2 - $dy * ($daten($n,$i)-$ymin(data))]
		lappend werte $x
		lappend werte $y
	    }
	    if {$izahl > 1} {
		eval .diagram create line $werte -width $lineWidth($n) \
		    -fill $lineColor($n)
	    }
	}

# Fehlerbalken der Y-Werte zeichnen:

	if {$yErrorBars == 1} {
	    if {$symbolShape($n) != "No Symbol" || $lineDraw($n) == 1} {
		set n1 [expr $n + 1]
		for {set i 1} {$i <= $izahl} {incr i} {
		    set x [expr $b1 + $dx * ($daten($absNr,$i)-$xmin(data))]
		    set y [expr $h1 + $h2 - $dy * ($daten($n,$i)-$ymin(data))]
		    set stdd [expr $dy * $daten($n1,$i)]
		    set y1 [expr $y + $stdd]
		    set y2 [expr $y - $stdd]
		    set xl [expr $x - 3]
		    set xr [expr $x + 3]
		    .diagram create line $x $y1 $x $y2 -width $errorBarWidth
		    .diagram create line $xl $y1 $xr $y1 -width $errorBarWidth
		    .diagram create line $xl $y2 $xr $y2 -width $errorBarWidth
		}
	    }
	}

    }

# Null-Linie:

    if {$ymin(data) <= 0.0} {
	set x1 $b1
	set y1 [expr $h1 + $h2]
	set x2 [expr $b1 + $b2]
	set y2 $y1
	.diagram create line $x1 $y1 $x2 $y2 -width 1
    }

### Abdecken:

    set w [winfo width .diagram]
    set h [winfo height .diagram]
    .diagram create rectangle 0 0 $b1 $h \
	    -fill $diagramColor -outline $diagramColor
    .diagram create rectangle $b1 0 [expr $b1+$b2] $h1 \
	    -fill $diagramColor -outline $diagramColor
    .diagram create rectangle [expr $b1+$b2] 0 $w $h \
	    -fill $diagramColor -outline $diagramColor
    .diagram create rectangle $b1 [expr $h1+$h2] [expr $b1+$b2] $h \
	    -fill $diagramColor -outline $diagramColor

# Grosses Rechteck (Achsen):

    .diagram create rectangle $b1 $h1 [expr $b1+$b2] [expr $h1+$h2] \
	    -width $axesWidth -outline $axesColor

# Titel:

    set titleXpos [expr $b1 + 20]
    set titleYpos 21
    .diagram create text $titleXpos $titleYpos -anchor sw \
	    -text $plotTitle

# Beschriftung (X-Achse):

    for {set i 1} {$i <= $numXTics} {incr i} {
	set zahl [expr $beginXTics + ($i-1) * $stepXTics]
	set x [expr $b1 + $dx * ($zahl-$xmin(data))]
        set y [expr $h1 + $h2 + double($xNumSize)/2.0]
	.diagram create text $x $y -anchor n -text "$zahl"
    }
    set x [expr double($breite) * 250/380]
    set y [expr $h1 + $h2 + 45]
    .diagram create text $x $y -anchor sw -text $xAxisLabel

# Beschriftung (Y-Achse):

    for {set i 1} {$i <= $numYTics} {incr i} {
	set x [expr $b1 - 7] 
	set zahl [expr $beginYTics + ($i-1) * $stepYTics]
	set y [expr $h1 + $h2 - $dy * ($zahl-$ymin(data))]
	.diagram create text $x $y -anchor e -text "$zahl"
    }

}

#==============================================================================
#                Formulare mit Schiebebalken (Abschnitt 4.2, S. 130)
#==============================================================================

proc scrollform_create {win} {
    frame $win -class Scrollform

    scrollbar $win.sbar -command "$win.vport yview"
    pack $win.sbar -side right -fill y

    canvas $win.vport -yscrollcommand "$win.sbar set"
    pack $win.vport -side left -fill both -expand true

    frame $win.vport.form
    $win.vport create window 0 0 -anchor nw -window $win.vport.form

    bind $win.vport.form <Configure> "scrollform_resize $win"
    return $win
}

proc scrollform_resize {win} {
    set bbox [$win.vport bbox all]
    set wid [winfo width $win.vport.form]
    $win.vport configure -width $wid \
	-scrollregion $bbox -yscrollincrement 0.1i
}

proc scrollform_interior {win} {
    return "$win.vport.form"
}

#==============================================================================
#        Titel und Kennung einlesen und Anzahl der Bloecke bestimmen
#==============================================================================

proc scanDataFile {file} {
    global fileSpec old blockZahl titel titelZahl countZahl countDist \
	colName colZahl xAchse temp_min temp_max blo_min blo_max phoZahl \
	x_anfang x_step x_ende y_anfang y_step y_ende spur_pho temp_pho

# Datei oeffnen:

    if {[catch {set fid [open $file r]} err] != 0} {
	tk_messageBox -icon error -type ok -title "Error" \
	    -message "Cannot open file:\n$file"
	return -1
    }

    gaugeWidget "Scanning Data File"

    set old 0
    set titelZahl 0
    set countDist 0.

    set size 0
    set maxSize [file size $file]

# Titel bis zur Kennung bzw. Titelende einlesen:

    while {[gets $fid line] >=0} {
#	set size [expr $size + [string length $line] + 1]
#	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	set title_end [lindex $line 0]
	if {$title_end == "DAT" && \
		[scan $line "%*s %f %f %f %f %f %d %d" \
		zeit bla step ble temp bnr izahl] == 7} {
	    set filspe $title_end
	    set countZahl 1
	    break
	} elseif {$title_end == "NEU" && \
		[scan $line "%*s %f %f %f %f %f %d %d" \
		zeit bla step ble temp bnr izahl] == 7} {
	    set filspe $title_end
	    set countZahl 1
	    break
	} elseif {$title_end == "DUO" && \
		[scan $line "%*s %f %f %f %f %f %d %d" \
		zeit bla step ble temp bnr izahl] == 7} {
	    set filspe $title_end
	    set countZahl 1
	    break
	} elseif {$title_end == "TOF" && \
		[scan $line "%*s %f %f %f %f %f %d %d" \
		zeit bla step ble temp bnr izahl] == 7} {
	    set filspe $title_end
	    set countZahl 1
	    break
	} elseif {$title_end == "SIM" && \
		[scan $line "%*s %f %f %f %f %d" \
		bla step ble temp izahl] == 5} {
	    set filspe $title_end
	    set countZahl 1
	    break
	} elseif {$title_end == "B" && \
		[scan $line "%*s %f %f %f %f %d" \
		bla step ble temp izahl] == 5} {
	    set filspe "SCN"
	    set countZahl 3
	    set countDist 20.
	    break
	} elseif {$title_end == "TITELENDE" || \
		$title_end == "titelende"} {
	    set filspe [lindex $line 1]
	    set old 1
	    if {$filspe == "HAS"} {
		set countZahl 3
	    } elseif {$filspe == "BEN"} {
		set countZahl 3
		set countDist 21.8
	    } else {
		set countZahl 1
	    }
	    break
	} else {
	    set titelZahl [expr $titelZahl + 1]
	    set titel($titelZahl) $line
	}
    }

# Falls es sich um kein bekanntes Datenformat handelt,
# Fehlermeldung ausgeben:

    if {[info exist filspe] == 0} {
	tk_messageBox -icon error -type ok -title "Error" \
		-message "Unknown data file format."
	set status -1

    } else {

	set fileSpec $filspe
	set status 0

# Beschriftung der Spalten einlesen:

	if {$fileSpec == "FOR"} {
	    gets $fid line
#	    set size [expr $size + [string length $line] + 1]
#	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    set colZahl [llength $line]
	    for {set i 1} {$i <= $colZahl} {incr i} {
		set n [expr $i - 1]
		set colName($i) [lindex $line $n]
	    }
	    set xAchse $colName(1)
	    
# Spur-Temperatur-Zuordnung der Photometerdaten einlesen:
	
	} elseif {$fileSpec == "PHO"} {
	    set phoZahl 0
	    while {[gets $fid line] >= 0} {
		if {[llength $line] == 2} {
		    incr phoZahl 
		    scan $line "%f %f" spur_pho($phoZahl) temp_pho($phoZahl)
		} else {
		    break
		}
	    }
	    scan $line "%f %f %f %f %f %f " \
		    x_anfang x_step x_ende y_anfang y_step y_ende
	
	}
	
# Bloecke zaehlen:

	if {$old == 1} {
	    if {$fileSpec=="HAS" || $fileSpec=="BEN" || $fileSpec=="DUO"} {
		set blockZahl 0
		while {[gets $fid line] >=0} {
		    set size [expr $size + [string length $line] + 1]
		    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		    set bloMark [lindex $line 0]
		    if {$bloMark == "b" || $bloMark == "B"} {
			set blockZahl [expr $blockZahl + 1]
			gets $fid line
			set size [expr $size + [string length $line] + 1]
			gauge_value .gauge.g [expr round($size*100/$maxSize)]
			if {$fileSpec == "DUO"} {
			    scan $line "%f %f %f %f %f %d %d" \
				    zeit bla step ble temp bnr izahl
			} elseif {$fileSpec == "HAS"} {
			    scan $line "%f %f %f %f %d" \
				    temp bla step ble bnr
			} elseif {$fileSpec == "BEN"} {
			    scan $line "%f %f %f %f %f %f" \
				    zeit bla step ble temp t
			}
			if {$blockZahl == 1} {
			    set blo_min $bla
			    set blo_max $blo_min
			    set temp_min $temp
			    set temp_max $temp_min
			} else {
			    if {$bla < $blo_min} {set blo_min $bla}
			    if {$ble > $blo_max} {set blo_max $ble}
			    if {$temp < $temp_min} {set temp_min $temp}
			    if {$temp > $temp_max} {set temp_max $temp}
			}
		    }
		}
	    } elseif {$fileSpec == "PHO"} {
		set blockZahl [expr ($y_ende-$y_anfang)/$y_step + 1]
	    } else {
		set blockZahl 1
	    }
	} else {
	    if {$fileSpec=="SCN"} {
		set blockZahl 1
		while {[gets $fid line] >=0} {
#		    set size [expr $size + [string length $line] + 1]
#		    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		    set bloMark [lindex $line 0]
		    if {$bloMark == "b" || $bloMark == "B"} {
			set blockZahl [expr $blockZahl + 1]
		    }
		}
	    } elseif {$fileSpec=="SIM"} {
		set blockZahl 1
	    } else {
                set blockZahl 1
		while {[gets $fid line] >=0} {
#		    set size [expr $size + [string length $line] + 1]
#		    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		    if {[lindex $line 0] == $fileSpec} {
			set blockZahl [expr $blockZahl + 1]
		    }
		}
	    }
	}

    }

    close $fid
    destroy .gauge
    return $status

}

#==============================================================================
#                          Datenfile lesen 
#==============================================================================

proc readDataFile {file} {
    global old fileSpec titelZahl blockNr countDist countNr \
	   zeit bla step ble temp bnr izahl daten colZahl mitMonitor \
	   twoTheta ycalc yobs ydiff ysigma yback \
	   x_anfang x_step x_ende y_anfang y_step y_ende phoZahl\
	   xmax xmin ymax ymin ymax_all ymin_all
    
    set zeit 0.
    set bla 0.
    set step 0.
    set ble 0.
    set temp 0.
    set bnr 0
    set izahl 0
    set mitMonitor 0.

    set cmd {

# Datei oeffnen:

	set size 0
	set fid [open $file r]

# Titel ueberlesen:

	for {set i 1} {$i <= $titelZahl} {incr i} {
	    gets $fid line
#	    set size [expr $size + [string length $line] + 1]
#	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	}

# Block suchen und Kopfzeile einlesen, bei Photometerdaten Kopf ueberlesen:

	if {$old == 1} {
	    if {$fileSpec == "PHO"} {
		set bla $x_anfang
                set step $x_step
                set ble $x_ende
                set izahl [expr round(($ble-$bla)/double($step)+1)]
		for {set i 1} {$i <= [expr $phoZahl+2]} {incr i} {
		    gets $fid line
#		    set size [expr $size + [string length $line] + 1]
#		    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		}
		set zeilen [expr int($izahl/16)]
		set rest [expr $izahl - $zeilen * 16]
		if {$rest > 0} {incr zeilen}
		set zeilen [expr $zeilen * ($blockNr-1)]
		if {$zeilen > 0} {
		    for {set i 1} {$i <= $zeilen} {incr i} {
			gets $fid line
		    }
		}
	    } else {
		gets $fid line
#		set size [expr $size + [string length $line] + 1]
#		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {$fileSpec == "HAS" || $fileSpec == "BEN" || \
		    $fileSpec == "DUO"} {
		    set i 0
		    while {$i < $blockNr} {
			gets $fid line
#			set size [expr $size + [string length $line] + 1]
#			gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
			set block [lindex $line 0] 
			if { $block == "b" || $block == "B"} {incr i}
		    }
		}
		gets $fid line
#		set size [expr $size + [string length $line] + 1]
#		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
            }
	} else {
	    if {$fileSpec == "SCN"} {
		set i 0
		while {$i < $blockNr} {
		    gets $fid line
#		    set size [expr $size + [string length $line] + 1]
#		    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		    set block [lindex $line 0] 
		    if { $block == "b" || $block == "B"} {incr i}
		}
	    } elseif {$fileSpec == "DAT"} {
		set i 0
		while {$i < $blockNr} {
		    gets $fid line
#		    set size [expr $size + [string length $line] + 1]
#		    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		    if {[lindex $line 0] == "DAT"} {incr i}
		}
	    } else {
		gets $fid line
#		set size [expr $size + [string length $line] + 1]
#		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    }
	}

# Blockparameter bestimmen:

	if {$old == 1} {
	    switch $fileSpec {
		"DUO" {scan $line "%f %f %f %f %f %d %d" \
			zeit bla step ble temp bnr izahl}
		"HAS" {scan $line "%f %f %f %f %d" temp bla step ble bnr}
		"BEN" {scan $line "%f %f %f %f %f %f" \
			zeit bla step ble temp t}
		"GRE" {scan $line "%f %f %f %f %f %d %d" \
			zeit bla step ble temp idum izahl}
		"DMC" {scan $line "%f %f %f" bla step ble}
		"PHO" {}
		"FOR" {}
		"ERG" {}
	    }
	} else {
	    switch $fileSpec {
		"DAT" {scan $line "%*s %f %f %f %f %f %d %d" \
			zeit bla step ble temp bnr izahl} 
		"NEU" {scan $line "%*s %f %f %f %f %f %d %d" \
			zeit bla step ble temp bnr izahl}
		"DUO" {scan $line "%*s %f %f %f %f %f %d %d" \
			zeit bla step ble temp bnr izahl}
		"SIM" {scan $line "%*s %f %f %f %f %d" \
			bla step ble temp izahl}
		"SCN" {scan $line "%*s %f %f %f %f %d" \
			bla step ble temp izahl}
		"TOF" {scan $line "%*s %f %f %f %f %f %d %d" \
			zeit bla step ble temp bnr izahl} 
	    }	    
	}

# Daten einlesen:

# SIMREF-Datenformat:


	if {$fileSpec == "DAT"} {
	    set i 0
	    set ymax_all 0
	    set ymin_all 0
	    while {[gets $fid line] >=0} {
#		set size [expr $size + [string length $line] + 1]
#		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {[scan $line "%f %f %f %f" the obs sig bac] == 4} {
		    set i [expr $i + 1]
		    set twoTheta($i) $the
		    set yobs(1,$i) $obs
		    set ysigma($i) $sig
		    set yback($i) $bac
		    if {$obs > $ymax_all} {
			set ymax_all $obs
		    }
		    if {$obs < $ymin_all} {
			set ymin_all $obs
		    }
		    set max_bekannt 1
		} else {
		    break
		}
	    }
	    set izahl $i
	    set bla $twoTheta(1)
	    set ble $twoTheta($izahl)

# Time-of-Flight-Daten:

	} elseif {$fileSpec == "TOF"} {
	    set i 0
	    while {[gets $fid line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {[scan $line "%f %f %f" the obs sig] == 3} {
		    set i [expr $i + 1]
		    set twoTheta($i) $the
		    set yobs(1,$i) $obs
		    set ysigma($i) $sig
		} else {
		    break
		}
	    }
	    set izahl $i
	    set bla $twoTheta(1)
	    set ble $twoTheta($izahl)

# Neutronendaten vom ILL (Grenoble), HMI (Berlin), PSI (Villigen):

	} elseif {$fileSpec=="NEU" || $fileSpec=="GRE" || $fileSpec=="DMC"} {
	    if {$fileSpec == "GRE"} {
		set izahl [expr round(($ble-$bla)/$step) + 1]
	    }
	    set i 0
	    while {[gets $fid line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		set diff [expr $izahl - $i]
		if {$diff < 10} {
		    set num $diff
		} else {
		    set num 10
                }
		set anfang 2
		set ende 7
		for {set n 1} {$n <= $num} {incr n} {
		    incr i
		    set twoTheta($i) [expr $bla + ($i-1) * $step] 
		    set yobs(1,$i) \
			[expr double([string range $line $anfang $ende])]
		    incr anfang 8
		    incr ende 8
		}
		if {$diff < 10} {break}
	    }

# Tuebinger Roentgen-Daten im DUO-Format:

	} elseif {$fileSpec == "DUO"} {
	    set i 0 
	    while {[gets $fid line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		set diff [expr $izahl - $i]
		if {$diff < 8} {
		    set num $diff
		} else {
		    set num 8
                }
		set anfang 0
		set ende 7
		for {set n 1} {$n <= $num} {incr n} {
		    incr i
		    set twoTheta($i) [expr $bla + ($i-1) * $step]
		    set yobs(1,$i) \
			[expr double([string range $line $anfang $ende])]
		    incr anfang 8
		    incr ende 8
		}
		if {$diff < 8} {break}
	    }

# SIMREF-Plotfiles:

	} elseif {$fileSpec == "SIM"} {
	    set i 0
	    set ymax_all 1
	    set ymin_all 0
	    
	    while {[gets $fid line] >=0} {
#		set size [expr $size + [string length $line] + 1]
#		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {[scan $line "%f %f %f %f %f" \
			the obs calc diff bac] == 5} {
		    set i [expr $i + 1]
		    set twoTheta($i) $the
		    set yobs(1,$i) $obs
		    set ycalc($i) $calc
		    set ydiff($i) $diff
		    set yback($i) $bac
		    if {$obs > $ymax_all} {
			set ymax_all $obs
		    }
		    if {$calc > $ymax_all} {
			set ymax_all $calc
		    }
		    if {$obs < $ymin_all} {
			set ymin_all $obs
		    }
		    if {$calc < $ymin_all} {
			set ymin_all $calc
		    }
		} else {
		    break
		}
	    }
	    set izahl $i
	    set bla $twoTheta(1)
	    set ble $twoTheta($izahl)

# Messdaten vom Pulverdiffraktometer am B2 (HASYLAB):

	} elseif {$fileSpec == "HAS"} {
	    set i 0 
	    while {[gets $fid line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {[scan $line "%f %f %f %f" y(1) y(2) y(3) the] == 4} {
		    set i [expr $i + 1]
		    set twoTheta($i) $the
		    set yobs(1,$i) $y(1)
		    set yobs(2,$i) $y(2)
		    set yobs(3,$i) $y(3)
		} else {
		    break
		}
	    }
	    set izahl $i
	    set ble $the

# Messdaten vom Tuebinger Guinier-Diffraktometer (BEN9-Format):
 
	} elseif {$fileSpec == "BEN"} {
 	    if {$countNr == 2} {
		set shift $countDist
	    } else {
		set shift 0
	    }
	    set i 0
	    while {[gets $fid line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {[scan $line "%f %f %f" y(1) y(2) y(3)] == 3} {
		    set i [expr $i + 1]
		    set yobs(1,$i) $y(1)
		    set yobs(2,$i) $y(2)
		    set yobs(3,$i) $y(3)
		    set twoTheta($i) [expr $bla + ($i-1) * $step + $shift]
		    set mitMonitor [expr $mitMonitor + ($y(3) / $izahl)]
		} else {
		    break
		}
	    }
	    set izahl $i
	    set ble $twoTheta($izahl)

# Daten im freien Spaltenformat:

	} elseif {$fileSpec == "FOR"} {
	    set i 0
	    while {[gets $fid line] >= 0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		set i [expr $i + 1]
		for {set k 1} {$k <= $colZahl} {incr k} {
		    set n [expr $k - 1]
		    set daten($k,$i) [expr double([lindex $line $n])]
		}
	    }
	    set izahl $i

# W20-Ergebnisfiles:

	} elseif {$fileSpec == "ERG"} {
	    set i 0
	    while {[gets $fid line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	        set i [expr $i + 1]
		set anZahl [llength $line]
		if {$anZahl == 19} {
		    scan $line \
	        "%*d %f %*s %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f" \
			daten(1,$i) daten(2,$i) daten(4,$i) daten(6,$i) \
			daten(8,$i) daten(10,$i) daten(12,$i) daten(14,$i) \
			daten(16,$i) daten(18,$i) daten(19,$i) daten(20,$i) \
			daten(21,$i) daten(22,$i) daten(23,$i) daten(24,$i) \
			daten(25,$i)
		    gets $fid line
		    set size [expr $size + [string length $line] + 1]
		    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		    scan $line "%f %f %f %f %f %f %f %f" \
			daten(3,$i) daten(5,$i) daten(7,$i) daten(9,$i) \
			daten(11,$i) daten(13,$i) daten(15,$i) daten(17,$i)
		} elseif {$anZahl == 6} {
		    scan $line "%*s %f %f %f %f %f" daten(2,$i) \
		    daten(4,$i) daten(6,$i) daten(8,$i) daten(10,$i)
		    set k [expr $i - 1]
		    set daten(1,$i) $daten(1,$k)
		    for {set j 12} {$j <= $colZahl} {incr j} {
			set daten($j,$i) $daten($j,$k)
		    }
		    gets $fid line
		    set size [expr $size + [string length $line] + 1]
		    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		    scan $line "%f %f %f %f %f" \
			daten(3,$i) daten(5,$i) daten(7,$i) daten(9,$i) \
			daten(11,$i)
		}
	    }
	    set izahl $i

# Tuebinger Photometerdaten (VMS-Version):

	} elseif {$fileSpec == "PHO"} {
	    set i 0 
	    while {[gets $fid line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		set diff [expr $izahl - $i]
		if {$diff < 16} {
		    set num $diff
		} else {
		    set num 16
                }
		set anfang 0
		set ende 4
		for {set n 1} {$n <= $num} {incr n} {
		    incr i
		    set twoTheta($i) [expr $bla + ($i-1) * $step]
		    set yobs(1,$i) \
			[expr double([string range $line $anfang $ende])]
		    incr anfang 5
		    incr ende 5
		}
		if {$diff < 16} {break}
	    }

# Messdaten des Tuebinger Pulverdiffraktometers MONALISA:

	} elseif {$fileSpec == "SCN"} {
 	    if {$countNr == 1} {
		set shift $countDist
	    } else {
		set shift 0
	    }
	    set i 0
	    while {[gets $fid line] >=0} {
#		set size [expr $size + [string length $line] + 1]
#		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {[scan $line "%f %f %f %f %*f" the y(1) y(2) y(3)] == 4} {
		    set i [expr $i + 1]
		    set yobs(1,$i) $y(1)
		    set yobs(2,$i) $y(2)
		    set yobs(3,$i) $y(3)
		    set twoTheta($i) [expr $bla + ($i-1) * $step + $shift]
		    set mitMonitor [expr $mitMonitor + ($yobs(3,$i) / $izahl)]
		}
	    }
	    set ble $the

	}

	close $fid

    }

    gaugeWidget "Reading Block $blockNr"
    set size 0
    set maxSize [file size $file]
    if {[catch $cmd err] != 0} {
	tk_messageBox -icon error -type ok -title "Error" \
	    -message "Cannot read file:\n$file"
	destroy .gauge
	return -1
    } else {
	destroy .gauge
	.buttons.b1.plot configure -state "normal"
	.buttons.b1.w20plot configure -state "disabled"
	.buttons.b2.save configure -state "disabled"
	.display.block configure -text "Block $blockNr, Counter $countNr"
	.menuBar.file.m entryconfigure 2 -state "normal"
	.menuBar.data.m entryconfigure 5 -state "normal"
	.menuBar.data.m entryconfigure 6 -state "normal"
	.menuBar.plot.m entryconfigure 1 -state "normal"
	.menuBar.plot.m entryconfigure 2 -state "disabled"
	.menuBar.plot.m entryconfigure 6 -state "normal"
	.menuBar.plot.m entryconfigure 8 -state "normal"
	.menuBar.plot.m entryconfigure 12 -state "normal"
	.menuBar.plot.m entryconfigure 13 -state "normal"
	.menuBar.w20.m entryconfigure 14 -state "disabled"
	.menuBar.w20.m entryconfigure 4 -state "disabled"
	if {$fileSpec == "SIM"} {
	    .buttons.b1.input configure -state "disabled"
	    .buttons.b2.w20fit configure -state "disabled"
	    .menuBar.w20.m entryconfigure 15 -state "disabled"
	    .menuBar.plot.m entryconfigure 3 -state "disabled"
	    .menuBar.plot.m entryconfigure 7 -state "disabled"
	    .menuBar.plot.m entryconfigure 15 -state "disabled"
	    .menuBar.data.m entryconfigure 1 -state "disabled"
	    .menuBar.data.m entryconfigure 2 -state "disabled"
	} elseif {$fileSpec == "FOR" || $fileSpec == "ERG"} {
	    .buttons.b1.input configure -state "disabled"
	    .buttons.b2.w20fit configure -state "disabled"
	    .menuBar.w20.m entryconfigure 15 -state "disabled"
	    .menuBar.plot.m entryconfigure 3 -state "disabled"
	    .menuBar.plot.m entryconfigure 7 -state "normal"
	    .menuBar.plot.m entryconfigure 15 -state "disabled"
	    .menuBar.data.m entryconfigure 1 -state "disabled"
	    .menuBar.data.m entryconfigure 2 -state "disabled"
	} else {
	    .buttons.b1.input configure -state "normal"
	    .buttons.b2.w20fit configure -state "normal"
	    .menuBar.w20.m entryconfigure 15 -state "normal"
	    .menuBar.plot.m entryconfigure 3 -state "normal"
	    .menuBar.plot.m entryconfigure 7 -state "disabled"
	    .menuBar.data.m entryconfigure 1 -state "normal"
	    if {$fileSpec == "HAS"} {
		.menuBar.data.m entryconfigure 2 -state "normal"
	    } else {
		.menuBar.data.m entryconfigure 2 -state "disabled"
    	    }
	    if {$old == 1 && $fileSpec == "DUO"} {
		.menuBar.plot.m entryconfigure 15 -state "normal"
	    } else {
		.menuBar.plot.m entryconfigure 15 -state "disabled"
	    }
        }
	wm title . "TkPlot: [file tail $file]"
	return 0
    }

}

#==============================================================================
#              Minimum und Maximum der X-Werte bestimmen
#==============================================================================

proc xMinmax {filspe} {
    global bla ble xmin xmax countNr countDist daten absNr izahl\
	    xmin_all xmax_all
    
# Minimum und Maximum der X-Werte:

    if {$filspe == "BEN" && $countNr == 2} {
	set xmin(data) [expr $bla + $countDist]
	set xmax(data) [expr $ble + $countDist]
    } elseif {$filspe == "SCN" && $countNr == 1} {
	set xmin(data) [expr $bla + $countDist]
	set xmax(data) [expr $ble + $countDist]
    } elseif {$filspe == "FOR" || $filspe == "ERG"} {
	gaugeWidget "Setting X Range"
	for {set i 1} {$i <= $izahl} {incr i} {
	    gauge_value .gauge.g [expr round($i * 100 / $izahl)]
	    if {$i > 1} {
		if {$daten($absNr,$i) > $xmax(data)} {
		    set xmax(data) $daten($absNr,$i)
		} elseif {$daten($absNr,$i) < $xmin(data)} {
		    set xmin(data) $daten($absNr,$i)
		}
	    } else {
		set xmin(data) $daten($absNr,$i)
		set xmax(data) $xmin(data)
	    }
	}
	destroy .gauge
    } else {
	set xmin(data) $bla
	set xmax(data) $ble
    }
    set xmin_all $xmin(data)
    set xmax_all $xmax(data)
}

#==============================================================================
#              Minimum und Maximum der 2-Theta-Werte bestimmen
#==============================================================================

proc twoThMinmax {filspe} {
    global bla ble thmin thmax countNr countDist daten absNr izahl

# Minimum und Maximum der X-Werte:

    if {$filspe == "BEN" && $countNr == 2} {
	set thmin [expr $bla + $countDist]
	set thmax [expr $ble + $countDist]
    } elseif {$filspe == "SCN" && $countNr == 1} {
	set thmin [expr $bla + $countDist]
	set thmax [expr $ble + $countDist]
    } elseif {$filspe == "FOR" || $filspe == "ERG"} {
	gaugeWidget "Setting 2-Theta Range"
	for {set i 1} {$i <= $izahl} {incr i} {
	    gauge_value .gauge.g [expr round($i * 100 / $izahl)]
	    if {$i > 1} {
		if {$daten($absNr,$i) > $xmax(data)} {
		    set thmax $daten($absNr,$i)
		} elseif {$daten($absNr,$i) < $xmin(data)} {
		    set thmin $daten($absNr,$i)
		}
	    } else {
		set thmin $daten($absNr,$i)
		set thmax $thmin
	    }
	}
	destroy .gauge
    } else {
	set thmin $bla
	set thmax $ble
    }

}

#==============================================================================
#              Minimum und Maximum der Y-Werte bestimmen
#==============================================================================

proc yMinmax {filspe} {
    global daten izahl xmin xmax ymin ymax countNr absNr colZahl \
	symbolShape lineDraw bla step twoTheta yobs ycalc\
	ymin_all ymax_all

    gaugeWidget "Setting Y Range"

    set n 0
    if {$filspe == "SIM"} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    if {$twoTheta($i) >= $xmin(data) && \
		    $twoTheta($i) <= $xmax(data)} {
		incr n
		if {$n > 1} {
		    if {$yobs(1,$i) > $ymax(data)} {
			set ymax(data) $yobs(1,$i)
		    } elseif {$yobs(1,$i) < $ymin(data)} {
			set ymin(data) $yobs(1,$i)
		    }
		    if {$ycalc($i) > $ymax(data)} {
			set ymax(data) $ycalc($i)
		    } elseif {$ycalc($i) < $ymin(data)} {
			set ymin(data) $ycalc($i)
		    }
		} else {
		    if {$yobs(1,$i) < $ycalc($i)} {
			set ymin(data) $yobs(1,$i)
		    } else {
			set ymin(data) $ycalc($i)
		    }
		    set ymax(data) $ymin(data)
		}
	    } elseif {$twoTheta($i) > $xmax(data)} {
		break
            }
#	    gauge_value .gauge.g [expr round($i * 100 / $izahl)]
	}
    } elseif {$filspe == "FOR" || $filspe == "ERG"} {
	for {set i 1} {$i <= $izahl} {incr i} {
	    if {$daten($absNr,$i) >= $xmin(data) && \
		$daten($absNr,$i) <= $xmax(data)} {
		for {set k 1} {$k <= $colZahl} {incr k} {
		    if {$symbolShape($k) != "No Symbol" || \
			$lineDraw($k) == 1} {
			incr n
			if {$n > 1} {
			    if {$daten($k,$i) > $ymax(data)} {
				set ymax(data) $daten($k,$i)
			    } elseif {$daten($k,$i) < $ymin(data)} {
				set ymin(data) $daten($k,$i)
			    }
			} else {
			    set ymin(data) $daten($k,$i)
			    set ymax(data) $ymin(data)
			}
		    }
		}
            }
#	    gauge_value .gauge.g [expr round($i * 100 / $izahl)]
	}

    } else {
	for {set i 1} {$i <= $izahl} {incr i} {
	    if {$twoTheta($i) >= $xmin(data) && $twoTheta($i) <= $xmax(data)} {
		incr n
		if {$n == 1} {
		    set ymin(data) $yobs($countNr,$i)
		    set ymax(data) $ymin(data)
		} else {
		    if {$yobs($countNr,$i) > $ymax(data)} {
			set ymax(data) $yobs($countNr,$i)
		    } elseif {$yobs($countNr,$i) < $ymin(data)} {
			set ymin(data) $yobs($countNr,$i)
		    }
		}
	    } elseif {$twoTheta($i) > $xmax(data)} {
		break
            }
#	    gauge_value .gauge.g [expr round($i * 100 / $izahl)]
	}
    }
    destroy .gauge

}

#==============================================================================
#                    W20-Inputfile speichern
#==============================================================================

proc saveW20File {file} {
    global datFile refZahl cycZahl urf basPeak gewicht lambda \
	glFwhm glEta1 glEta2 glAsym backMod bacFile kSplit cycleNr \
	position fwhm intensity eta1 eta2 asymPar ratio back \
	refPos refFwhm refInt refEta1 refEta2 refAsym refBack refRatio \
	thmin thmax blockNr countNr profFunc winkelFehler

    set cmd {

	set fid [open $file w]

# BlockNr., ZaehlerNr., ProfilfunktionNr., Untergrund, Reflexlagen,
# Gewichtung, K_alpha1/K_alpha2-Aufspaltung, wdt, Winkelfehler

	puts -nonewline $fid "COD>  "
	puts -nonewline $fid "$blockNr  "
	puts -nonewline $fid "$countNr  "
	switch $profFunc {
	    "Gaussian"                {puts -nonewline $fid "1  "}
	    "Lorentzian"              {puts -nonewline $fid "2  "} 
	    "Modified Lorentzian"     {puts -nonewline $fid "3  "}
	    "Intermediate Lorentzian" {puts -nonewline $fid "4  "}
	    "Edgeworth Serie"         {puts -nonewline $fid "5  "}
	    "Pseudo-Voigt"            {puts -nonewline $fid "6  "}
	    "Pearson VII"             {puts -nonewline $fid "7  "}
	    "Double Gaussian"         {puts -nonewline $fid "8  "}
	    default                   {puts -nonewline $fid "1  "}
	}
	switch $backMod {
	    "Refinable linear Background" {puts -nonewline $fid "0  "}
	    "Use Background Data File"    {puts -nonewline $fid "1  "}
	    "Use Values from Data File"   {puts -nonewline $fid "2  "}
	    default                       {puts -nonewline $fid "0  "}
	}
	switch $gewicht {
	    "1"                         {puts -nonewline $fid "1  "}
	    "Use Values from Data File" {puts -nonewline $fid "2  "}
	    "Method of Rebmann et al."  {puts -nonewline $fid "3  "}
	    "1/y_obs"                   {puts -nonewline $fid "4  "}
	    default                     {puts -nonewline $fid "1  "}
	}
	switch $kSplit {
	    "Yes"   {puts -nonewline $fid "1  "}
	    "No"    {puts -nonewline $fid "0  "}
	    default {puts -nonewline $fid "0  "}
	}
	puts -nonewline $fid "$basPeak  "
	puts $fid $winkelFehler

# Zahl der Zyklen, urf, Zwangsbedingungen der Parameter:

	puts -nonewline $fid "FIT>  "
	puts -nonewline $fid "$cycZahl  "
	puts -nonewline $fid "$urf  "
	switch $glFwhm {
	    "individual" {puts -nonewline $fid "0  "}
	    "common"     {puts -nonewline $fid "1  "}
	    default      {puts -nonewline $fid "1  "}
	}
	switch $glEta1 {
	    "individual" {puts -nonewline $fid "0  "}
	    "common"     {puts -nonewline $fid "1  "}
	    default      {puts -nonewline $fid "1  "}
	}
	switch $glEta2 {
	    "individual" {puts -nonewline $fid "0  "}
	    "common"     {puts -nonewline $fid "1  "}
	    default      {puts -nonewline $fid "1  "}
	}
	switch $glAsym {
	    "individual" {puts $fid 0}
	    "common"     {puts $fid 1}
	    default      {puts $fid 1}
	}

# Winkelbereich, Wellenlaengen:

	puts -nonewline $fid "THE>  "
	puts -nonewline $fid "$thmin  "
	puts -nonewline $fid "$thmax  "
	puts -nonewline $fid "$lambda(1)  "
	puts $fid $lambda(2)

# Name of the data file:

	puts -nonewline $fid "DAT> "
	puts $fid $datFile(1)
	
# Name of the background data file:

	puts -nonewline $fid "BAC> "
	puts $fid $bacFile
	
# Globale Parameter:

	puts -nonewline $fid "GLB>  "
	puts -nonewline $fid "$back(1)  "
	puts -nonewline $fid "$back(2)  "
	puts -nonewline $fid "$ratio  "
	puts -nonewline $fid "$refBack(1)  "
	puts -nonewline $fid "$refBack(2)  "
	puts $fid $refRatio

# Reflexparameter:

	for {set i 1} {$i <= $refZahl} {incr i} {
	    puts -nonewline $fid "PAR>  "
	    puts -nonewline $fid "$position($i)  "
	    puts -nonewline $fid "$fwhm($i)  "
	    puts -nonewline $fid "$intensity($i)  "
	    puts -nonewline $fid "$eta1($i)  "
	    puts -nonewline $fid "$eta2($i)  "
	    puts -nonewline $fid "$asymPar($i)  "
	    puts -nonewline $fid "$refPos($i)  "
	    puts -nonewline $fid "$refFwhm($i)  "
	    puts -nonewline $fid "$refInt($i)  "
	    puts -nonewline $fid "$refEta1($i)  "
	    puts -nonewline $fid "$refEta2($i)  "
	    puts $fid $refAsym($i)
	}

	close $fid
    }

    if {[catch $cmd err] != 0} {
	tk_messageBox -icon error -type ok -title "Error" \
	    -message "Cannot save file:\n$file"
    }

}

#==============================================================================
#                      W20-Inputfile einlesen
#==============================================================================

proc readW20file {file} {
    global fileSpec blockNr countNr profFunc backMod gewicht kSplit \
	basPeak winkelFehler cycZahl urf glFwhm glEta1 glEta2 glAsym \
	thmin thmax lambda datFile bacFile back refBack ratio refRatio \
	refZahl position fwhm intensity eta1 eta2 asymPar \
        refPos refFwhm refInt refEta1 refEta2 refAsym \
	stddPos stddFwhm stddInt stddEta1 stddEta2 stddAsym

    set cmd {

	set fid [open $file r]

# BlockNr., ZaehlerNr., ProfilfunktionNr., Untergrund, Reflexlagen,
# Gewichtung, K_alpha1/K_alpha2-Aufspaltung, wdt, Winkelfehler

	while {[gets $fid line] >= 0} {
	    set kennung [string range $line 0 2]
	    set i [string first ">" $line]
	    set laenge [string length $line]
	    set i [expr $i + 1]
	    set zeile [string range $line $i $laenge]
	    if {$kennung == "COD" && $i > 0 && [llength $zeile] == 8} {
		set blockNr [lindex $zeile 0]
		set countNr [lindex $zeile 1]
		switch [lindex $zeile 2] {
		    "1" {set profFunc "Gaussian"}
		    "2" {set profFunc "Lorentzian"}
		    "3" {set profFunc "Modified Lorentzian"}
		    "4" {set profFunc "Intermediate Lorentzian"}
		    "5" {set profFunc "Edgeworth Serie"}
		    "6" {set profFunc "Pseudo-Voigt"}
		    "7" {set profFunc "Pearson VII"}
		    "8" {set profFunc "Double Gaussian"}
		    default {set $profFunc "Gaussian"}
		}
		switch [lindex $zeile 3] {
		    "0" {set backMod "Refinable linear Background"}
		    "1" {set backMod "Use Background Data File"}
		    "2" {set backMod "Use Values from Data File"}
		    default {set $backMod "Refinable linear Background"} 
		}
		switch [lindex $zeile 4] {
		    "1" {set gewicht "1"}
		    "2" {set gewicht "Use Values from Data File"}
		    "3" {set gewicht "Method of Rebmann et al."}
		    "4" {set gewicht "1/y_obs"}
		    default {set gewicht "1/y_obs"}
		}
		switch [lindex $zeile 5] {
		    "0" {set kSplit "No"}
		    "1" {set kSplit "Yes"}
		    default {set kSplit "No"}
		}
		set basPeak [lindex $zeile 6]
		set winkelFehler [lindex $zeile 7]
		break
	    } else {
		tk_messageBox -icon error -type ok -title "Error" \
		    -message "Error reading file:\n$file"
		return
	    }
	}

# Zahl der Zyklen, urf, Zwangsbedingungen der Parameter:

	while {[gets $fid line] >= 0} {
	    set kennung [string range $line 0 2]
	    set i [string first ">" $line]
	    set laenge [string length $line]
	    set i [expr $i + 1]
	    set zeile [string range $line $i $laenge]
	    if {$kennung == "FIT" && $i > 0 && [llength $zeile] == 6} {
		set cycZahl [lindex $zeile 0]
		set urf [lindex $zeile 1]
		switch [lindex $zeile 2] {
		    "0" {set glFwhm "individual"}
		    "1" {set glFwhm "common"}
		    default {set glFwhm "common"}
		}
		switch [lindex $zeile 3] {
		    "0" {set glEta1 "individual"}
		    "1" {set glEta1 "common"}
		    default {set glEta1 "common"}
		}
		switch [lindex $zeile 4] {
		    "0" {set glEta2 "individual"}
		    "1" {set glEta2 "common"}
		    default {set glEta2 "common"}
		}
		switch [lindex $zeile 5] {
		    "0" {set glAsym "individual"}
		    "1" {set glAsym "common"}
		    default {set glAsym "common"}
		}
		break
	    } else {
		tk_messageBox -icon error -type ok -title "Error" \
		    -message "Error reading file:\n$file"
		return
	    }
	}

# Winkelbereich, Wellenlaengen:

	while {[gets $fid line] >= 0} {
	    set kennung [string range $line 0 2]
	    set i [string first ">" $line]
	    set laenge [string length $line]
	    set i [expr $i + 1]
	    set zeile [string range $line $i $laenge]
	    if {$kennung == "THE" && $i > 0 && [llength $zeile] == 4} {
		set thmin [lindex $zeile 0]
		set thmax [lindex $zeile 1]
		set lambda(1) [lindex $zeile 2]
		set lambda(2) [lindex $zeile 3]
		break
	    } else {
		tk_messageBox -icon error -type ok -title "Error" \
		    -message "Error reading file:\n$file"
		return
	    }
	}

# Name of the data file:

	while {[gets $fid line] >= 0} {
	    set kennung [string range $line 0 2]
	    set i [string first ">" $line]
	    set laenge [string length $line]
	    set i [expr $i + 1]
	    set zeile [string range $line $i $laenge]
	    if {$kennung == "DAT" && $i > 0} {
		if {[llength $zeile] > 0} {set datFile(1) [string trim $zeile]}
		break
	    } else {
		tk_messageBox -icon error -type ok -title "Error" \
		    -message "Error reading file:\n$file"
		return
	    }
	}

# Name of the background data file:

	while {[gets $fid line] >= 0} {
	    set kennung [string range $line 0 2]
	    set i [string first ">" $line]
	    set laenge [string length $line]
	    set i [expr $i + 1]
	    set zeile [string range $line $i $laenge]
	    if {$kennung == "BAC" && $i > 0} {
		if {[llength $zeile] > 0} {set bacFile [string trim $zeile]}
		break
	    } else {
		tk_messageBox -icon error -type ok -title "Error" \
		    -message "Error reading file:\n$file"
		return
	    }
	}

# Globale Parameter:

	while {[gets $fid line] >= 0} {
	    set kennung [string range $line 0 2]
	    set i [string first ">" $line]
	    set laenge [string length $line]
	    set i [expr $i + 1]
	    set zeile [string range $line $i $laenge]
	    if {$kennung == "GLB" && $i > 0 && [llength $zeile] == 6} {
		set back(1) [lindex $zeile 0]
		set back(2) [lindex $zeile 1]
		set ratio [lindex $zeile 2]
		set refBack(1) [lindex $zeile 3]
		set refBack(2) [lindex $zeile 4]
		set refRatio [lindex $zeile 5]
		set stddback(1) 0.
		set stddback(2) 0.
		set stddRatio 0.
		break
	    } else {
		tk_messageBox -icon error -type ok -title "Error" \
		    -message "Error reading file:\n$file"
		return
	    }
	}

# Reflexparameter:

	set refZahl 0
	while {[gets $fid line] >= 0} {
	    set kennung [string range $line 0 2]
	    set i [string first ">" $line]
	    set laenge [string length $line]
	    set i [expr $i + 1]
	    set zeile [string range $line $i $laenge]
	    if {$kennung == "PAR" && $i > 0 && [llength $zeile] == 12} {
		set refZahl [expr $refZahl + 1]
		set position($refZahl) [lindex $zeile 0]
		set fwhm($refZahl) [lindex $zeile 1]
		set intensity($refZahl) [lindex $zeile 2]
		set eta1($refZahl) [lindex $zeile 3]
		set eta2($refZahl) [lindex $zeile 4]
		set asymPar($refZahl) [lindex $zeile 5]
		set refPos($refZahl) [lindex $zeile 6]
		set refFwhm($refZahl) [lindex $zeile 7]
		set refInt($refZahl) [lindex $zeile 8]
		set refEta1($refZahl) [lindex $zeile 9]
		set refEta2($refZahl) [lindex $zeile 10]
		set refAsym($refZahl) [lindex $zeile 11]
		set stddPos($refZahl) 0.
		set stddFwhm($refZahl) 0.
		set stddInt($refZahl) 0.
		set stddEta1($refZahl) 0.
		set stddEta2($refZahl) 0.
		set stddAsym($refZahl) 0.
	    } else {
		tk_messageBox -icon error -type ok -title "Error" \
		    -message "Error reading file:\n$file"
		return
	    }
	}

	close $fid

    }


    if {[catch $cmd err] != 0} {
	tk_messageBox -icon error -type ok -title "Error" \
	    -message "Cannot read file:\n$file"
    } else {
	busy_eval {
	    if {[scanDataFile $datFile(1)] >= 0} {
		plotInit $fileSpec
		readDataFile $datFile(1)
	    }
	}
    }

}

#==============================================================================
#
#==============================================================================

proc showOutput {} {

    set w .showOutput
    catch {destroy $w}
    toplevel $w
    wm title $w "Simref Output"

    after idle {
	update idletasks
        set xw [winfo screenwidth .showOutput]
	set x [expr ($xw - [winfo reqwidth .showOutput]-10)]
	wm geometry .showOutput "+$x+0"
    }

    frame $w.out
    scrollbar $w.out.xsbar -orient horizontal \
	-command {.showOutput.out.text xview}
    scrollbar $w.out.ysbar -orient vertical \
	-command {.showOutput.out.text yview}
    text $w.out.text -width 80 -height 25 -wrap none \
	-xscrollcommand {.showOutput.out.xsbar set} \
	-yscrollcommand {.showOutput.out.ysbar set}
    pack $w.out.xsbar -side bottom -fill x
    pack $w.out.ysbar -side right -fill y
    pack $w.out.text -side left -expand yes -fill both
    pack $w.out  -expand yes -fill both -padx 4 -pady 4

# Knoepfe:

    frame $w.buttons
    button $w.buttons.save -text "Save..." -width 7 -command "saveSimOutput"
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command "destroy $w"
    pack $w.buttons.save $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

}

#==============================================================================
#                    Save Simref Output
#==============================================================================

proc saveSimOutput {} {
    global logFile

    set types {
	{"Input Files" {.log}}
	{"All Files"       *}
    }

    set file [tk_getSaveFile -filetypes $types -parent .showOutput \
	-initialfile $logFile -title "Save Simref Output"]

    if [string compare $file ""] {
	set logFile $file
	busy_eval {
	    set fid [open $logFile w]
	    puts $fid [.showOutput.out.text get 1.0 end] 
	    close $fid
	}
    }

}

#==============================================================================
#                   Simref-Eingabedatei auswaehlen
#==============================================================================

proc openInpFile {w} {
    global inpFile parFile dataSetZahl plotFile datFile fileSpec \
	diagramType xmin xmax thmin thmax

    set types {
	{"Input Files" {.inp}}
	{"All Files"        *}
    }
    set file [tk_getOpenFile -filetypes $types -parent $w \
		-initialfile $inpFile(Simref) -title "Simref Input File"]

    if [string compare $file ""] {
	set dataSetZahl 0
	set inpFile(Simref) $file
	editInpFile
	if {[readInpFile $file] >= 0} {
	    if {[readParFile $parFile] >= 0} {
		.menuBar.simref.m entryconfigure 2 -state "normal"
		.menuBar.simref.m entryconfigure 3 -state "normal"
		.menuBar.simref.m entryconfigure 5 -state "normal"
		.menuBar.data.m entryconfigure 4 -state "normal"
		.menuBar.simref.m entryconfigure 10 -state "normal"
		.buttons.b2.simref configure -state "normal"
		set name [file rootname $inpFile(Simref)]
		if {$dataSetZahl > 1} {
		    for {set i 1} {$i <= $dataSetZahl} {incr i} {
			set plotFile($i) "$name$i.plt"
		    }
		} else {
		    set plotFile(1) "$name.plt"
		}
		busy_eval {
		    if {[scanDataFile $datFile(1)] >= 0} {
			plotInit $fileSpec
			readDataFile $datFile(1)
			xMinmax $fileSpec
			set thmin $xmin(data)
			set thmax $xmax(data)
			yMinmax $fileSpec
			set diagramType data
			diagram_redraw $diagramType
		    }
		}
	    }
	}
    }

}

#==============================================================================
#                Simref-Eingabedatei einlesen
#==============================================================================

proc readInpFile {file} {
    global parFile dataSetZahl inpTitle datFile

    if {[catch {set fid [open $file r]} err] != 0} {
	tk_messageBox -icon warning -type ok -title "Warning" \
	    -message "Cannot open file:\n$file"
	return -1
    } else {
	set dataSetZahl 0
	while {[gets $fid line] >= 0} {
	    switch [string range $line 0 2] {
		"INI" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"FIT" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"PAR" {.editInpFile.inp.text insert end "$line\n" eingabe
			set i1 [expr [string first ">" $line] + 1]
			set i2 [string length $line]
			set parFile [string range $line $i1 $i2]
			set parFile [string trim $parFile]
                      }
		"TIT" {.editInpFile.inp.text insert end "$line\n" eingabe
			incr dataSetZahl
			set i1 [expr [string first ">" $line] + 1]
			set i2 [string length $line]
			set inpTitle($dataSetZahl) [string range $line $i1 $i2]
		      }
		"COD" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"DAT" {.editInpFile.inp.text insert end "$line\n" eingabe
			set i1 [expr [string first ">" $line] + 1]
			set i2 [string length $line]
			set datFile($dataSetZahl) [string range $line $i1 $i2]
			set datFile($dataSetZahl) \
			    [string trim $datFile($dataSetZahl)]
		      }
		"BAC" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"SCL" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"THE" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"EXC" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"WVL" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"SCA" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"COR" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"PHS" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"PRO" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"SPG" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"SYM" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"ETA" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"ASY" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"PRE" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"MET" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"WAV" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"ATO" {.editInpFile.inp.text insert end "$line\n" eingabe}
		"INT" {.editInpFile.inp.text insert end "$line\n" eingabe}
		default {.editInpFile.inp.text insert end "$line\n" text}
	    }
	    while {[string last "\\" $line] > 0} {
		if {[gets $fid line] >= 0} {
		    .editInpFile.inp.text insert end "$line\n" eingabe
                } else {
		    break
		}
            }
        }
	close $fid
	return 0
    }
}

#==============================================================================
#                Simref-Eingabedatei speichern
#==============================================================================

proc saveInpFile {file} {
    
    set cmd {
	set fid [open $file w]
	puts $fid [.editInpFile.inp.text get 1.0 end] 
	close $fid
    }
    if {[catch $cmd err] != 0} {
	tk_messageBox -icon error -type ok -title "Error" \
	    -message "Cannot save file:\n$file"
    }

}

#==============================================================================
#               Simref Eingabedatei editieren     
#==============================================================================

proc editInpFile {} {
    global inpFile parFile

    set w .editInpFile
    catch {destroy $w}
    toplevel $w
    wm title $w "Simref Input Files"

# Eingabedatei:

    frame $w.if
    label $w.if.label -text "Input File:" -anchor w
    label $w.if.name -textvariable inpFile(Simref) -anchor w
    pack $w.if.label $w.if.name -side left -padx 4
    pack $w.if -side top -fill x -pady 2

    frame $w.inp
    scrollbar $w.inp.xsbar -orient horizontal \
	-command {.editInpFile.inp.text xview}
    scrollbar $w.inp.ysbar -orient vertical \
	-command {.editInpFile.inp.text yview}
    text $w.inp.text -width 80 -height 25 -wrap none -background white \
	-xscrollcommand {.editInpFile.inp.xsbar set} \
	-yscrollcommand {.editInpFile.inp.ysbar set}
    pack $w.inp.xsbar -side bottom -fill x
    pack $w.inp.ysbar -side right -fill y
    pack $w.inp.text -side left -expand yes -fill both
    pack $w.inp  -expand yes -fill both -padx 4 -pady 4

    $w.inp.text tag configure eingabe -foreground black
    $w.inp.text tag configure text -foreground grey

# Parameterdatei:

    frame $w.pf
    label $w.pf.label -text "Parameter File:" -anchor w
    label $w.pf.name -textvariable parFile -anchor w
    pack $w.pf.label $w.pf.name -side left -padx 4
    pack $w.pf -side top -fill x -pady 2

    frame $w.par
    scrollbar $w.par.xsbar -orient horizontal \
	-command {.editInpFile.par.text xview}
    scrollbar $w.par.ysbar -orient vertical \
	-command {.editInpFile.par.text yview}
    text $w.par.text -width 80 -height 10 -wrap none -background white \
	-xscrollcommand {.editInpFile.par.xsbar set} \
	-yscrollcommand {.editInpFile.par.ysbar set}
    pack $w.par.xsbar -side bottom -fill x
    pack $w.par.ysbar -side right -fill y
    pack $w.par.text -side left -expand yes -fill both
    pack $w.par  -expand yes -fill both -padx 4 -pady 4

    $w.par.text tag configure eingabe -foreground black
    $w.par.text tag configure text -foreground grey

# Knoepfe:

    frame $w.buttons
    button $w.buttons.save -text "Save" -width 7 -command {
	saveInpFile $inpFile(Simref)
	saveParFile $parFile
    }
    button $w.buttons.refine -text "Refine" -width 7 \
	-command "simRefFit \$inpFile(Simref)"
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command "destroy $w"
    pack $w.buttons.save $w.buttons.refine $w.buttons.dismiss \
	-side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

}

#==============================================================================
#                Simref-Parameter-Datei einlesen
#==============================================================================

proc readParFile {file} {

    if {[catch {set fid [open $file r]} err] != 0} {
	tk_messageBox -icon warning -type ok -title "Warning" \
	    -message "Cannot open file:\n$file"
	return -1
    } else {
	while {[gets $fid line] >= 0} {
	    switch [string range $line 0 2] {
		"TIT" {.editInpFile.par.text insert end "$line\n" eingabe}
		"PAR" {.editInpFile.par.text insert end "$line\n" eingabe}
		"GLB" {.editInpFile.par.text insert end "$line\n" eingabe}
		"PRO" {.editInpFile.par.text insert end "$line\n" eingabe}
		"MET" {.editInpFile.par.text insert end "$line\n" eingabe}
		"ATO" {.editInpFile.par.text insert end "$line\n" eingabe}
		default {.editInpFile.par.text insert end "$line\n" text}
	    }
	    while {[string last "\\" $line] > 0} {
		if {[gets $fid line] >= 0} {
		    .editInpFile.par.text insert end "$line\n" eingabe
                } else {
		    break
		}
            }
        }
	close $fid
	return 0
    }
}

#==============================================================================
#                Simref-Parameter-Datei speichern
#==============================================================================

proc saveParFile {file} {
    
    set cmd {
	set fid [open $file w]
	puts $fid [.editInpFile.par.text get 1.0 end] 
	close $fid
    }
    if {[catch $cmd err] != 0} {
	tk_messageBox -icon error -type ok -title "Error" \
	    -message "Cannot save file:\n$file"
    }

}

#==============================================================================
#               Simref Parameter-Datei editieren     
#==============================================================================

proc editParFile {file} {

    set w .editParFile
    catch {destroy $w}
    toplevel $w
    wm title $w "Simref Parameter File"

    frame $w.par
    scrollbar $w.par.xsbar -orient horizontal \
	-command {.editParFile.par.text xview}
    scrollbar $w.par.ysbar -orient vertical \
	-command {.editParFile.par.text yview}
    text $w.par.text -width 80 -height 25 -wrap none -background white \
	-xscrollcommand {.editParFile.par.xsbar set} \
	-yscrollcommand {.editParFile.par.ysbar set}
    pack $w.par.xsbar -side bottom -fill x
    pack $w.par.ysbar -side right -fill y
    pack $w.par.text -side left -expand yes -fill both
    pack $w.par  -expand yes -fill both -padx 4 -pady 4

    $w.par.text tag configure eingabe -foreground black
    $w.par.text tag configure text -foreground grey

# Knoepfe:

    frame $w.buttons
    button $w.buttons.save -text "Save" -command "saveParFile $file"
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command "destroy $w"
    pack $w.buttons.save $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

}

#==============================================================================
#                   W20-Input-Datei auswaehlen und einlesen
#==============================================================================

proc openW20File {w} {
    global inpFile datFile fileSpec diagramType xmin xmax thmin thmax

    set types {
	{"Input Files" {.inp}}
	{"All Files"        *}
    }
    set file [tk_getOpenFile -filetypes $types -parent $w \
		-initialfile $inpFile(W20) -title "Input File"]

    if [string compare $file ""] {
	set inpFile(W20) $file
	busy_eval {
	    readW20file $inpFile(W20)
	    set xmin(data) $thmin
	    set xmax(data) $thmax
	    yMinmax $fileSpec
	    set diagramType data
	    diagram_redraw $diagramType
	}
    }

}

#==============================================================================
#                   W20-Input-Datei auswaehlen und einlesen
#==============================================================================

proc w20Save {w} {
    global inpFile

    set types {
	{"Input Files" {.inp}}
	{"All Files"       *}
    }
    set file [tk_getSaveFile -filetypes $types -parent $w \
		-initialfile $inpFile(W20) -title "Input File"]

    if [string compare $file ""] {
	set inpFile(W20) $file
	busy_eval {saveW20File $inpFile(W20)}
    }

}

#==============================================================================
#              Cursor auf Uhr-Symbol umschalten
#==============================================================================

proc busy_eval {script} {
    set fwin [focus]
    focus .busylock
    grab set .busylock

    set i 0 
    foreach w { "." ".xyRange" ".cursorInput" ".w20PlotPar" ".errorBars" \
	"dataPlotPar" ".aboutBox" ".colPlotPar" ".output" ".showOutput" \
	".markRefWidget" ".backgrFile" ".showDataPar" ".picTitle" ".xyAxes" \
        ".settings" ".reflexPar" ".globalPar" ".convWidget" ".mergeWidget" \
	".gauge" ".dplot" ".positionData" ".ergFileWidget" ".residuals" \
	".selBlockPar" ".backgroundData" ".twothRange" ".editInpFile" \
	".selDataFile"} {
	if {[winfo exists $w] == 1} { 
	    set cursor($w) [$w cget -cursor]
	    $w configure -cursor watch
	}
    }
    update

    set status [catch {uplevel $script} result]

    set i 0 
    foreach w { "." ".xyRange" ".cursorInput" ".w20PlotPar" ".errorBars" \
	"dataPlotPar" ".aboutBox" ".colPlotPar" ".output" ".showOutput" \
	".markRefWidget" ".backgrFile" ".showDataPar" ".picTitle" ".xyAxes" \
        ".settings" ".reflexPar" ".globalPar" ".convWidget" ".mergeWidget" \
	".gauge" ".dplot" ".positionData" ".ergFileWidget" ".residuals" \
	".selBlockPar" ".backgroundData" ".twothRange" ".editInpFile" \
	".selDataFile"} {
	if {[winfo exists $w] == 1} { 
	    $w configure -cursor $cursor($w)
	}
    }

    grab release .busylock
    focus $fwin
    return -code $status $result
}

#==============================================================================
#                   Plot nach dem Einlesen initialisieren
#==============================================================================

proc plotInit {filspe} {
    global plotTitel titel colZahl colName xAchse absNr xErrorBars yErrorBars \
	xAxisLabel yAxisLabel symbolShape symbolSize symbolColor \
	symbolFillColor lineDraw lineColor lineWidth inpType dateiTyp refPhaseColor\
	hkl_size hklDraw refAlpha hkl_alpha1 hkl_alpha2 hkl_pos \
	xmin_all xmax_all ymax_all ymin_all

    if {$filspe == "ERG"} {
	set colZahl 25
	set colName(1)  "Temperature / Kelvin"
	set colName(2)  "2-theta / degrees"
	set colName(3)  "Stdd."
	set colName(4)  "FWHM / degrees"
	set colName(5)  "Stdd."
	set colName(6)  "Intensity"
	set colName(7)  "Stdd."
	set colName(8)  "Eta (Pseudo Voigt)"
	set colName(9)  "Stdd."
	set colName(10) "Asymmetry Parameter"
	set colName(11) "Stdd."
	set colName(12) "I(K_alpha2) / I(K_alpha1)"
	set colName(13) "Stdd."
	set colName(14) "Background B0 (constant)"
	set colName(15) "Stdd."
	set colName(16) "Background B1 (slope)"
	set colName(17) "Stdd."
	set colName(18) "R-weighted pattern" 
	set colName(19) "R-expected"
	set colName(20) "Goodness-of-fit"
	set colName(21) "Chisquare"
	set colName(22) "N-P"
	set colName(23) "N_sigma"
	set colName(24) "Durbin-Watson d"
	set colName(25) "Durbin-Watson Q"
	set absNr 2
	set xAchse $colName(2)
	set xAxisLabel $colName(2)
	set yAxisLabel $colName(4)
	for {set i 1} {$i <= $colZahl} {incr i} {
	    set symbolShape($i) "No Symbol"
	    set symbolColor($i) "black"
	    set symbolFillColor($i) "red"
	    set symbolSize($i) 7
	    set lineDraw($i) 0
	    set lineColor($i) "black"
	    set lineWidth($i) 1
	}
	set symbolShape(4) "Circle"
	set lineDraw(4) 1
	set xErrorBars 0
	set yErrorBars 1
    } elseif {$filspe == "FOR"} {
	set absNr 1
	set xAchse $colName(1)
	set xErrorBars 0
	set yErrorBars 0
	set xAxisLabel $colName(1)
	set yAxisLabel $colName(2)
	for {set i 1} {$i <= $colZahl} {incr i} {
	    set symbolShape($i) "No Symbol"
	    set symbolColor($i) "black"
	    set symbolFillColor($i) "red"
	    set symbolSize($i) 7
	    set lineDraw($i) 0
	    set lineColor($i) "black"
	    set lineWidth($i) 1
	}
	set symbolShape(2) "Circle"
	set lineDraw(2) 1
    } else {
	set symbolSize(1) 7
	set symbolColor(1) "black"
	set symbolFillColor(1) "red"
	if {$filspe == "SIM"} {
	    set plotTitle $titel(1)
	    set lineDraw(1) 0
	    set symbolShape(1) "Circle"
 	    set markRef 1
	} else {
	    set lineDraw(1) 1
	    set symbolShape(1) "No Symbol"
 	    set markRef 0
	}
	set lineColor(1) "black"
	set lineWidth(1) 1
	set lineDraw(2) 1
	set lineWidth(2) 1
	set lineColor(2) "blue"
	set lineDraw(3) 1
	set lineWidth(3) 1
	set lineColor(3) "green"
	set lineDraw(4) 0
	set lineWidth(4) 1
	set lineColor(4) "red"
	set lineDraw(5) 0
	set lineWidth(5) 1
	set lineColor(5) "black"
	set xAxisLabel "2-theta"
	set yAxisLabel "Counts"
	set refPhaseColor(1) "black"
	set refPhaseColor(2) "blue"
	set refPhaseColor(3) "green"
	set refPhaseColor(4) "off"
	set refPhaseColor(5) "off"
	set hkl_alpha1 1
	set hkl_alpha2 0
	set hklDraw 0
	set hkl_pos "below mark"
	set ymax_all 1
	set ymin_all 20
	set xmin_all 0
	set xmax_all 1
	set max_bekannt 0
    }

    switch $filspe {
	"DAT" {set inpType $dateiTyp(1)} 
	"DUO" {set inpType $dateiTyp(2)}
	"NEU" {set inpType $dateiTyp(3)}
	"SIM" {set inpType $dateiTyp(4)}
	"HAS" {set inpType $dateiTyp(5)}
	"BEN" {set inpType $dateiTyp(6)}
	"SCN" {set inpType $dateiTyp(7)}
	"PHO" {set inpType $dateiTyp(8)}
	"GRE" {set inpType $dateiTyp(9)}
	"DMC" {set inpType $dateiTyp(10)}
	"ERG" {set inpType $dateiTyp(11)}
	"FOR" {set inpType $dateiTyp(12)}
	"TOF" {set inpType $dateiTyp(13)}
	default {set inpType "Unknown"}
    }

}

#==============================================================================
#                    H a u p t p r o g r a m m 
#==============================================================================

eval destroy [winfo child .]
wm title . "TkPlot"
wm iconname . "TkPlot"

# Pfad fuer ausfuerbare Programme:
#set binPath "/home/ritter/bin"

# Auswahllisten:
set symbolList {}
foreach i {"No Symbol" "Circle" "Square" "Up Triangle" "Down Triangle"
	   "Diamond" "Cross (+)" "Cross (x)" "Rebmannflche"} {
    lappend symbolList $i
}	
set colorList {}
foreach i {"black" "white" "grey" "red" "blue" "green" "yellow" "brown" \
	   "orange"} {
    lappend colorList $i
}
set posList {}
foreach i {"below mark" "above peak"} {
    lappend posList $i
}

set colorListT {}
foreach i {"black" "white" "grey" "red" "blue" "green" "yellow" "brown" \
	   "orange" "Off"} {
    lappend colorListT $i
}


# Allgemein
set diagramColor "white"
set xmin(data) 0.
set xmax(data) 1.
set ymin(data) 0.
set ymax(data) 1.
set xmin(W20) 0.
set xmax(W20) 1.
set ymin(W20) 0.
set ymax(W20) 1.
set plotTitle ""
set xAxisLabel "X-axis"
set yAxisLabel "Y-axis"
set axesColor "black"
set axesWidth 1
set numXTics 5
set numYTics 4

# Diffraktometer-Daten:
set symbolShape(1) "Circle" 
set symbolSize(1) 7
set symbolColor(1) "black"
set symbolFillColor(1) "red"
set lineDraw(1) 0
set lineColor(1) "black"
set lineWidth(1) 1
set lineDraw(2) 1
set lineWidth(2) 1
set lineColor(2) "blue"
set lineDraw(3) 1
set lineWidth(3) 1
set lineColor(3) "green"
set lineDraw(4) 0
set lineWidth(4) 1
set lineColor(4) "red"
set lineDraw(5) 0
set lineWidth(5) 1
set lineColor(5) "black"
set lineConnect 0
set rebfakt(1) 0.001
# W20-Plotfile:
set w20symbolShape(1) "Circle" 
set w20symbolSize(1) 7
set w20symbolColor(1) "black"
set w20symbolFillColor(1) "red"
set w20lineDraw(1) 0
set w20lineColor(1) "black"
set w20lineWidth(1) 1
set w20lineDraw(2) 1
set w20lineWidth(2) 1
set w20lineColor(2) "blue"
set w20lineDraw(3) 1
set w20lineWidth(3) 1
set w20lineColor(3) "green"
set w20lineDraw(4) 0
set w20lineWidth(4) 1
set w20lineColor(4) "red"

# Daten im Spaltenformat:
set xErrorBars 0
set yErrorBars 0
set errorBarColor "black"
set errorBarWidth 1
set xAchse ""
set absNr 1
set colZahl 0

# Datei mit Reflexlagen:
set refFile ""
set ref_Zahl 0
set markRef 0
set markRefColor "black"
set markRefWidth 1

# Datei mit Untergrundstuetzpunkten: 
set untFile ""
set unt_Zahl 0
set backgrDraw 0
set backgrColor "black"
set backgrWidth 1

# Cursor-Koorddinaten:
set zahlPos 0

# Schriftarten definitionen
set hkl_size 8
font create "hkl_font" -size $hkl_size

# Dateitypen:
set dateiTyp(1) "Simref 2.x Data Format"
set dateiTyp(2) "Tuebingen DUO Format"
set dateiTyp(3) "Simref 2.x Neutron Data Format"
set dateiTyp(4) "Simref 2.x Plot File"
set dateiTyp(5) "HASYLAB Synchrotron Data (B2)"
set dateiTyp(6) "Tuebingen Guinier Data (BEN9)"
set dateiTyp(7) "Tuebingen Guinier Data (MONALISA)"
set dateiTyp(8) "Tuebingen Photometer Data"
set dateiTyp(9) "ILL Neutron Data (D2B)"
set dateiTyp(10) "PSI Neutron Data (DMC)"
set dateiTyp(11) "W20 Result Files"
set dateiTyp(12) "Data in Column Format"
set dateiTyp(13) "Time-of-Flight Data"

# W20-Voreinstellungen:
set hkl_Zahl 0
set plotFile(1) "w20.plt"
set inpFile(W20) "w20.inp"
set ergFile "w20.erg"
set profFunc "Pseudo-Voigt"
set gewicht "1/y_obs"
set winkelFehler 0.
set basPeak 3.5
set urf 1.0
set cycZahl 50
set glFwhm "common"
set glEta1 "common"
set glEta2 "common"
set glAsym "common"
set lambda(1) 1.5406
set lambda(2) 1.5444
set ratio 0.5
set kSplit "No"
set refRatio 0
set stddRatio 0.
set back(1) 0.
set back(2) 0.
set refBack(1) 1
set refBack(2) 1
set stddBack(1) 0.
set stddBack(2) 0.
set backMod "Refinable linear Background"
set thmin 0.
set thmax 0.
set bacFile ""
set hklFile ""

# Reflex 1:
set refZahl 1
set position(1)  0.
set fwhm(1) 0.1
set intensity(1) 100.
set eta1(1) 0.
set eta2(1) 0.
set asymPar(1) 0.
set refPos(1) 1
set refFwhm(1) 1
set refInt(1) 1
set refEta1(1) 0
set refEta2(1) 0
set refAsym(1) 0
set stddPos(1) 0.
set stddFwhm(1) 0.
set stddInt(1) 0.
set stddEta1(1) 0.
set stddEta2(1) 0.
set stddAsym(1) 0.

# R-Werte:
set chisq 0.
set relaxF 0.
set cycleNr 0
set num 0
set ndata 0
set npar 0
set rp 0.
set rwp 0.
set rexp 0.
set nsigma 0.
set gof 0.
set q_DW 0.
set d_DW 0.
set q4_DW 0.

# Convert:
set conMode 1
set conType 1

# Dreide:
set auge3D(1) 20.
set auge3D(2) -40.
set auge3D(3) 40.
set zScale3D 0.
set zGrund3D 0.
set zDach3D  0.
set box3D "Unvisible"
set line3D "Only Lines in x Direction (2-theta)"
set edge3D "Suppress all covered Edges"
set face3D "Unvisible"
set dar3D "Equidistant Blocks"
set tmin 0.
set tmax 0.
set velosity "5"
set zoomfac "1."
set picfont "2"
set outType "PostScript"
set outFile "output.ps"

# Datensatz-Parameter:
set datFile(1) ""
set titelZahl 0
set old 0
set fileSpec ""
set inpType "Unknown"
set blockNr 1
set countNr 1
set blockZahl 1
set countZahl 1
set countDist 0.
set step 0.
set zeit 0.
set temp 0.
set bnr 0
set izahl 0
set bla 0.
set ble 0.

# Simref:
set simMultRef 0
set inpFile(Simref) ""
set parFile ""
set logFile "simref.log"
set dataSetZahl 1
set plotFileNr 1

### Drucker Menue erzeugen:

printer_create .print

#------------------------------------------------------------------------------

### Pull-Down-Menues:

#set font -*-Helvetica-Medium-R-Normal--*-140-*-*-*-*-*-*

set w .menuBar
frame $w -relief raised -bd 2
pack $w -side top -fill x

### File-Menue:

set m $w.file.m
menubutton $w.file -text "File" -menu $m -underline 0 
menu $m 
$m add command -label "Open... " -underline 0 -command "openDataFile $w" \
	-accelerator Ctrl-o
bind . <Control-o> {openW20File $w}
$m add command -label "Reopen" -underline 0 -state "disabled" -command {
    busy_eval {
	readDataFile $datFile(1)
#	yMinmax $fileSpec
	if {$fileSpec == "SIM"} {readhklFile $refFile}
	Plot_all
	set diagramType data
	diagram_redraw $diagramType
    }
}
$m add command -label "Print... "  -underline 0 \
	-command "printer_print .print draw_print"
$m add sep
$m add command -label "Exit" -command "destroy ." -underline 0
pack $w.file -side left

### Data-Menue:

set m $w.data.m
menubutton $w.data -text "Data" -menu $m -underline 0 
menu $m 
$m add command -label "Convert... " -underline 0 -command "convWidget" \
	-accelerator Ctrl-c -state "disabled" 
bind . <Control-c> {convWidget}
$m add command -label "Merge... " -underline 0 -command "mergeWidget" \
	-accelerator Ctrl-m -state "disabled"
bind . <Control-c> {mergeWidget} 
$m add sep
$m add command -label "Data Sets..." -underline 0 \
	-command {selDataFile} -state "disabled"
$m add command -label "Block and Counter... " -underline 0 \
	-command "selBlockPar" -state "disabled"
$m add command -label "Show Parameters... " -underline 0 \
	-command "showDataPar" -state "disabled"
pack $w.data -side left

### Plot-Menue:

set m $w.plot.m
menubutton $w.plot -text "Plot" -menu $m -underline 0
menu $m
$m add command -label "Data... " -underline 0 -state "disabled" \
	-accelerator Ctrl-p -command {
	set diagramType data 
	busy_eval {diagram_redraw $diagramType}
}
bind . <Control-p> {
	set diagramType data 
	busy_eval {diagram_redraw $diagramType}
}
$m add command -label "Fitted Data... " -underline 0  -state "disabled" \
	-accelerator Ctrl-f -command {
	set diagramType W20 
	busy_eval {diagram_redraw $diagramType}
}
bind . <Control-f> {
	set diagramType W20 
	busy_eval {diagram_redraw $diagramType}
}
$m add command -label "Cursor Input... " -underline 0  -state "disabled" \
	-accelerator Ctrl-i -command {cursorInput}
bind . <Control-i> {cursorInput}
$m add sep
$m add command -label "Canvas..." -underline 0 -command "canvasWidget" \
	-state "normal"
$m add command -label "Symbols and Lines..." -underline 0 -command "plotPar" \
	-state "disabled"
$m add command -label "Error Bars... " -underline 0 -command "errorBars" \
	-state "disabled"
$m add command -label "Scaling..." -underline 0 -command "xyRange" \
	-state "disabled"
$m add command -label "Axes... " -underline 0 -command "xyAxes"
$m add command -label "Title... " -underline 0 -command "picTitle"
$m add sep
$m add command -label "Reflections... " -underline 0 \
	-command "markRefWidget" -state "disabled"
$m add command -label "Background... " -underline 0 \
	-command "backgroundData" -state "disabled"
$m add sep
$m add command -label "3D Plot... " -underline 0 -command "dplot" \
	-state "disabled"
pack $w.plot -side left

### W20-Menue:

set m $w.w20.m
menubutton $w.w20 -text "W20" -menu $m -underline 0
menu $m
$m add command -label "Open... " -underline 0 -command "openW20File $w"
$m add command -label "Save " -underline 0 -command "saveW20File $inpFile(W20)"
$m add command -label "Save As... " -underline 5 -command "w20Save $w"
$m add command -label "Save Results... " -underline 5 -state "disabled" \
	-command "ergFileWidget"
$m add sep
$m add command -label "Reflections Parameters... " \
	-underline 12 -command "reflexPar"
$m add command -label "Global Parameters... " -underline 0 -command "globalPar"
$m add command -label "2-theta Range... " -underline 2 -command "twoThRange"
$m add command -label "Settings... " -underline 0 -command "settings"
$m add sep
$m add command -label "Reflection Data File... " -underline 11 \
	-command "positionData"
$m add command -label "Background Data File... " -underline 0 \
-command "backgrFile"
$m add sep
$m add command -label "Show Residuals" -underline 3 -state "disabled"\
	-command "showResiduals"
$m add command -label "Refine " -underline 0 -state "disabled" \
	-accelerator Ctrl-w -command "w20fit $inpFile(W20)"
bind . <Control-w> {w20fit $inpFile(W20)}
pack $w.w20 -side left

### Simref:

set m $w.simref.m
menubutton $w.simref -text "Simref" -menu $m -underline 0
menu $m
$m add command -label "Open... " -underline 0 -command {openInpFile .}
$m add command -label "Save " -underline 0 -state "disabled" 
$m add command -label "Save As... " -underline 0 -state "disabled"
$m add sep
$m add command -label "Edit Input... " -underline 0  -state "disabled" \
	-command {
	    editInpFile
	    readInpFile $inpFile(Simref)
	    readParFile $parFile
}
$m add command -label "Base Vectors..." -underline 0 -state "disabled"
$m add sep
$m add cascade -label "Options..." -menu $m.options -underline 0
$m add sep
$m add command -label "Refine" -underline 0 -state "disabled" \
	-accelerator Ctrl-s -command "simRefFit \$inpFile(Simref)"
bind . <Control-s> {simrefFit $inpFile(Simref)}
menu $m.options -tearoff 0
$m.options add radio -label "Single Refinement" -variable simMultRef -value 0
$m.options add radio -label "Multiple Refinement" -variable simMultRef -value 1

pack $w.simref -side left

### Help-Menue:

set m $w.help.m
menubutton $w.help -text Help -menu $m -underline 0
menu $m
$m add command -label "About... " -command "aboutBox" -underline 0
pack $w.help -side right

#------------------------------------------------------------------------------

### Anzeige der Koordinaten:

frame .display
label .display.coords -text "X, Y =" -anchor w
label .display.block -text ""
pack .display.coords -side left -padx 4
pack .display.block -side right -padx 4
pack .display -side top -fill x -pady 2
frame .sep2 -height 2 -borderwidth 1 -relief sunken
pack .sep2 -fill x -pady 0

### Diagramm zeichnen:

set dinA4 1.3809524
set hoehe 280
set breite [expr $hoehe * $dinA4]
canvas .diagram -width $breite -height $hoehe \
    -background $diagramColor -highlightthickness 0
bind .diagram <ButtonPress-1> "cursorCoords %x %y"
bind .diagram <Motion> "displayCoords %x %y"
pack .diagram -fill both -expand yes
bind .diagram <ButtonPress-3> "setleft %x %y"
bind .diagram <ButtonRelease-3> "setright %x %y"
set diagramType data
diagram_$diagramType $breite $hoehe
bind .diagram <Configure> {busy_eval {diagram_redraw $diagramType}}

### Buttons:

frame .sep3 -height 2 -borderwidth 1 -relief sunken
pack .sep3 -fill x -pady 0

frame .buttons
pack .buttons -side top -fill x -pady 2m

frame .buttons.b1
pack .buttons.b1 -side top -padx 4
button .buttons.b1.plot -text "Plot All" -width 15 \
	-state "disabled" -command {
                Plot_all
#	    set diagramType data 
#    busy_eval {diagram_redraw $diagramType}
}
bind .buttons.b1.plot <Enter> {.statusBar.lab config -text "Plot data"}
bind .buttons.b1.plot <Leave> {.statusBar.lab config -text " "}
button .buttons.b1.w20plot -text "Plot Refined Data" -width 15 \
	-state "disabled" -command {
	    set diagramType W20
	    busy_eval {diagram_redraw $diagramType}
}
bind .buttons.b1.w20plot <Enter> \
	{.statusBar.lab config -text "Plot fitted data"}
bind .buttons.b1.w20plot <Leave> {.statusBar.lab config -text " "}
button .buttons.b1.input -text "Input Cursor Data" -width 15 \
	-state "disabled" -command {cursorInput}
bind .buttons.b1.input <Enter> \
	{.statusBar.lab config -text "Input cursor data"}
bind .buttons.b1.input <Leave> {.statusBar.lab config -text " "}
pack .buttons.b1.plot -side left -expand 1
pack .buttons.b1.w20plot -side left -expand 1
pack .buttons.b1.input -side left -expand 1

frame .buttons.b2
pack .buttons.b2 -side top -padx 4
button .buttons.b2.save -text "Save W20 Results" -width 15 \
	-command "ergFileWidget" -state "disabled"
bind .buttons.b2.save <Enter> \
	{.statusBar.lab config -text "Save refined profile parameters (W20)"}
bind .buttons.b2.save <Leave> {.statusBar.lab config -text " "}
button .buttons.b2.w20fit -text "Start W20" -width 15 \
	-command "w20fit \$inpFile(W20)" -state "disabled"
bind .buttons.b2.w20fit <Enter> {.statusBar.lab config \
	-text "Start profile refinement (W20)"}
bind .buttons.b2.w20fit <Leave> {.statusBar.lab config -text " "}
button .buttons.b2.simref -text "Start Simref" -width 15 \
	-command "simRefFit \$inpFile(Simref)" -state "disabled"
bind .buttons.b2.simref <Enter> {.statusBar.lab config \
	-text "Start Rietveld refinement (Simref)"}
bind .buttons.b2.simref <Leave> {.statusBar.lab config -text " "}
pack .buttons.b2.save -side left -expand 1
pack .buttons.b2.w20fit -side left -expand 1
pack .buttons.b2.simref -side left -expand 1

### Status Bar:

frame .statusBar
label .statusBar.lab -text "" -relief sunken -bd 1 \
    -font -*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-* -anchor w
label .statusBar.foo -textvariable diagramType -relief sunken -bd 1 \
    -font -*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-* -anchor w \
    -width 10
pack .statusBar.lab -side left -padx 2 -expand yes -fill both
pack .statusBar.foo -side left -padx 2
pack .statusBar -side top -fill x -pady 2

### busylock:

frame .busylock
bind .busylock <KeyPress> break
place .busylock -x -2 -y -2

#==============================================================================
#                  Koordinaten fuer die Anzeige berechnen
#==============================================================================

proc displayCoords {xc yc} {
    global xmin xmax ymin ymax diagramType diagramWidth diagramHeight

    set h1 30
    set b1 80
    set x0 [expr ($xc-$b1) / double($diagramWidth) * \
	    ($xmax($diagramType)-$xmin($diagramType)) + $xmin($diagramType)]
    set y0 [expr ($h1+$diagramHeight-$yc) / double($diagramHeight) * \
	    ($ymax($diagramType)-$ymin($diagramType)) + $ymin($diagramType)]
    .display.coords config -text "X, Y = \[$x0, $y0\]"

}

#==============================================================================
#          Koordinaten fuer Cursor-Eingabe berechnen
#==============================================================================

proc cursorCoords {xc yc} {
    global xmin xmax ymin ymax diagramType diagramWidth diagramHeight \
	zahlPos x_alt y_alt connect1 connect2 lineConnect

    if {[winfo exists .cursorInput] == 1} {
	set h1 30
	set b1 80
	set x0 [expr ($xc-$b1) / double($diagramWidth) * \
	    ($xmax($diagramType)-$xmin($diagramType)) + $xmin($diagramType)]
	set y0 [expr ($h1+$diagramHeight-$yc) / double($diagramHeight) * \
	    ($ymax($diagramType)-$ymin($diagramType)) + $ymin($diagramType)]
	incr zahlPos
	.cursorInput.frame.list insert end "$x0, $y0"
	if {$lineConnect == 1 && $zahlPos > 1} {
	    set connect2 \
	       [.diagram create line $x_alt $y_alt $xc $yc -width 1 -fill blue]
	    if {[info exist connect1] == 0} {set connect1 $connect2}
	}
	set x_alt $xc
	set y_alt $yc
    }

}

#==============================================================================
#                        Ausschnitt vergrern
#==============================================================================

proc setleft {xc yc} {
     global xmin xmax ymin ymax diagramType diagramWidth diagramHeight \
	     zahlPos x_alt y_alt connect1 connect2 lineConnect x_eins y_eins
    
    set h1 30
    set b1 80
    set x0 [expr ($xc-$b1) / double($diagramWidth) * \
	    ($xmax($diagramType)-$xmin($diagramType)) + $xmin($diagramType)]
    set y0 [expr ($h1+$diagramHeight-$yc) / double($diagramHeight) * \
	    ($ymax($diagramType)-$ymin($diagramType)) + $ymin($diagramType)]
    set x_eins $x0
    set y_eins $y0
}

proc setright {xc yc} {
 global xmin xmax ymin ymax diagramType diagramWidth diagramHeight \
	zahlPos x_alt y_alt connect1 connect2 lineConnect x_eins y_eins

    set h1 30
    set b1 80
    set x0 [expr ($xc-$b1) / double($diagramWidth) * \
	    ($xmax($diagramType)-$xmin($diagramType)) + $xmin($diagramType)]
    set y0 [expr ($h1+$diagramHeight-$yc) / double($diagramHeight) * \
	    ($ymax($diagramType)-$ymin($diagramType)) + $ymin($diagramType)]
    # Skalierung aufrufen
    if { $x_eins > $x0} {
	set thmin($diagramType) $x0
        set thmax($diagramType) $x_eins
	set xmin($diagramType) $thmin($diagramType)
	set xmax($diagramType) $thmax($diagramType)
    } else {
	set thmin($diagramType) $x_eins
	set thmax($diagramType) $x0
	set xmin($diagramType) $thmin($diagramType)
	set xmax($diagramType) $thmax($diagramType)
    }
    if {$y_eins > $y0} {
	set ymin($diagramType) $y0
	set ymax($diagramType) $y_eins
    } else {
	set ymin($diagramType) $y_eins
	set ymax($diagramType) $y0
    }
    busy_eval {diagram_redraw $diagramType}
}
proc  Plot_all {} {
    global diagramType bla_w20 fileSpec w20Minmax \
    xmin_all xmax_all ymin_all ymax_all xmin xmax thmin thmax ymin ymax
    if {$fileSpec == "SIM" || $fileSpec == "DAT"} {
	set xmin($diagramType) $xmin_all
	set xmax($diagramType) $xmax_all
	set thmin $xmin_all
	set thmax $xmax_all
#	busy_eval {w20Minmax}
        set ymin($diagramType) $ymin_all
        set ymax($diagramType) $ymax_all
    } else {
	set diagramType data 
	xMinmax $fileSpec
	yMinmax $fileSpec
    }
    busy_eval {diagram_redraw $diagramType}
     
}
#==============================================================================
#                        Select Plot File
#==============================================================================

proc selDataFile {} {
    global plotFileNr bla_w20 ble_w20 xmin xmax diagramType dataSetZahl \
	plotFile inpTitle dfile dtit datFile fileSpec

    set w .selDataFile
    catch {destroy $w}
    toplevel $w
    wm title $w "Plot Files"

    set pltNr $plotFileNr

# Datensaetze:

    frame $w.text
    label $w.text.label -text "Data Sets:" -anchor w
    pack $w.text.label -side left -padx 4
    pack $w.text -side top -fill x -pady 4

    frame $w.frame
    pack $w.frame -side top -expand yes -fill both
    scrollbar $w.frame.yscroll -command "$w.frame.list yview"
    scrollbar $w.frame.xscroll -orient horizontal \
	-command "$w.frame.list xview"
    listbox $w.frame.list -width 50 -height 7 -setgrid 1 \
	-yscroll "$w.frame.yscroll set" -xscroll "$w.frame.xscroll set"
    for {set i 1} {$i <= $dataSetZahl} {incr i} {
	$w.frame.list insert end $inpTitle($i)
    }
    set dfile $datFile($plotFileNr)
    set dtit $inpTitle($plotFileNr)
    $w.frame.list selection set [expr $plotFileNr-1] [expr $plotFileNr-1]
    bind $w.frame.list <ButtonRelease-1> {
	set plotFileNr [expr [.selDataFile.frame.list curselection] + 1]
	set dfile $datFile($plotFileNr)
	set dtit $inpTitle($plotFileNr)
    }

    pack $w.frame.yscroll -side right -fill y
    pack $w.frame.list -side top -expand 1 -fill both
    pack $w.frame.xscroll -side top -fill x

    frame $w.text2
    label $w.text2.label -text "Selected Data Set:" -anchor w
    pack $w.text2.label -side left -padx 4
    pack $w.text2 -side top -fill x -pady 4
    frame $w.title
    label $w.title.label -text "Title:" -anchor w
    label $w.title.name -textvariable dtit -anchor w
    pack $w.title.label $w.title.name -side left -padx 4
    pack $w.title -side top -fill x -pady 4
    frame $w.file
    label $w.file.label -text "File:" -anchor w
    label $w.file.name -textvariable dfile -anchor w
    pack $w.file.label $w.file.name -side left -padx 4
    pack $w.file -side top -fill x -pady 4

# Knoepfe:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.ok -text "Ok" -width 7 -command {
	    destroy .selDataFile
	    busy_eval {
		if {[file exists $plotFile($plotFileNr)] == 1} {
		    readPlotFile $plotFile($plotFileNr)
		    set xmin(W20) $bla_w20
		    set xmax(W20) $ble_w20
		    w20Minmax
		    readRefFile "[file rootname $plotFile($plotFileNr)].hkl"
                }
		if {[scanDataFile $datFile($plotFileNr)] >= 0} {
		    plotInit $fileSpec
		    readDataFile $datFile($plotFileNr)
		    .buttons.b1.w20plot configure -state "normal"
		    xMinmax $fileSpec
		    set thmin $xmin(data)
		    set thmax $xmax(data)
		    yMinmax $fileSpec
		    set diagramType data
		    diagram_redraw $diagramType
		}
	    }
    }
    button $w.buttons.cancel -text "Cancel" -width 7 \
	    -command "set plotFileNr \"$pltNr\"; destroy $w"
    pack $w.buttons.ok $w.buttons.cancel -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .selDataFile

}

#==============================================================================
#                          Error Bars
#==============================================================================

proc errorBars {} {
    global xErrorBars yErrorBars errorBarColor errorBarWidth diagramType \
	colorList colorListT

    set w .errorBars
    catch {destroy $w}
    toplevel $w
    wm title $w "Error Bars"

# Fehlerbalken:

    frame $w.x
    checkbutton $w.x.errbar -text "X Error Bars" \
	-variable xErrorBars -relief flat
    pack $w.x.errbar -side left -padx 4
    pack $w.x -side top -fill x -pady 4

    frame $w.y
    checkbutton $w.y.errbar -text "Y Error Bars" \
	-variable yErrorBars -relief flat
    pack $w.y.errbar -side left -padx 4
    pack $w.y -side top -fill x -pady 4

# Linien-Eigenschaften:

    frame $w.line -relief groove -borderwidth 2
    pack $w.line -side top -expand yes -pady 4 -padx 4 -fill both
    label $w.line.label -text "Line Properties:" -anchor w
    pack $w.line.label -side top -fill x -padx 4 -pady 4

# Farbe:

    frame $w.line.color
    label $w.line.color.label -text "Color:" -anchor w
    [eval tk_optionMenu $w.line.color.value errorBarColor $colorList] \
	configure -tearoff no
    $w.line.color.value config -anchor w
    pack $w.line.color.label -side left -padx 4
    pack $w.line.color.value -side right -padx 4
    pack $w.line.color -side top -fill x -pady 4

# Linienbreite:

    frame $w.line.width
    label $w.line.width.label -text "Width:" -anchor w
    entry $w.line.width.entry -textvariable errorBarWidth -width 8
    bind $w.line.width.entry <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.line.width.label -side left -padx 4
    pack $w.line.width.entry -side right -padx 4
    pack $w.line.width -side top -fill x -pady 4

# Knoepfe:

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command {
	destroy .errorBars
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.apply -text "Apply" -width 7 -command {
	busy_eval {diagram_redraw $diagramType}
    }
    pack $w.buttons.apply $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .errorBars

}

#==============================================================================
#         2-theta Winkelbereich fuer W20 einstellen
#==============================================================================

proc twoThRange {} {
    global thmin thmax

    set w .twoThRange
    catch {destroy $w}
    toplevel $w
    wm title $w "2-theta Range"

    set x1 $thmin
    set x2 $thmax

    frame $w.title
    label $w.title.label -text "2-theta Range in Degrees:" -anchor w
    pack $w.title.label -side left -padx 4
    pack $w.title -side top -fill x -pady 4

    frame $w.min
    label $w.min.label -text "Minimum:" -anchor w
    entry $w.min.value -width 10 -textvariable thmin
    bind $w.min.value <KeyPress> \
	{if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    pack $w.min.label -side left -padx 4
    pack $w.min.value -side right -padx 4
    pack $w.min -side top -fill x -pady 4

    frame $w.max
    label $w.max.label -text "Maximum:" -anchor w
    entry $w.max.value -width 10 -textvariable thmax
    bind $w.max.value <KeyPress> \
	{if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    pack $w.max.label -side left -padx 4
    pack $w.max.value -side right -padx 4
    pack $w.max -side top -fill x -pady 4

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

# Knoepfe:

    frame $w.buttons
    button $w.buttons.cancel -text "Cancel" -width 7 \
	-command "set thmin \"$x1\";
		set thmax \"$x2\";
		destroy $w;"
    button $w.buttons.ok -text "Ok" -width 7 -command "destroy $w"
    button $w.buttons.maximize -text "Maximize" -width 7 -command ""
    pack $w.buttons.ok $w.buttons.maximize $w.buttons.cancel \
	    -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .twoThRange

}

#==============================================================================
#               Simref-Plotfile einlesen und Extrema bestimmen
#==============================================================================

proc readPlotFile {file} {
    global y_obs y_calc y_diff y_back twoTheta_w20 ndata bla_w20 ble_w20\
	    ymin ymax xmin xmax ymin_all ymax_all xmax_all xmin_all thmin thmax

# Simref-Plotfile einlesen und Extrema bestimmen:

    set cmd {
	set fid [open $file r]
	gets $fid line
	set size [expr $size + [string length $line] + 1]
	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	gets $fid line
	set size [expr $size + [string length $line] + 1]
	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	set bla_w20 [lindex $line 1]
	set ble_w20 [lindex $line 3]
	set i 0
	while {[gets $fid line] >=0} {
	    set size [expr $size + [string length $line] + 1]
	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    if {[scan $line "%f %f %f %f %f" \
		    the yobs ycalc ydiff ybac] == 5} {
		set i [expr $i + 1]
		set twoTheta_w20($i) $the
		set y_obs($i) $yobs
		set y_calc($i) $ycalc
		set y_diff($i) $ydiff
		set y_back($i) $ybac
		if {$yobs > $ymax(w20)} {
		    set ymax($diagramType) $yobs
		}
		if {$ycalc > $ymax(w20)} {
		    set ymax($diagramType) $ycalc
		}
		if {$yobs < $ymin(w20)} {
		    set ymin($diagramType) $yobs
		}
		if {$ycalc < $ymin(w20)} {
		    set ymin($diagramType) $ycalc
		}
	    } else {
		break
	    }
	    update
	}
	set ndata $i
	close $fid
    }

    gaugeWidget "Reading Plot File"
    set size 0
    set maxSize [file size $file]
    if {[catch $cmd err] != 0} {
	tk_messageBox -icon error -type ok -title "Error" \
	    -message "Cannot read file:\n$file"
    }
    destroy .gauge

}

#==============================================================================
#        Minimum und Maximum der Y-Werte des Simref-Plotfiles bestimmen
#==============================================================================

proc w20Minmax {} {
    global xmin xmax ymin ymax twoTheta_w20 y_obs y_calc y_diff ndata\
	   xmin_all ymin_all ymax_all ymin_all 
    gaugeWidget "Setting Y Range"

    set n 0
    for {set i 1} {$i <= $ndata} {incr i} {
	if {$twoTheta_w20($i) >= $xmin(W20) && \
	    $twoTheta_w20($i) <= $xmax(W20)} {
	    set n [expr $n + 1]
	    if {$n > 1} {
		if {$y_obs($i) > $ymax(W20)} {
		    set ymax(W20) $y_obs($i)
		} elseif {$y_obs($i) < $ymin(W20)} {
		    set ymin(W20) $y_obs($i)
		}
		if {$y_calc($i) > $ymax(W20)} {
		    set ymax(W20) $y_calc($i)
		} elseif {$y_calc($i) < $ymin(W20)} {
		    set ymin(W20) $y_calc($i)
		}
	    } else {
		if {$y_obs($i) < $y_calc($i)} {
		    set ymin(W20) $y_obs($i)
		} else {
		    set ymin(W20) $y_calc($i)
		}
		set ymax(W20) $ymin(W20)
	    }
	    gauge_value .gauge.g [expr round($i * 100 / $ndata)]
	}
    }
#    set ymax_all $ymax(w20) 
#    set ymin_all $ymin(w20) 
#    set xmin_all $xmax(w20)
#    set xmax_all $xmax(w20)
    destroy .gauge

}

#==============================================================================
#                   Reflexlagen (Cursor-Positionen)
#==============================================================================

proc cursorInput {} {
    global zahlPos position fwhm intensity eta1 eta2 asymPar \
	refPos refFwhm refInt refEta1 refEta2 refAsym stddPos stddFwhm \
	stddInt stddEta1 stddEta2 stddAsym refZahl cursor1 lineConnect \
	connect1 connect2

    set w .cursorInput
    catch {destroy $w}
    toplevel $w
    wm title $w "Cursor Data"

    set zahlPos 0

    set cursor1 [.diagram cget -cursor]
    .diagram configure -cursor crosshair
    update

# Listbox der (x,y)-Werte:

    frame $w.titel
    label $w.titel.label -text "Values (2-theta, counts):" -anchor w
    pack $w.titel.label -side left -padx 4
    pack $w.titel -side top -fill x -pady 4

    frame $w.frame
    pack $w.frame -side top -expand yes -fill both

    scrollbar $w.frame.scroll -command "$w.frame.list yview"
    listbox $w.frame.list -yscroll "$w.frame.scroll set" -setgrid 1 -height 6
    pack $w.frame.scroll -side right -fill y
    pack $w.frame.list -side left -expand 1 -fill both

# Anzahl der Werte:

    frame $w.anzahl
    label $w.anzahl.label -text "Number of Values:" -anchor w
    label $w.anzahl.wert -textvariable zahlPos -anchor e -width 6 \
	    -relief sunken
    pack $w.anzahl.label -side left -padx 4
    pack $w.anzahl.wert -side left -padx 4
    pack $w.anzahl -side top -fill x -pady 4

# Werte mit einer Linie verbinden:

    frame $w.line
    checkbutton $w.line.draw -text "Connect selected Values" \
	-variable lineConnect -relief flat
    pack $w.line.draw -side left -padx 4
    pack $w.line -side top -fill x -pady 4

# Knoepfe:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command {
	    .diagram configure -cursor $cursor1
	    if {[info exists connect1] == 1} {
		for {set i $connect1} {$i <= $connect2} {incr i} {
		    .diagram delete $i
		}
		unset connect1
	    }
	    destroy .cursorInput
    }
    button $w.buttons.accept -text "Accept" -width 7 -command {
	    if {$zahlPos > 0} {
		for {set i 1} {$i <= $zahlPos} {incr i} { 
		    scan [.cursorInput.frame.list get [expr $i-1]] \
			"%f, %f" xPos yPos
		    set position($i) $xPos 
		    set fwhm($i) 0.1
		    set intensity($i) [expr $yPos * $fwhm($i)]
		    set eta1($i) 0.
		    set eta2($i) 0.
		    set asymPar($i) 0.
		    set refPos($i) 1
		    set refFwhm($i) 1
		    set refInt($i) 1
		    set refEta1($i) 0
		    set refEta2($i) 0
		    set refAsym($i) 0
		    set stddPos($i) 0.
		    set stddFwhm($i) 0.
		    set stddInt($i) 0.
		    set stddEta1($i) 0.
		    set stddEta2($i) 0.
		    set stddAsym($i) 0.
		}
		set refZahl $zahlPos
	    }
	    .diagram configure -cursor $cursor1
	    if {[info exists connect1] == 1} {
		for {set i $connect1} {$i <= $connect2} {incr i} {
		    .diagram delete $i
		}
		unset connect1
	    }
	    destroy .cursorInput
    }
    button $w.buttons.save -text "Save..." -command "saveCursorInput $w"
    pack $w.buttons.accept $w.buttons.save $w.buttons.dismiss \
	    -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

}

#==============================================================================
#              Untergrund-Daten in eine Datei abspeichern
#==============================================================================

proc saveCursorInput {w} {
    global datFile zahlPos

    set cursorFile "cursor.out"

    set types {
	{"Cursor Output Files"   {.out}}
	{"Reflection Data Files" {.hkl}}
	{"Background Data Files" {.bac}}
	{"All Files"                  *}
    }
    set file [tk_getSaveFile -filetypes $types -parent $w \
	-initialfile $cursorFile -title "Cursor Output File"]
	    
    if {$file != ""} {
	set fid [open $file w] 
	puts $fid "Cursor data of $datFile(1)"
	for {set i 1} {$i <= $zahlPos} {incr i} {
	    scan [.cursorInput.frame.list get [expr $i-1]] "%f, %f" xPos yPos
	    puts $fid "$xPos  $yPos"
	}
	close $fid
    }

}

#==============================================================================
#                      3D-Plot speichern 
#==============================================================================

proc saveDreide {fileName w} {
    global outFile outType device

    switch $outType {
	"PostScript" \
	    {set types { {"PostScript Files" {.ps} } 
			 {"All files"           *} } } 
	"Encapsulated PostScript" \
	    {set types { {"Encapsulated PostScript Files" {.eps} }
			 {"All files"           *} } }
	"HP Plot File" \
	    {set types { {"HP Plot Files"    {.hp} }
			 {"All files"           *} } }
	"PICASSO File" \
	    {set types { {"Picasso Files"   {.zwf} }
			 {"All files"           *} } }
    }

    switch $outType {
	"PostScript"              {set defExt ".ps"}
	"Encapsulated PostScript" {set defExt ".eps"}
	"HP Plot File"            {set defExt ".hp"}
	"PICASSO File"            {set defExt ".zwf"}
    }
    set file [tk_getSaveFile -filetypes $types -parent $w \
	-title "Save 3D Plot As..." \
	-initialfile $outFile -defaultextension $defExt]
    if {$file != ""} {
	set outFile $file
	set device $outType
	busy_eval {dreidePlot $fileName}
    }

}

#==============================================================================
#                       3D-Picasso-Plot starten
#==============================================================================

proc dreidePlot {file} {
    global xmin xmax tmin tmax outFile zoomfac velosity device \
	auge3D zScale3D zGrund3D zDach3D box3D line3D edge3D face3D dar3D

    set fid [open "| dreide" "w"]
    fconfigure $fid -buffering line

# Datenfile:

    puts $fid $file

# Minimum und Maximum in 2-Theta:

    puts $fid $xmin(data)
    puts $fid $xmax(data)

# Minimum und Maximum der Temperatur:

    puts $fid $tmin
    puts $fid $tmax

# Bloecke aequidistant darstellen:

    switch $dar3D {
	"Equidistant Blocks"                   {puts $fid 1}
	"Temperature Scaled and Sorted Blocks" {puts $fid 0}
	default                                {puts $fid 1}
    }

# Ausgabe-Device:
 
    switch $device { 
	"Screen"                  {puts $fid "1"}
	"Picasso File"            {puts $fid "3"}
	"HP Plot File"            {puts $fid "2"}
	"PostScript"              {puts $fid "4"}
	"Encapsulated Postscript" {puts $fid "5"}
	"Graphical Input"         {puts $fid "6"}
	default                   {puts $fid "4"}
    }

# Ausgabe-Datei:

    puts $fid $outFile

# Blickwinkel des Auges:

    puts $fid $auge3D(1)
    puts $fid $auge3D(2)
    puts $fid $auge3D(3)

# Z-Achse:

    puts $fid $zGrund3D
    puts $fid $zDach3D
    puts $fid $zScale3D

# Kasten, Netzlinien, Kanten:

    switch $box3D {
	"Unvisible"                      {puts $fid 0}
	"Visible"                        {puts $fid 1}
        "Visible, lower part hatched"    {puts $fid 2}
 	"Visible, superscribed, hatched" {puts $fid 3}
	default                          {puts $fid 0}
    }

    switch $line3D {
	"All Lines"                               {puts $fid 0}
	"Only Lines in x Direction (2-theta)"     {puts $fid 1}
        "Only lines in y Direction (Temperature)" {puts $fid 2}
	default                                   {puts $fid 1}
    }

    if {$face3D == "Unvisible"} {
	switch $edge3D {
	    "Draw all covered Edges"             {puts $fid 0}
	    "Suppress all covered Edges"         {puts $fid 1}
	    "Draw only covered Edges of the Box" {puts $fid 2}
	    default                              {puts $fid 2}
	}
    } elseif {$face3D == "Visible"} {
	switch $edge3D {
	    "Draw all covered Edges"             {puts $fid  0}
	    "Suppress all covered Edges"         {puts $fid -1}
	    "Draw only covered Edges of the Box" {puts $fid -2}
	    default                              {puts $fid -2}
	}
    }

# Zoomfaktor, Plotterstiftgeschwindigkeit:

    puts $fid $zoomfac
    puts $fid $velosity

    close $fid

}

#==============================================================================
#                         AboutBox
#==============================================================================

proc aboutBox {} {

    set w .aboutBox
    catch {destroy $w}
    toplevel $w
    wm title $w "About W20"

    after idle {
	update idletasks
	set x_max [winfo screenwidth .aboutBox]
	set y_max [winfo screenheight .aboutBox]
	set x [expr ($x_max-[winfo reqwidth .aboutBox])/2]
	set y [expr ($y_max-[winfo reqheight .aboutBox])/2]
	wm geometry .aboutBox "+$x+$y"
    }

    frame $w.f -bd 3 -relief raised
    pack $w.f -ipadx 4 -ipady 4
    message $w.m -aspect 1000 -justify left \
	    -text "W20\n\nVersion 3.0a (1999)\n\nHarald Ritter\nInstitut fuer Kristallographie\nder Universitaet Tuebingen\nCharlottenstrasse 33\nD-72070 Tuebingen\nGermany\n\nTel: +49(0)7071-2976389\nE-mail: harald.ritter@uni-tuebingen.de"
    pack $w.m -in $w.f -padx 4 -pady 4 -ipady 1

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -in $w.f -fill x -pady 4

    frame $w.buttons
    button $w.buttons.ok -text "Ok" -width 7 -command "destroy $w;"
    pack $w.buttons.ok -in $w.buttons -side left -expand yes -padx 4
    pack $w.buttons -in $w.f -side bottom -fill x -pady 4

    grab set .aboutBox
}

#==============================================================================
#              Eingabe des Dateinamens und Einlesen der Daten
#==============================================================================

proc openDataFile {w} {
    global fileSpec datFile blockNr countNr diagramType refFile \
	xmin xmax thmin thmax ref_Zahl ref_Theta ref_Fwhm refPhase phsZahl\
	ymin_all ymax_all ymin ymax

    set types {
	{"Data Files"		   {.dat}}
	{"DUO Data Files"          {.duo}}
	{"MONALISA Data Files"     {.scn}}
	{"Simref Plot Files"	   {.plt}}
	{"W20 Result Files"        {.erg}}
	{"All Files"		        *}
    }
    set file [tk_getOpenFile -filetypes $types -parent $w \
		-initialfile $datFile(1) -title "Data File"]

    if [string compare $file ""] {
	set datFile(1) $file
	set blockNr 1
	set countNr 1
	set diagramType data
	busy_eval {
	    if {[scanDataFile $datFile(1)] >= 0} {
		plotInit $fileSpec
		readDataFile $datFile(1)
		xMinmax $fileSpec
		if {$fileSpec=="SIM"} {
		    set ymax(data) $ymax_all
		    set ymin(data) $ymin_all
		} else {
		    yMinmax $fileSpec
		}		
                # yMinmax $fileSpec
		set thmin $xmin(data)
		set thmax $xmax(data)
		if {$fileSpec == "SIM"} {
		    set refFile "[file rootname $datFile(1)].hkl"
		    readhklFile $refFile
		}
		diagram_redraw $diagramType
	    }
	}
    }

}

#==============================================================================
#             Reflexlagen-Datei fuer Reflexmarkierungen einlesen
#==============================================================================

proc markRefWidget {} {
    global diagramType refFile ref_Zahl ref_Theta ref_Fwhm \
	    markRef markRefColor markRefWidth colorList

    set w .markRefWidget
    catch {destroy $w}
    toplevel $w
    wm title $w "Reflection Data"

# Datei mit Reflexlagen (*.hkl):

    frame $w.hklName
    label $w.hklName.label -text "File:" -anchor w
    label $w.hklName.file -textvariable refFile
    pack $w.hklName.label -side left -padx 4
    pack $w.hklName.file -side right -padx 4 -expand yes -fill x
    pack $w.hklName -side top -fill x -pady 4

# Listbox der (x,y)-Werte:

    frame $w.titel
    label $w.titel.label -text "Positions (2-theta), FWHM:" -anchor w
    pack $w.titel.label -side left -padx 4
    pack $w.titel -side top -fill x -pady 2

    frame $w.frame
    pack $w.frame -side top -expand yes -fill y -fill x -padx 4 -pady 4

    scrollbar $w.frame.scroll -command "$w.frame.list yview"
    listbox $w.frame.list -yscroll "$w.frame.scroll set" -setgrid 1 -height 6
    pack $w.frame.scroll -side right -fill y
    pack $w.frame.list -side left -expand 1 -fill both

    for {set i 1} {$i <= $ref_Zahl} {incr i} {
	$w.frame.list insert 0 "$ref_Theta($i), $ref_Fwhm($i)"
    }

# Anzahl der Werte:

    frame $w.anzahl
    label $w.anzahl.label -text "Number of Values:" -anchor w
    label $w.anzahl.wert -textvariable ref_Zahl -width 6 -anchor e \
	    -relief sunken
    pack $w.anzahl.label -side left -padx 4
    pack $w.anzahl.wert -side left -padx 4
    pack $w.anzahl -side top -fill x -pady 4

# Reflexe markieren:

    frame $w.reflection
    checkbutton $w.reflection.mark -text "Mark Reflection Positions" \
	-variable markRef -relief flat
    pack $w.reflection.mark -side left -padx 4
    pack $w.reflection -side top -fill x -pady 4

# Linien-Eigenschaften:

    frame $w.line -relief groove -borderwidth 2
    pack $w.line -side top -expand yes -pady 4 -padx 4 -fill both
    label $w.line.label -text "Line Properties:" -anchor w
    pack $w.line.label -side top -fill x -padx 4 -pady 4

# Farbe:

    frame $w.line.color
    label $w.line.color.label -text "Color:" -anchor w
    [eval tk_optionMenu $w.line.color.value markRefColor $colorList] \
	configure -tearoff no
    $w.line.color.value config -anchor w
    pack $w.line.color.label -side left -padx 4
    pack $w.line.color.value -side right -padx 4
    pack $w.line.color -side top -fill x -pady 4

# Linienbreite:

    frame $w.line.width
    label $w.line.width.label -text "Width:" -anchor w
    entry $w.line.width.entry -textvariable markRefWidth -width 8
    bind $w.line.width.entry <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.line.width.label -side left -padx 4
    pack $w.line.width.entry -side right -padx 4
    pack $w.line.width -side top -fill x -pady 4

# Knoepfe:

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -command {
	destroy .markRefWidget
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.apply -text "Apply" -command {
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.open -text "Open..." -command {
	set types {
	    {"Reflection Data Files" {.hkl}}
	    {"All Files"                  *}
	}
	set file [tk_getOpenFile -filetypes $types -parent $w \
	-initialfile $refFile -title "Reflection Data File"]
	if [string compare $file ""] {
	    set refFile $file
	    .markRefWidget.frame.list delete 0 end
	    set fid [open $refFile r] 
	    gets $fid
	    gets $fid
	    set ref_Zahl 0
	    while {[gets $fid line] >=0} {
		set the [lindex $line 3]
		set hwb [lindex $line 5]
		.markRefWidget.frame.list insert end "$the, $hwb"
		incr ref_Zahl
		set ref_Theta($ref_Zahl) $the
		set ref_Fwhm($ref_Zahl) $hwb
	    }
	    close $fid
	}
    }
    pack $w.buttons.open $w.buttons.apply $w.buttons.dismiss \
	-side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .markRefWidget
}

#==============================================================================
#                 Untergrund-Datei einlesen
#==============================================================================

proc backgroundData {} {
    global diagramType untFile unt_Zahl unt_Theta unt_Wert backgrDraw \
	backgrColor backgrWidth colorList

    set w .backgroundData
    catch {destroy $w}
    toplevel $w
    wm title $w "Background Data"

# Datei mit Untergrundwerten (*.bak):

    frame $w.untName
    label $w.untName.label -text "File:" -anchor w
    label $w.untName.file -textvariable untFile
    pack $w.untName.label -side left -padx 4
    pack $w.untName.file -side right -padx 4 -expand yes -fill x
    pack $w.untName -side top -fill x -pady 4

# Listbox der (x,y)-Werte:

    frame $w.titel
    label $w.titel.label -text "Positions (2-theta), Values:" -anchor w
    pack $w.titel.label -side left -padx 4
    pack $w.titel -side top -fill x -pady 2

    frame $w.frame
    pack $w.frame -side top -expand yes -fill y -fill x -padx 4 -pady 4

    scrollbar $w.frame.scroll -command "$w.frame.list yview"
    listbox $w.frame.list -yscroll "$w.frame.scroll set" -setgrid 1 -height 6
    pack $w.frame.scroll -side right -fill y
    pack $w.frame.list -side left -expand 1 -fill both

    for {set i 1} {$i <= $unt_Zahl} {incr i} {
	$w.frame.list insert 0 "$unt_Theta($i), $unt_Wert($i)"
    }

# Anzahl der Werte:

    frame $w.anzahl
    label $w.anzahl.label -text "Number of Values:" -anchor w
    label $w.anzahl.wert -textvariable unt_Zahl -anchor e -width 6 \
	    -relief sunken
    pack $w.anzahl.label -side left -padx 4
    pack $w.anzahl.wert -side left -padx 4
    pack $w.anzahl -side top -fill x -pady 4

# Untergrund ins Diagram einzeichnen:

    frame $w.backgr
    checkbutton $w.backgr.draw -text "Draw Background Data" \
	-variable backgrDraw -relief flat
    pack $w.backgr.draw -side left -padx 4
    pack $w.backgr -side top -fill x -pady 4

# Linien-Eigenschaften:

    frame $w.line -relief groove -borderwidth 2
    pack $w.line -side top -expand yes -pady 4 -padx 4 -fill both
    label $w.line.label -text "Line Properties:" -anchor w
    pack $w.line.label -side top -fill x -padx 4 -pady 4

# Farbe:

    frame $w.line.color
    label $w.line.color.label -text "Color:" -anchor w
    [eval tk_optionMenu $w.line.color.value backgrColor $colorList] \
	configure -tearoff no
    $w.line.color.value config -anchor w
    pack $w.line.color.label -side left -padx 4
    pack $w.line.color.value -side right -padx 4
    pack $w.line.color -side top -fill x -pady 4

# Linienbreite:

    frame $w.line.width
    label $w.line.width.label -text "Width:" -anchor w
    entry $w.line.width.entry -textvariable backgrWidth -width 8
    bind $w.line.width.entry <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.line.width.label -side left -padx 4
    pack $w.line.width.entry -side right -padx 4
    pack $w.line.width -side top -fill x -pady 4

# Knoepfe:

    frame $w.sep2 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep2 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -command {
	destroy .backgroundData
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.apply -text "Apply" -command {
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.open -text "Open..." -command {
	set types {
	    {"Background Data Files" {.bac}}
	    {"All Files"                  *}
	}
	set file [tk_getOpenFile -filetypes $types -parent $w \
	-initialfile $untFile -title "Background Data File"]
	if [string compare $file ""] {
	    set untFile $file
	    .backgroundData.frame.list delete 0 end
	    set fid [open $untFile r] 
	    gets $fid
	    set unt_Zahl 0
	    while {[gets $fid line] >=0} {
		if {$line == "excluded regions"} {break}
		set n [expr [llength $line] - 1]
		set the [lindex $line 0]
		set bac [lindex $line $n]
		.backgroundData.frame.list insert end "$the, $bac"
		incr unt_Zahl
		set unt_Theta($unt_Zahl) $the
		set unt_Wert($unt_Zahl) $bac
	    }
	    close $fid
	}
    }
    pack $w.buttons.open $w.buttons.apply $w.buttons.dismiss \
	-side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .backgroundData
}

#==============================================================================
#                  Daten im freien Spaltenformat plotten
#==============================================================================

proc colPlotPar {} {
    global colName colZahl absNr xAchse xAxisLabel symbolShape symbolList \
	symbolColor symbolFillColor colorList symbolSize lineColor lineWidth \
	lineDraw selectValue symShape symColor symFillColor symSize \
	lnColor lnWidth lnDraw selectNr fileSpec

    set w .colPlotPar
    catch {destroy $w}
    toplevel $w
    wm title $w "Symbols and Lines"

# Knoepfe:

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command {
	for {set i 1} {$i <= $colZahl} {incr i} {
	    if {$colName($i) == $selectValue} {
		set symbolShape($i) $symShape
		set symbolColor($i) $symColor
		set symbolFillColor($i) $symFillColor
		set symbolSize($i) $symSize
		set lineDraw($i) $lnDraw
		set lineColor($i) $lnColor
		set lineWidth($i) $lnWidth
		break
	    }
	}
	destroy .colPlotPar
	xMinmax $fileSpec
	yMinmax $fileSpec
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.apply -text "Apply" -command {
	for {set i 1} {$i <= $colZahl} {incr i} {
	    if {$colName($i) == $selectValue} {
		set symbolShape($i) $symShape
		set symbolColor($i) $symColor
		set symbolFillColor($i) $symFillColor
		set symbolSize($i) $symSize
		set lineDraw($i) $lnDraw
		set lineColor($i) $lnColor
		set lineWidth($i) $lnWidth
		break
	    }
	}
	xMinmax $fileSpec
	yMinmax $fileSpec
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.axes -text "Axes" -command "xyAxes"
    button $w.buttons.errbars -text "Error Bars" -command "errorBars"
    pack $w.buttons.apply $w.buttons.axes $w.buttons.errbars \
	$w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

# Linie:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -side bottom -fill x -pady 4

# Linker und rechter Rahmen:

    frame $w.left
    frame $w.right
    pack $w.left $w.right -side left -expand yes -pady 4 -padx 4 -fill both
    pack $w.right -side left -expand yes -fill both

# Listbox der Spalten:

    set selectValue $colName(1)
    set selectNr 1

    frame $w.left.column
    label $w.left.column.label -text "Selected Column:" -anchor w
    pack $w.left.column.label -side left -padx 4
    pack $w.left.column -side top -fill x -pady 4

    frame $w.left.select
    label $w.left.select.label -textvariable selectValue \
	-anchor w -relief sunken
    pack $w.left.select.label -side left -fill x -expand yes -padx 2
    pack $w.left.select -side top -fill x -pady 4

    frame $w.left.titel
    label $w.left.titel.label -text "Columns:" -anchor w
    pack $w.left.titel.label -side left -padx 4
    pack $w.left.titel -side top -fill x -pady 4

    frame $w.left.frame
    pack $w.left.frame -side top -expand yes -fill both
    scrollbar $w.left.frame.scroll -command "$w.left.frame.list yview"
    listbox $w.left.frame.list -yscroll "$w.left.frame.scroll set" -setgrid 1
    bind $w.left.frame.list <ButtonPress-1> {
	for {set i 1} {$i <= $colZahl} {incr i} {
	    if {$colName($i) == $selectValue} {
		set symbolShape($i) $symShape
		set symbolColor($i) $symColor
		set symbolFillColor($i) $symFillColor
		set symbolSize($i) $symSize
		set lineDraw($i) $lnDraw
		set lineColor($i) $lnColor
		set lineWidth($i) $lnWidth
		break
	    }
	}
    }
    bind $w.left.frame.list <ButtonRelease-1> {
	set selectValue [selection get]
	for {set i 1} {$i <= $colZahl} {incr i} {
	    if {$colName($i) == $selectValue} {
		set selectNr $i
		set symShape $symbolShape($i)
		set symColor $symbolColor($i)
		set symFillColor $symbolFillColor($i)
		set symSize $symbolSize($i)
		set lnDraw $lineDraw($i)
		set lnColor $lineColor($i)
		set lnWidth $lineWidth($i)
		break
	    }
	}
    }
    pack $w.left.frame.scroll -side right -fill y
    pack $w.left.frame.list -side left -expand 1 -fill both

    for {set i 1} {$i <= $colZahl} {incr i} {
	.colPlotPar.left.frame.list insert end $colName($i)
    }
    .colPlotPar.left.frame.list selection set 0

# X-Achse:

    frame $w.left.xAchse
    label $w.left.xAchse.label -text "X-Axis:" -anchor w
    menubutton $w.left.xAchse.spalten -textvariable xAchse -indicatoron 1 \
	-menu $w.left.xAchse.spalten.menu \
	-relief raised -bd 2 -highlightthickness 2 -anchor c
    menu $w.left.xAchse.spalten.menu -tearoff 0
    for {set i 1} {$i <= $colZahl} {incr i} {
	$w.left.xAchse.spalten.menu add command -label $colName($i) \
	-command "[list set xAchse $colName($i)];
                  [list set absNr $i];
		  [list set xAxisLabel $colName($i)]"
    }
    $w.left.xAchse.spalten config -anchor w 
    pack $w.left.xAchse.label -side left -padx 4
    pack $w.left.xAchse.spalten -side left -padx 4
    pack $w.left.xAchse -side top -fill x -pady 4 

    frame $w.right.symbol -relief groove -borderwidth 2
    frame $w.right.line -relief groove -borderwidth 2
    pack $w.right.symbol -side top -expand yes -pady 4 -padx 4 -fill both
    pack $w.right.line -side top -expand yes -pady 4 -padx 4 -fill both

# Symbole und Linien:

    set symShape $symbolShape(1)
    set symColor $symbolColor(1)
    set symFillColor $symbolFillColor(1)
    set symSize $symbolSize(1)
    set lnDraw $lineDraw(1)
    set lnColor $lineColor(1)
    set lnWidth $lineWidth(1)

# Symbol properties:

    label $w.right.symbol.titel -text "Symbol Properties:" -anchor w
    pack $w.right.symbol.titel -side top -fill x -padx 4 -pady 4

    frame $w.right.symbol.shape
    label $w.right.symbol.shape.label -text "Shape:" -anchor w
    [eval tk_optionMenu $w.right.symbol.shape.value symShape \
	$symbolList] configure -tearoff no
    $w.right.symbol.shape.value config -anchor w
    pack $w.right.symbol.shape.label -side left -padx 4
    pack $w.right.symbol.shape.value -side right -padx 4
    pack $w.right.symbol.shape -side top -fill x -pady 4

    frame $w.right.symbol.color
    label $w.right.symbol.color.label -text "Color:" -anchor w
    [eval tk_optionMenu $w.right.symbol.color.value symColor \
	$colorList] configure -tearoff no
    $w.right.symbol.color.value config -anchor w
    pack $w.right.symbol.color.label -side left -padx 4
    pack $w.right.symbol.color.value -side right -padx 4
    pack $w.right.symbol.color -side top -fill x -pady 4

    frame $w.right.symbol.fill
    label $w.right.symbol.fill.label -text "Fill Color:" -anchor w
    [eval tk_optionMenu $w.right.symbol.fill.value symFillColor \
	$colorList] configure -tearoff no
    $w.right.symbol.fill.value config -anchor w
    pack $w.right.symbol.fill.label -side left -padx 4
    pack $w.right.symbol.fill.value -side right -padx 4
    pack $w.right.symbol.fill -side top -fill x -pady 4

    frame $w.right.symbol.size
    label $w.right.symbol.size.label -text "Size:" -anchor w
    entry $w.right.symbol.size.value -textvariable symSize -width 8
    bind $w.right.symbol.size.value <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.right.symbol.size.label -side left -padx 4
    pack $w.right.symbol.size.value -side right -padx 4
    pack $w.right.symbol.size -side top -fill x -pady 4
   
# Line properties:

    label $w.right.line.titel -text "Line Properties:" -anchor w
    pack $w.right.line.titel -side top -fill x -padx 4 -pady 4

    frame $w.right.line.draw
    checkbutton $w.right.line.draw.button -text "Draw Line" \
	    -variable lnDraw -relief flat
    pack $w.right.line.draw.button -side left -padx 4
    pack $w.right.line.draw -side top -fill x -pady 4

    frame $w.right.line.color
    label $w.right.line.color.label -text "Color:" -anchor w
    [eval tk_optionMenu $w.right.line.color.value lnColor $colorList] \
	configure -tearoff no
    $w.right.line.color.value config -anchor w
    pack $w.right.line.color.label -side left -padx 4
    pack $w.right.line.color.value -side right -padx 4
    pack $w.right.line.color -side top -fill x -pady 4

    frame $w.right.line.width
    label $w.right.line.width.label -text "Width:" -anchor w
    entry $w.right.line.width.value -textvariable lnWidth -width 8
    bind $w.right.line.width.value <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.right.line.width.label -side left -padx 4
    pack $w.right.line.width.value -side right -padx 4
    pack $w.right.line.width -side top -fill x -pady 4

    grab set .colPlotPar
}

#==============================================================================
#                      Daten im freien Spaltenformat
#==============================================================================

proc colFormat {} {
    global colZahl colName xAchse symbol lineStyle lineType groe \
	absNr datFile fileSpec symList conList kurvList xAxisLabel

    set w .colFormat
    catch {destroy $w}
    toplevel $w
    wm title $w "Columns"

    label $w.ytitel -text "Y-Axis:"
    pack $w.ytitel -anchor w -padx 4

    frame $w.sep1 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep1 -fill x -pady 4

    scrollform_create $w.sform
    pack $w.sform -expand yes -fill both

    set form [scrollform_interior $w.sform]

    for {set i 1} {$i <= $colZahl} {incr i} {
	set s $form.spalte$i
	frame $s
	label $s.label -text "$colName($i):" -anchor w
	pack $s.label -side left -padx 4 
	[eval tk_optionMenu $s.kurve lineType($i) $kurvList] configure \
	    -tearoff no
	$s.kurve config -anchor w
	pack $s.kurve -side right -padx 4
	[eval tk_optionMenu $s.line lineStyle($i) $conList] configure \
	    -tearoff no
	$s.line config -anchor w
	pack $s.line -side right -padx 4
	[eval tk_optionMenu $s.symbol symbol($i) $symList] configure \
	    -tearoff no
	$s.symbol config -anchor w
	pack $s.symbol -side right -padx 4
	pack $s -side top -fill x -pady 4
    }

# X-Achse:

    frame $w.sep2 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep2 -fill x -pady 4

    set a $w.xAchse
    frame $a
    label $a.label -text "X-Axis:" -anchor w
    menubutton $a.spalten -textvariable xAchse -indicatoron 1 \
    -menu $a.spalten.menu \
    -relief raised -bd 2 -highlightthickness 2 -anchor c
    menu $a.spalten.menu -tearoff 0
    for {set i 1} {$i <= $colZahl} {incr i} {
	$a.spalten.menu add command -label $colName($i) \
	-command "[list set xAchse $colName($i)];
                  [list set absNr $i];
		  [list set xAxisLabel $colName($i)]"
    }
    $a.spalten config -anchor w 
    pack $a.label -side left -padx 4
    pack $a.spalten -side left -padx 4
    pack $a -side top -fill x -pady 4 

# Buttons:

    frame $w.sep3 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep3 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.cancel -text "Cancel" -width 7 -command "destroy $w"
    button $w.buttons.ok -text "Ok" -width 7 -command {
	busy_eval {
	    xMinmax $fileSpec
	    yMinmax $fileSpec
	}
	destroy .colFormat
    }
    pack $w.buttons.cancel $w.buttons.ok -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .colFormat
}

#==============================================================================
#                             Plot-Parameter
#==============================================================================

proc output {file} {
    set w .output
    catch {destroy $w}
    toplevel $w
    wm title $w "Output"

    global zoomfac velosity picfont outType info outFile

# Name des Ausgabefiles:

    frame $w.ergName
    label $w.ergName.label -text "File:" -anchor w
    label $w.ergName.entry -textvariable outFile -width 25 \
	    -relief sunken -anchor w
    pack $w.ergName.label -side left -padx 4
    pack $w.ergName.entry -side right -padx 4
    pack $w.ergName -side top -fill x -pady 4

# Filetyp:

    frame $w.device
    label $w.device.label -text "File Type:" -anchor w
    tk_optionMenu $w.device.option outType \
	    "PostScript" \
	    "Encapsulated PostScript" \
	    "HP Plot File" \
	    "Picasso File" 
    $w.device.option config -anchor w
    pack $w.device.label -side left -padx 4
    pack $w.device.option -side right -padx 4
    pack $w.device -side top -fill x -pady 4

# Zoomfaktor:

    frame $w.zoomFac
    label $w.zoomFac.label -text "Zoom Factor:" -anchor w
    entry $w.zoomFac.entry -textvariable zoomfac -width 8
    bind $w.zoomFac.entry <KeyPress> \
	{if {![regexp "\[0-9\t\b.\]" "%A"]} {bell;break}}
    pack $w.zoomFac.label -side left -padx 4
    pack $w.zoomFac.entry -side right -padx 4
    pack $w.zoomFac -side top -fill x -pady 4

# Plotterstift-Geschwindigkeit:

    frame $w.penVelo
    label $w.penVelo.label -text "Pen Velosity:" -anchor w
    entry $w.penVelo.entry -textvariable velosity -width 5
    bind $w.penVelo.entry <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.penVelo.label -side left -padx 4
    pack $w.penVelo.entry -side right -padx 4
    pack $w.penVelo -side top -fill x -pady 4

# Nummer des Picasso-Fonts:

    frame $w.picFont
    label $w.picFont.label -text "Picasso Font No.:" -anchor w
    entry $w.picFont.entry -textvariable picfont -width 5
    bind $w.picFont.entry <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.picFont.label -side left -padx 4
    pack $w.picFont.entry -side right -padx 4
    pack $w.picFont -side top -fill x -pady 4

# Info-Box:

    frame $w.infoBox
    label $w.infoBox.label -text "Plot Date and File Name:"
    tk_optionMenu $w.infoBox.option info "Yes" "No"
    $w.infoBox.option config -anchor w
    pack $w.infoBox.label -side left -padx 4
    pack $w.infoBox.option -side right -padx 4
    pack $w.infoBox -side top -fill x -pady 4

# Buttons:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -command "destroy $w"
    button $w.buttons.save -text "Save..." -command "picSave $file $w"
    pack $w.buttons.save $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .output
}

#==============================================================================
#                Datei Untergrund-Datei fuer W20-Fit
#==============================================================================

proc backgrFile {} {
    global bacFile

    set w .backgrFile
    catch {destroy $w}
    toplevel $w
    wm title $w "Background Data File"

    set back $bacFile

# Name der Datei:

    frame $w.bacName
    label $w.bacName.label -text "Name:" -anchor w
    entry $w.bacName.entry -textvariable bacFile -width 35 
    pack $w.bacName.label -side left -padx 4
    pack $w.bacName.entry -side right -padx 4
    pack $w.bacName -side top -fill x -pady 4

# Buttons:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.cancel -text "Cancel" -width 7 \
	-command "set bacFile \"$back\"; 
		  destroy $w"
    button $w.buttons.ok -text "Ok" -width 7 -command "destroy $w"
    button $w.buttons.browse -text "Browse..." -command {
	set types {
	    {"Background Data Files" {.bac}}
	    {"All Files"                  *}
	}
	set bacFile [tk_getOpenFile -filetypes $types -parent $w \
	    -initialfile $bacFile -title "Background Data File"]
    }
    pack $w.buttons.cancel $w.buttons.browse $w.buttons.ok \
	-side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .backgrFile
}

#==============================================================================
#                       Datensatz-Parameter
#==============================================================================

proc showDataPar {} {
    set w .showDataPar
    catch {destroy $w}
    toplevel $w
    wm title $w "Data Set"

    global fileSpec inpType zeit bla step ble temp bnr izahl \
	titel titelZahl countDist

    set b $w.titel
    frame $b
    scrollbar $b.xsbar -orient horizontal \
	-command {.showDataPar.titel.text xview}
    scrollbar $b.ysbar -orient vertical \
	-command {.showDataPar.titel.text yview}
    text $b.text -width 30 -height 5 -wrap none \
	-xscrollcommand {.showDataPar.titel.xsbar set} \
	-yscrollcommand {.showDataPar.titel.ysbar set}
    .showDataPar.titel.text delete 1.0 end
    for {set i 1} {$i <= $titelZahl} {incr i} {
	.showDataPar.titel.text insert end "$titel($i)\n"
    }
    pack $b.xsbar -side bottom -fill x
    pack $b.ysbar -side right -fill y
    pack $b.text -side left -expand yes -fill both
    pack $b  -expand yes -fill both -padx 4 -pady 4

    set b $w.fileType
    frame $b
    label $b.label -text "File Type:" -width 15 -anchor w
    label $b.wert -text $inpType -width 35 -anchor w -relief sunken
    pack $b.label $b.wert -side left -padx 4
    pack $b -side top -fill x -pady 4

    set b $w.bereich
    frame $b
    label $b.label1 -text "Minimum:" -width 15 -anchor w
    label $b.wert1 -textvariable bla -width 8 -anchor w -relief sunken
    label $b.label2 -text "Maximum:" -width 15 -anchor w
    label $b.wert2 -textvariable ble -width 8 -anchor w -relief sunken

    pack $b.label1 $b.wert1 -side left -padx 4
    pack $b.label2 $b.wert2 -side left -padx 4
    pack $b -side top -fill x -pady 4

    set b $w.stepTemp
    frame $b
    label $b.label1 -text "Stepwidth:" -width 15 -anchor w
    label $b.wert1 -textvariable step -width 8 -anchor w -relief sunken
    label $b.label2 -text "Temperature:" -width 15 -anchor w
    label $b.wert2 -textvariable temp -width 8 -anchor w -relief sunken
    pack $b.label1 $b.wert1 -side left -padx 4
    pack $b.label2 $b.wert2 -side left -padx 4
    pack $b -side top -fill x -pady 4

    set b $w.block
    frame $b
    label $b.label1 -text "Block No.:" -width 15 -anchor w
    label $b.wert1 -textvariable bnr -width 8 -anchor w -relief sunken
    label $b.label2 -text "Number of Blocks:" -width 15 -anchor w
    label $b.wert2 -textvariable blockZahl -width 8 -anchor w -relief sunken
    pack $b.label1 $b.wert1 -side left -padx 4
    pack $b.label2 $b.wert2 -side left -padx 4

    pack $b -side top -fill x -pady 4

    set b $w.numZeit
    frame $b
    label $b.label1 -text "Number of Data:" -width 15 -anchor w
    label $b.wert1 -textvariable izahl -width 8 -anchor w -relief sunken
    label $b.label2 -text "Time/Step:" -width 15 -anchor w
    label $b.wert2 -textvariable zeit -width 8 -anchor w -relief sunken
    pack $b.label1 $b.wert1 -side left -padx 4
    pack $b.label2 $b.wert2 -side left -padx 4
    pack $b -side top -fill x -pady 4


    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command "destroy $w"
    pack $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .showDataPar
}

#==============================================================================
#                        Title of the Plot
#==============================================================================

proc picTitle {} {
    global plotTitle diagramType

    set w .picTitle
    catch {destroy $w}
    toplevel $w
    wm title $w "Title"

    set text $plotTitle

    frame $w.text
    label $w.text.label -text "Title:" -anchor w
    entry $w.text.entry -width 40 -textvariable plotTitle
    pack $w.text.label $w.text.entry -side left -padx 4
    pack $w.text -side top -fill x  -pady 4

# Knoepfe:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.cancel -text "Cancel" -width 7 \
	    -command "set plotTitle \"$text\"; destroy $w"
    button $w.buttons.ok -text "Ok" -width 7 -command {
	    destroy .picTitle
	    busy_eval {diagram_redraw $diagramType}
    }
    pack $w.buttons.cancel $w.buttons.ok -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .picTitle
}

#==============================================================================
#                       Canvas-Einstellungen
#==============================================================================

proc canvasWidget {} {
    global diagramColor colorList

    set w .canvasWidget
    catch {destroy $w}
    toplevel $w
    wm title $w "Canvas"

# Hintergrundfarbe:

    frame $w.back
# -relief groove -borderwidth 2
    pack $w.back -side top -expand yes -pady 4 -padx 4 -fill both
    frame $w.back.color
    label $w.back.color.label -text "Background Color:" -anchor w
    [eval tk_optionMenu $w.back.color.value diagramColor $colorList] \
	configure -tearoff no
    $w.back.color.value config -anchor w
    pack $w.back.color.label -side left -padx 4
    pack $w.back.color.value -side right -padx 4
    pack $w.back.color -side top -fill x -pady 4

# Knoepfe:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.apply -text "Apply" -width 7 -command {
	.diagram configure -background $diagramColor
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command {
	destroy .canvasWidget
	.diagram configure -background $diagramColor
	busy_eval {diagram_redraw $diagramType}
    }
    pack $w.buttons.apply $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .canvasWidget
}

#==============================================================================
#                         X and Y Axis
#==============================================================================

proc xyAxes {} {
    global xAxisLabel yAxisLabel axesColor axesWidth colorList

    set w .xyAxes
    catch {destroy $w}
    toplevel $w
    wm title $w "Axes"

# X-Achsen-Beschriftung:

    frame $w.x
    label $w.x.label -text "X Label:"  -anchor w
    entry $w.x.entry -width 20 -textvariable xAxisLabel 
    pack $w.x.label -side left -padx 4
    pack $w.x.entry -side right -padx 4
    pack $w.x -side top -pady 4 -fill x

# X-Achsen-Beschriftung:

    frame $w.y
    label $w.y.label -text "Y Label:"  -anchor w
    entry $w.y.entry -width 20 -textvariable yAxisLabel 
    pack $w.y.label -side left -padx 4
    pack $w.y.entry -side right -padx 4
    pack $w.y -side top -pady 4 -fill x

# Linien-Eigenschaften:

    frame $w.line -relief groove -borderwidth 2
    pack $w.line -side top -expand yes -pady 4 -padx 4 -fill both
    label $w.line.label -text "Line Properties:" -anchor w
    pack $w.line.label -side top -fill x -padx 4 -pady 4

# Farbe:

    frame $w.line.color
    label $w.line.color.label -text "Color:" -anchor w
    [eval tk_optionMenu $w.line.color.value axesColor $colorList] \
	configure -tearoff no
    $w.line.color.value config -anchor w
    pack $w.line.color.label -side left -padx 4
    pack $w.line.color.value -side right -padx 4
    pack $w.line.color -side top -fill x -pady 4

# Linienbreite:

    frame $w.line.width
    label $w.line.width.label -text "Width:" -anchor w
    entry $w.line.width.entry -textvariable axesWidth -width 8
    bind $w.line.width.entry <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.line.width.label -side left -padx 4
    pack $w.line.width.entry -side right -padx 4
    pack $w.line.width -side top -fill x -pady 4

# Knoepfe:

    frame $w.buttons
    button $w.buttons.apply -text "Apply" -width 7 -command {
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command {
	destroy .xyAxes
	busy_eval {diagram_redraw $diagramType}
    }
    pack $w.buttons.apply $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .xyAxes
}

#==============================================================================
#                          Voreinstellungen
#==============================================================================

proc settings {} {
    global profFunc gewicht winkelFehler basPeak cycZahl urf \
	glFwhm glEta1 glEta2 glAsym backMod kSplit \
	refZahl position fwhm intensity eta1 eta2 asymPar \
	refPos refFwhm refInt refEta1 refEta2 refAsym \
	stddPos stddFwhm stddInt stddEta1 stddEta2 stddAsym \
	refZahl_old

    set w .settings
    catch {destroy $w}
    toplevel $w
    wm title $w "Settings"

    set profkt $profFunc
    set mcycle $cycZahl
    set refZahl_old $refZahl
    set weight $gewicht
    set wdt $basPeak
    set glfw $glFwhm
    set gle1 $glEta1
    set gle2 $glEta2
    set glas $glAsym
    set bmod $backMod
    set dupl $kSplit
    set relax $urf
    set wf $winkelFehler

# Peak shape function:

    frame $w.prof
    label $w.prof.label -text "Peak Shape function:" -anchor w
    tk_optionMenu $w.prof.option profFunc \
	    "Gaussian" \
	    "Lorentzian" \
	    "Modified Lorentzian" \
	    "Intermediate Lorentzian" \
 	    "Edgeworth Serie" \
	    "Pseudo-Voigt" \
            "Pearson VII" \
	    "Double Gaussian"
    $w.prof.option config -anchor w
    pack $w.prof.label -side left -padx 4
    pack $w.prof.option -side right -padx 4
    pack $w.prof -side top -fill x -pady 4

# Weighting:

    frame $w.weight
    label $w.weight.label -text "Weights:" -anchor w
    tk_optionMenu $w.weight.option gewicht \
	    "1" \
	    "Use Values from Data File" \
	    "Method of Rebmann et al." \
	    "1/y_obs" \
    $w.weight.option config -anchor w
    pack $w.weight.label -side left -padx 4
    pack $w.weight.option -side right -padx 4
    pack $w.weight -side top -fill x -pady 4

# Winkelfehler der Rebmann'schen Gewichtung:

    frame $w.wf
    label $w.wf.label -text "Stdd. of angular Positions:" -anchor w
    entry $w.wf.entry -textvariable winkelFehler -width 8
    bind $w.wf.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.\]" "%A"]} {bell;break}}
    pack $w.wf.label -side left -padx 4
    pack $w.wf.entry -side right -padx 4
    pack $w.wf -side top -fill x -pady 4

# Base of Peak:

    frame $w.base
    label $w.base.label -text "Base of Peak:" -anchor w
    entry $w.base.entry -textvariable basPeak -width 8
    bind $w.base.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.\]" "%A"]} {bell;break}}
    pack $w.base.label -side left -padx 4
    pack $w.base.entry -side right -padx 4
    pack $w.base -side top -fill x -pady 4

# Relaxationsfaktor:

    frame $w.urf
    label $w.urf.label -text "Under-Relaxation Factor:" -anchor w
    entry $w.urf.entry -textvariable urf -width 8
    bind $w.urf.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.\]" "%A"]} {bell;break}}
    pack $w.urf.label -side left -padx 4
    pack $w.urf.entry -side right -padx 4
    pack $w.urf -side top -fill x -pady 4

# Zahl der Zyklen:

    frame $w.cycles
    label $w.cycles.label -text "Number of Cycles:" -anchor w
    entry $w.cycles.entry -textvariable cycZahl -width 8
    bind $w.cycles.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.cycles.label -side left -padx 4
    pack $w.cycles.entry -side right -padx 4
    pack $w.cycles -side top -fill x -pady 4

# Zahl der Reflexe:

    frame $w.numRef
    label $w.numRef.label -text "Number of Reflections:" -anchor w
    entry $w.numRef.entry -textvariable refZahl -width 8
    bind $w.numRef.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.numRef.label -side left -padx 4
    pack $w.numRef.entry -side right -padx 4
    pack $w.numRef -side top -fill x -pady 4

# Zwangsbedingungen der Parameter:

    frame $w.fwhm
    label $w.fwhm.label -text "FWHM Parameters:" -anchor w
    tk_optionMenu $w.fwhm.option glFwhm \
	    "individual" \
	    "common"
    $w.fwhm.option config -anchor w
    pack $w.fwhm.label -side left -padx 4
    pack $w.fwhm.option -side right -padx 4
    pack $w.fwhm -side top -fill x -pady 4

    frame $w.eta1
    label $w.eta1.label -text "Eta_1 Parameters:" -anchor w
    tk_optionMenu $w.eta1.option glEta1 \
	    "individual" \
	    "common"
    $w.eta1.option config -anchor w
    pack $w.eta1.label -side left -padx 4
    pack $w.eta1.option -side right -padx 4
    pack $w.eta1 -side top -fill x -pady 4

    frame $w.eta2
    label $w.eta2.label -text "Eta_2 Parameters:" -anchor w
    tk_optionMenu $w.eta2.option glEta2 \
	    "individual" \
	    "common"
    $w.eta2.option config -anchor w
    pack $w.eta2.label -side left -padx 4
    pack $w.eta2.option -side right -padx 4
    pack $w.eta2 -side top -fill x -pady 4

    frame $w.asym
    label $w.asym.label -text "Asymmetry Parameters:" -anchor w
    tk_optionMenu $w.asym.option glAsym \
	    "individual" \
	    "common"
    $w.asym.option config -anchor w
    pack $w.asym.label -side left -padx 4
    pack $w.asym.option -side right -padx 4
    pack $w.asym -side top -fill x -pady 4

# Untergrund-Modus:

    frame $w.backgr
    label $w.backgr.label -text "Background:" -anchor w
    tk_optionMenu $w.backgr.option backMod \
	    "Refinable linear Background" \
	    "Use Background Data File" \
	    "Use Values from Data File"
    $w.backgr.option config -anchor w
    pack $w.backgr.label -side left -padx 4
    pack $w.backgr.option -side right -padx 4
    pack $w.backgr -side top -fill x -pady 4

# K_alpha1/K_alpha2-Splitting:

    frame $w.split
    label $w.split.label -text "K_alpha1/K_alpha2 Splitting:" -anchor w
    tk_optionMenu $w.split.option kSplit \
	    "Yes" \
	    "No"
    $w.split.option config -anchor w
    pack $w.split.label -side left -padx 4
    pack $w.split.option -side right -padx 4
    pack $w.split -side top -fill x -pady 4

# Knoepfe:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.cancel -text "Cancel" -width 7 \
	-command "set profFunc \"$profkt\";
                  set basPeak \"$wdt\";
		  set cycZahl \"$mcycle\";
		  set refZahl \"$refZahl_old\";
		  set gewicht \"$weight\";
		  set glFwhm \"$glfw\";
		  set glEta1 \"$gle1\";
		  set glEta2 \"$gle2\";
		  set glAsym \"$glas\";
		  set backMod \"$bmod\";
		  set kSplit \"$dupl\";
		  set urf \"$relax\";
		  set winkelFehler \"$wf\";
		  destroy $w" 
    button $w.buttons.ok -text "Ok" -width 7 -command {
	    if {$refZahl > $refZahl_old} {
		for {set i [expr $refZahl_old + 1]} {$i <= $refZahl} {incr i} {
		    set position($i) 0.
		    set fwhm($i) 0.1
		    set intensity($i) 100.
		    set eta1($i) 0.
		    set eta2($i) 0.
		    set asymPar($i) 0.
		    set refPos($i) 1
		    set refFwhm($i) 1
		    set refInt($i) 1
		    set refEta1($i) 0
		    set refEta2($i) 0
		    set refAsym($i) 0
		    set stddPos($i) 0.
		    set stddFwhm($i) 0.
		    set stddInt($i) 0.
		    set stddEta1($i) 0.
		    set stddEta2($i) 0.
		    set stddAsym($i) 0.
		}
	    }
	    if {$refZahl > 1} {
		if {$glFwhm == "common"} {
		    for {set i 2} {$i <= $refZahl} {incr i} {
			set fwhm($i) $fwhm(1)
			set refFwhm($i) $refFwhm(1)
		    }
		}
		if {$glEta1 == "common"} {
		    for {set i 2} {$i <= $refZahl} {incr i} {
			set eta1($i) $eta1(1)
			set refEta1($i) $refEta1(1)
		    }
		}
		if {$glEta2 == "common"} {
		    for {set i 2} {$i <= $refZahl} {incr i} {
			set eta2($i) $eta2(1)
			set refEta2($i) $refEta2(1)
		    }
		}
		if {$glAsym == "common"} {
		    for {set i 2} {$i <= $refZahl} {incr i} {
			set asymPar($i) $asymPar(1)
			set refAsym($i) $refAsym(1)
		    }
		}
            }
	    destroy .settings
    }
    pack $w.buttons.cancel $w.buttons.ok -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .settings
}

#==============================================================================
#                        Reflexparameter
#==============================================================================

proc reflexPar {} {
    global position fwhm intensity eta1 eta2 asymPar refZahl \
	refPos refFwhm refInt refEta1 refEta2 refAsym reflexNr \
	par refPar wert r_wert stdd glFwhm glEta1 glEta2 glAsym \
	stddPos stddFwhm stddInt stddEta1 stddEta2 stddAsym

    set w .reflexPar
    catch {destroy $w}
    toplevel $w
    wm title $w "Reflection"

    for {set i 1} {$i <= $refZahl} {incr i} {
	set par($i,1) $position($i)
	set par($i,2) $fwhm($i)
	set par($i,3) $intensity($i)
	set par($i,4) $eta1($i)
	set par($i,5) $eta2($i)
	set par($i,6) $asymPar($i)
	set refPar($i,1) $refPos($i)
	set refPar($i,2) $refFwhm($i)
	set refPar($i,3) $refInt($i)
	set refPar($i,4) $refEta1($i)
	set refPar($i,5) $refEta2($i)
	set refPar($i,6) $refAsym($i)
    }
    set reflexNr 1
    set wert(1) $position(1)
    set wert(2) $fwhm(1)
    set wert(3) $intensity(1)
    set wert(4) $eta1(1)
    set wert(5) $eta2(1)
    set wert(6) $asymPar(1)
    set r_wert(1) $refPar(1,1)
    set r_wert(2) $refPar(1,2)
    set r_wert(3) $refPar(1,3)
    set r_wert(4) $refPar(1,4)
    set r_wert(5) $refPar(1,5)
    set r_wert(6) $refPar(1,6)
    set stdd(1) $stddPos(1)
    set stdd(2) $stddFwhm(1)
    set stdd(3) $stddInt(1)
    set stdd(4) $stddEta1(1)
    set stdd(5) $stddEta2(1)
    set stdd(6) $stddAsym(1)

    frame $w.reflexe
    label $w.reflexe.label -text "Reflection:" -width 10 -anchor w
    menubutton $w.reflexe.nummer -textvariable reflexNr -indicatoron 1 \
	-menu $w.reflexe.nummer.menu \
	-relief raised -bd 2 -highlightthickness 2 -anchor c
    menu $w.reflexe.nummer.menu -tearoff 0
    for {set i 1} {$i <= $refZahl} {incr i} {
	$w.reflexe.nummer.menu add command -label $i \
	    -command "set reflexNr $i;
		      set wert(1) \$par($i,1);
		      set wert(2) \$par($i,2);
		      set wert(3) \$par($i,3);
		      set wert(4) \$par($i,4);
		      set wert(5) \$par($i,5);
		      set wert(6) \$par($i,6);
		      set r_wert(1) \$refPar($i,1);
		      set r_wert(2) \$refPar($i,2);
		      set r_wert(3) \$refPar($i,3);
		      set r_wert(4) \$refPar($i,4);
		      set r_wert(5) \$refPar($i,5);
		      set r_wert(6) \$refPar($i,6);
		      set stdd(1) \$stddPos($i);
		      set stdd(2) \$stddFwhm($i);
		      set stdd(3) \$stddInt($i);
		      set stdd(4) \$stddEta1($i);
		      set stdd(5) \$stddEta2($i);
		      set stdd(6) \$stddAsym($i)"
    }
    $w.reflexe.nummer config -width 5 -anchor w
    pack $w.reflexe.label $w.reflexe.nummer -side left -padx 4
    pack $w.reflexe -side top -fill x -pady 4

    frame $w.sep1 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep1 -fill x -pady 4

# Reflexlage:

    frame $w.position
    checkbutton $w.position.chk -text "Position (2-theta):" \
	-variable r_wert(1) -relief flat -anchor w
    bind $w.position.chk <Leave> {set refPar($reflexNr,1) $r_wert(1)}
    entry $w.position.entry -textvariable wert(1) -width 10
    bind $w.position.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    bind $w.position.entry <Leave> {set par($reflexNr,1) $wert(1)}
    label $w.position.plusmin -text "+-" -anchor w 
    label $w.position.stdd -textvariable stdd(1) -anchor e -width 10 \
	    -relief sunken 
    pack $w.position.chk -side left -padx 4
    pack $w.position.stdd $w.position.plusmin $w.position.entry \
	-side right -padx 4
    pack $w.position -side top -fill x -pady 4

# Halbwertsbreite:

    frame $w.fwhm
    checkbutton $w.fwhm.chk -text "FWHM (2-theta):" \
	-variable r_wert(2) -relief flat -anchor w
    bind $w.fwhm.chk <Leave> {
	    if {$glFwhm == "common"} {
		for {set i 1} {$i <= $refZahl} {incr i} {
		    set refPar($i,2) $r_wert(2)
		}
	    } else {
		set refPar($reflexNr,2) $r_wert(2)
	    }
    }
    entry $w.fwhm.entry -textvariable wert(2) -width 10
    bind $w.fwhm.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.\]" "%A"]} {bell;break}}
    bind $w.fwhm.entry <Leave> {
	    if {$glFwhm == "common"} {
		for {set i 1} {$i <= $refZahl} {incr i} {
		    set par($i,2) $wert(2)
		}
	    } else {
		set par($reflexNr,2) $wert(2)
	    }
    }
    label $w.fwhm.plusmin -text "+-" -anchor w 
    label $w.fwhm.stdd -textvariable stdd(2) -anchor e -width 10 \
	    -relief sunken 
    pack $w.fwhm.chk -side left -padx 4
    pack $w.fwhm.stdd $w.fwhm.plusmin $w.fwhm.entry -side right -padx 4
    pack $w.fwhm -side top -fill x -pady 4

# Integrale Intensitaet:

    frame $w.intensity
    checkbutton $w.intensity.chk -text "Intensity:" \
	-variable r_wert(3) -relief flat -anchor w
    bind $w.intensity.chk <Leave> {set refPar($reflexNr,3) $r_wert(3)}
    entry $w.intensity.entry -textvariable wert(3) -width 10
    bind $w.intensity.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.\]" "%A"]} {bell;break}}
    bind $w.intensity.entry <Leave> {set par($reflexNr,3) $wert(3)}
    label $w.intensity.plusmin -text "+-" -anchor w 
    label $w.intensity.stdd -textvariable stdd(3) -anchor e -width 10 \
	    -relief sunken 
    pack $w.intensity.chk -side left -padx 4
    pack $w.intensity.stdd $w.intensity.plusmin $w.intensity.entry \
	    -side right -padx 4
    pack $w.intensity -side top -fill x -pady 4

# Parameter der Profilfunktion (eta1):

    frame $w.eta1
    checkbutton $w.eta1.chk -text "Eta_1:" \
	-variable r_wert(4) -relief flat -anchor w
    bind $w.eta1.chk <Leave> {
	    if {$glEta1 == "common"} {
		for {set i 1} {$i <= $refZahl} {incr i} {
		    set refPar($i,4) $r_wert(4)
		}
	    } else {
		set refPar($reflexNr,4) $r_wert(4)
	    }
    }
    entry $w.eta1.entry -textvariable wert(4) -width 10
    bind $w.eta1.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    bind $w.eta1.entry <Leave> {
	    if {$glEta1 == "common"} {
		for {set i 1} {$i <= $refZahl} {incr i} {
		    set par($i,4) $wert(4)
		}
	    } else {
		set par($reflexNr,4) $wert(4)
	    }
    }
    label $w.eta1.plusmin -text "+-" -anchor w 
    label $w.eta1.stdd -textvariable stdd(4) -anchor e -width 10 \
	    -relief sunken 
    pack $w.eta1.chk -side left -padx 4
    pack $w.eta1.stdd $w.eta1.plusmin $w.eta1.entry -side right -padx 4
    pack $w.eta1 -side top -fill x -pady 4

# Parameter der Profilfunktion (eta2):

    frame $w.eta2
    checkbutton $w.eta2.chk -text "Eta_2:" \
	-variable r_wert(5) -relief flat -anchor w
    bind $w.eta2.chk <Leave> {
	    if {$glEta2 == "common"} {
		for {set i 1} {$i <= $refZahl} {incr i} {
		    set refPar($i,5) $r_wert(5)
		}
	    } else {
		set refPar($reflexNr,5) $r_wert(5)
	    }
    }
    entry $w.eta2.entry -textvariable wert(5) -width 10
    bind $w.eta2.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    bind $w.eta2.entry <Leave> {
	    if {$glEta2 == "common"} {
		for {set i 1} {$i <= $refZahl} {incr i} {
		    set par($i,5) $wert(5)
		}
	    } else {
		set par($reflexNr,5) $wert(5)
	    }
    }
    label $w.eta2.plusmin -text "+-" -anchor w 
    label $w.eta2.stdd -textvariable stdd(5) -anchor e -width 10 \
	    -relief sunken 
    pack $w.eta2.chk -side left -padx 4
    pack $w.eta2.stdd $w.eta2.plusmin $w.eta2.entry -side right -padx 4
    pack $w.eta2 -side top -fill x -pady 4

# Asymmetrie-Parameter:

    frame $w.asym
    checkbutton $w.asym.chk -text "Asymmetry Parameter:" \
	-variable r_wert(6) -relief flat -anchor w
    bind $w.asym.chk <Leave> {
	    if {$glAsym == "common"} {
		for {set i 1} {$i <= $refZahl} {incr i} {
		    set refPar($i,6) $r_wert(6)
		}
	    } else {
		set refPar($reflexNr,6) $r_wert(6)
	    }
    }
    entry $w.asym.entry -textvariable wert(6) -width 10
    bind $w.asym.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    bind $w.asym.entry <Leave> {
	    if {$glAsym == "common"} {
		for {set i 1} {$i <= $refZahl} {incr i} {
		    set par($i,6) $wert(6)
		}
	    } else {
		set par($reflexNr,6) $wert(6)
	    }
    }
    label $w.asym.plusmin -text "+-" -anchor w 
    label $w.asym.stdd -textvariable stdd(6) -anchor e -width 10 \
	    -relief sunken 
    pack $w.asym.chk -side left -padx 4
    pack $w.asym.stdd $w.asym.plusmin $w.asym.entry -side right -padx 4
    pack $w.asym -side top -fill x -pady 4

# Knoepfe:

    frame $w.sep2 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep2 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.cancel -text "Cancel" -width 7 \
	-command "destroy $w"
    button $w.buttons.ok -text "Ok" -width 7 -command {
	for {set i 1} {$i <= $refZahl} {incr i} {
	    set position($i) $par($i,1)
	    set fwhm($i) $par($i,2)
	    set intensity($i) $par($i,3)
	    set eta1($i) $par($i,4)
	    set eta2($i) $par($i,5)
	    set asymPar($i) $par($i,6)
	    set refPos($i) $refPar($i,1)
	    set refFwhm($i) $refPar($i,2)
	    set refInt($i) $refPar($i,3)
	    set refEta1($i) $refPar($i,4)
	    set refEta2($i) $refPar($i,5)
	    set refAsym($i) $refPar($i,6)
	}
	destroy .reflexPar
    }
    pack $w.buttons.cancel $w.buttons.ok -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .reflexPar
}

#==============================================================================
#                    Globale Parameter
#==============================================================================

proc globalPar {} {
    global ratio lambda refRatio back refBack stddBack datFile

    set w .globalPar
    catch {destroy $w}
    toplevel $w
    wm title $w "Global Parameters"

    set ra $ratio
    set reRa $refRatio
    set wl(1) $lambda(1)
    set wl(2) $lambda(2)
    set bk(1) $back(1)
    set bk(2) $back(2)
    set reBk(1) $refBack(1)
    set reBk(2) $refBack(2)

# Background Parameters:

    frame $w.titel1
    label $w.titel1.label -text "Background:" -anchor w
    pack $w.titel1.label -side left -padx 4
    pack $w.titel1 -side top -fill x -pady 4

    frame $w.b0
    checkbutton $w.b0.chk -text "B0 (constant):" \
	    -variable refBack(1) -relief flat -anchor w
    entry $w.b0.ent -textvariable back(1) -width 10
    bind $w.b0.ent <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    label $w.b0.plusmin -text "+-" -anchor w 
    label $w.b0.stdd -textvariable stddBack(1) -anchor e -width 10 \
	    -relief sunken
    pack $w.b0.chk -side left -padx 4
    pack $w.b0.stdd $w.b0.plusmin $w.b0.ent -side right -padx 4
    pack $w.b0 -side top -fill x -pady 4

    frame $w.b1
    checkbutton $w.b1.chk -text "B1 (slope):" \
	    -variable refBack(2) -relief flat -anchor w
    entry $w.b1.ent -textvariable back(2) -width 10
    bind $w.b1.ent <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    label $w.b1.lab -text " " -width 5 -anchor w
    label $w.b1.plusmin -text "+-" -anchor w 
    label $w.b1.stdd -textvariable stddBack(2) -anchor e -width 10 \
	    -relief sunken
    pack $w.b1.chk -side left -padx 4
    pack $w.b1.stdd $w.b1.plusmin $w.b1.ent -side right -padx 4
    pack $w.b1 -side top -fill x -pady 4

    frame $w.sep1 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep1 -fill x -pady 4

# Intensitaetsverhaeltnis:

    frame $w.titel2
    label $w.titel2.label -text "Intensity Ratio K_alpha2/K_alpha1:" -anchor w
    pack $w.titel2.label -side left -padx 4
    pack $w.titel2 -side top -fill x -pady 4

    frame $w.ratio
    checkbutton $w.ratio.chk -text "I(K_alpha2) / I(K_alpha1):" \
	-variable refRatio -relief flat -anchor w
    entry $w.ratio.ent -textvariable ratio -width 10
    bind $w.ratio.ent <KeyPress> \
	{if {![regexp "\[0-9\t\b.\]" "%A"]} {bell;break}}
    label $w.ratio.plusmin -text "+-" -anchor w 
    label $w.ratio.stdd -textvariable stddRatio -anchor e -width 10 \
	    -relief sunken
    pack $w.ratio.chk -side left -padx 4
    pack $w.ratio.stdd $w.ratio.plusmin $w.ratio.ent -side right -padx 4
    pack $w.ratio -side top -fill x -pady 4

# Wellenlaengen:

    frame $w.sep2 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep2 -fill x -pady 4

    frame $w.titel3
    label $w.titel3.label -text "Wavelengths:" -anchor w
    pack $w.titel3.label -side left -padx 4
    pack $w.titel3 -side top -fill x -pady 4

    frame $w.lambda
    label $w.lambda.lab1 -text "K_alpha1:" -anchor w
    label $w.lambda.lab2 -text "K_alpha2:" -anchor w
    entry $w.lambda.ent1 -textvariable lambda(1) -width 10
    bind $w.lambda.ent1 <KeyPress> \
	    {if {![regexp "\[0-9\t\b.\]" "%A"]} {bell;break}}
    entry $w.lambda.ent2 -textvariable lambda(2) -width 10
    bind $w.lambda.ent2 <KeyPress> \
	    {if {![regexp "\[0-9\t\b.\]" "%A"]} {bell;break}}
    pack $w.lambda.lab1 $w.lambda.ent1 $w.lambda.lab2 $w.lambda.ent2 \
	    -side left -padx 4
    pack $w.lambda -side top -fill x -pady 8

# Knoepfe:

    frame $w.sep3 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep3 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.cancel -text "Cancel" -width 7 \
	-command "set ratio \"$ra\";
		  set refRatio \"$reRa\";
		  set lambda(1) \"$wl(1)\";
		  set lambda(2) \"$wl(2)\";
		  set back(1) \"$bk(1)\";
		  set back(2) \"$bk(2)\";
		  set refBack(1) \"$reBk(1)\";
		  set refBack(2) \"$reBk(2)\";
		  destroy $w"
    button $w.buttons.ok -text "Ok" -width 7 -command "destroy $w"
    pack $w.buttons.cancel $w.buttons.ok -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .globalPar
}

#==============================================================================
#                             Convert-Eingabe
#==============================================================================

proc convWidget {} {
    global datFile conMode conType conFile conMoni fileSpec

    set w .convWidget
    catch {destroy $w}
    toplevel $w
    wm title $w "Convert"

# Ausgabe-Datei:

    set conFile [file rootname $datFile(1)]

    frame $w.titel
    label $w.titel.label -text "Output Filename without Extension:" -anchor w
    pack $w.titel.label -side left -padx 4
    pack $w.titel -side top -fill x -pady 4

    frame $w.file
    entry $w.file.entry -textvariable conFile -width 35
    pack $w.file.entry -side left -padx 4 -expand yes -fill x
    pack $w.file -side top -fill x -pady 4

# Altes oder neues Format:

    frame $w.sep1 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep1 -fill x -pady 4

    frame $w.text
    label $w.text.label -text "Output File Format:" -anchor w
    pack $w.text.label -side left -padx 4
    pack $w.text -side top -fill x -pady 4

    set f $w.ftyp
    frame $f
    pack $f -side top -fill x -padx 4 -pady 4
    frame $f.type
    pack $f.type -side top -fill x
    radiobutton $f.type.t1 -variable conType -relief flat -value 1 \
	    -text "Old DUO File Format"
    radiobutton $f.type.t2 -variable conType -relief flat -value 2 \
	    -text "Simref DUO File Format"
    radiobutton $f.type.t3 -variable conType -relief flat -value 3 \
	    -text "Simref Data File Format"
    radiobutton $f.type.t4 -variable conType -relief flat -value 4 \
	    -text "Simref Neutron Data File Format"
    pack $f.type.t1 $f.type.t2 $f.type.t3 $f.type.t4 \
	    -side top -pady 2 -anchor w

# Ausgabe-Modus:

    frame $w.sep2 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep2 -fill x -pady 4

    set f $w.fmod
    frame $f
    pack $f -side top -fill x -padx 4 -pady 4
    frame $f.modus
    pack $f.modus -side top -fill x
    radiobutton $f.modus.m1 -variable conMode -relief flat -value 1 \
	    -text "All Blocks in one File"
    radiobutton $f.modus.m2 -variable conMode -relief flat -value 2 \
	    -text "All Blocks in separate Files"
    pack $f.modus.m1 $f.modus.m2 -side top -pady 2 -anchor w

# Normierung auf mittlere Monitorrate:

    if {$fileSpec == "SCN" || $fileSpec == "BEN"} {
	frame $w.monitor
	pack $w.monitor -side top -fill x -padx 4 -pady 10
	pack $w.monitor -side top -fill x
	checkbutton $w.monitor.m1 -variable conMoni -relief flat \
		-text "Scaling on Monitor Counting Rate"
	pack $w.monitor.m1 -side top -pady 2 -anchor w
    } else {
	set conMoni 0
    }

# Knoepfe:

    frame $w.sep3 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep3 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command "destroy $w"
    button $w.buttons.convert -text "Convert" -width 7 -command { \
	gaugeWidget "Convert Data"
	busy_eval {convertData $datFile(1)}
	destroy .gauge
    }
    pack $w.buttons.convert $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .convWidget
}

#==============================================================================
#                         Daten konvertieren
#==============================================================================

proc convertData {fileName} {
    global old blockZahl countZahl countDist titel titelZahl conType conFile \
	conMode conMoni fileSpec mitMonitor phoZahl izahl bla step ble zeit \
	spur_pho temp_pho y_anfang y_step y_ende

# Extension und Dateikennung:

    if {$conType == 1 || $conType == 2} {
	set filspe "DUO"
	set extension "duo"
    } elseif {$conType == 3} {
	set filspe "DAT"
	set extension "dat"
    } elseif {$conType == 4} {
	set filspe "NEU"
	set extension "neu"
    }

# Datenfile oeffnen und Titel ueberlesen:

    set maxSize [file size $fileName]
    set size 0
    set inp [open $fileName "r"]
    for {set i 1} {$i <= $titelZahl} {incr i} {
	gets $inp line
	set size [expr $size + [string length $line] + 1]
	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
    }
    if {$old == 1} {
	if {$fileSpec == "PHO"} {
	    for {set i 1} {$i <= [expr $phoZahl + 2]} {incr i} {
		gets $inp line
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]	
            }
        } else {
	    gets $inp line
	    set size [expr $size + [string length $line] + 1]
	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]	
        }
    }

# Schleife ueber alle Bloecke:

    for {set blockNr 1} {$blockNr <= $blockZahl} {incr blockNr} {

# Block einlesen:

	if {$fileSpec == "DUO"} {
	    gets $inp line
	    set size [expr $size + [string length $line] + 1]
	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    if {$old == 1} {
		gets $inp line
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		scan $line "%f %f %f %f %f %d %d" \
		    zeit bla step ble temp bnr izahl
	    } else {
		scan $line "%*s %f %f %f %f %f %d %d" \
		    zeit bla step ble temp bnr izahl
            }
	    set i 0 
	    while {[gets $inp line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		set diff [expr $izahl - $i]
		if {$diff < 8} {
		    set num $diff
		} else {
		    set num 8
                }
		set anfang 0
		set ende 7
		for {set n 1} {$n <= $num} {incr n} {
		    incr i
		    set daten(1,$i) \
			[expr double([string range $line $anfang $ende])]
		    incr anfang 8
		    incr ende 8
		}
		update
		if {$diff < 8} {break}
	    }

	} elseif {$fileSpec == "DAT"} {
	    if {$blockNr == 1} {
		gets $inp line
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    }
	    scan $line "%*s %f %f %f %f %f %d %d" \
		    zeit bla step ble temp bnr izahl
	    set i 0
	    while {[gets $fid line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {[scan $line "%f %f %f %f" the yobs ysig ybac] == 4} {
		    set i [expr $i + 1]
		    set daten(1,$i) $the
		    set daten(2,$i) $yobs
		    set daten(3,$i) $ysig
		    set daten(4,$i) $ybac
		} else {
		    break
		}
		update
	    }
	    set izahl $i
	    set bla $daten(1,1)
	    set ble $daten(1,$izahl)

	} elseif {$fileSpec == "NEU" || $fileSpec == "GRE" || \
		    $fileSpec == "DMC"} {
	    gets $inp line
	    set size [expr $size + [string length $line] + 1]
	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    if {$fileSpec == "GRE"} {
		gets $inp line
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		scan $line "%f %f %f %f %f %d %d" \
		    zeit bla step ble temp idum izahl
	    } elseif {$fileSpec == "DMC"} {
		gets $inp line
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		scan $line "%f %f %f" bla step ble
		set zeit 0.
		set temp 0.
		set izahl [expr round(($ble-$bla) / $step) + 1]
	    } else {
		scan $line "%*s %f %f %f %f %f %d %d" \
		    zeit bla step ble temp bnr izahl
            }
	    set i 0
	    while {[gets $inp line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		set diff [expr $izahl - $i]
		if {$diff < 10} {
		    set num $diff
		} else {
		    set num 10
                }
		set anfang 2
		set ende 7
		for {set n 1} {$n <= $num} {incr n} {
		    incr i
		    set daten(1,$i) \
			[expr double([string range $line $anfang $ende])]
		    incr anfang 8
		    incr ende 8
		}
		update
		if {$diff < 10} {break}
	    }

	} elseif {$fileSpec == "HAS"} {
	    gets $inp line
	    set size [expr $size + [string length $line] + 1]
	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    gets $inp line
	    set size [expr $size + [string length $line] + 1]
	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    scan $line "%f %f %f %f %d" temp bla step ble bnr
	    set i 0 
	    while {[gets $inp line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {[scan $line "%f %f %f %f" y1 y2 y3 the] == 4} {
		    set i [expr $i + 1]
		    set daten(1,$i) $y1
		    set daten(2,$i) $y2
		    set daten(3,$i) $y3
		} else {
		    break
		}
		update
	    }
	    set izahl $i
	    set ble $the
	    set zeit 0.

	} elseif {$fileSpec == "SCN"} {
	    if {$blockNr == 1} {
		gets $inp line
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    }
	    scan $line "%*s %f %f %f %f %d" bla step ble temp izahl
	    set i 0
	    while {[gets $inp line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {[scan $line "%f %f %f %f %*f" the y1 y2 y3] == 4} {
		    set i [expr $i + 1]
		    set daten(1,$i) $y1
		    set daten(2,$i) $y2
		    set daten(3,$i) $y3
		    set mitMonitor [expr $mitMonitor + ($daten(3,$i) / $izahl)]
		}
		update
	    }
	    set ble $the
	    set zeit 0.

	} elseif {$fileSpec == "BEN"} {
	    if {$blockNr == 1} {
		gets $inp line
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    }
	    gets $inp line
	    set size [expr $size + [string length $line] + 1]
	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    scan $line "%f %f %f %f %f %f" zeit bla step ble temp t
	    set i 0
	    while {[gets $inp line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		if {[scan $line "%f %f %f" y1 y2 y3] == 3} {
		    set i [expr $i + 1]
		    set daten(1,$i) $y1
		    set daten(2,$i) $y2
		    set daten(3,$i) $y3
		    set mitMonitor [expr $mitMonitor + ($daten(3,$i) / $izahl)]
		} else {
		    break
		}
		update
	    }
	    set izahl $i
	    set ble [expr $bla + ($izahl-1) * $step]

	} elseif {$fileSpec == "PHO"} {
	    if {$phoZahl == 0} {
		set temp [expr double($blockNr)]
	    } elseif {$phoZahl == 1} {
		set temp $temp_pho(1)
	    } else {
		set ymm [expr $y_anfang + ($blockNr-1) * $y_step]
		if {$ymm <= $spur_pho(1)} {
		    set temp [yskal $temp_pho(1) $temp_pho(2) $spur_pho(1) \
				    $spur_pho(2) $ymm]
		} else {
		    if {$ymm >= $spur_pho($phoZahl)} {
			set i1 [expr $phoZahl - 1]
			set temp [yskal $temp_pho($i1) $temp_pho($phoZahl) \
				    $spur_pho($i1) $spur_pho($phoZahl) $ymm]
		    } else {
			for {set i 1} {$i < $phoZahl} {incr i} {
			    set i1 [expr $i + 1]
			    if {$ymm > $spur_pho($i) && $ymm <= $spur_pho($i1)} {
				set temp [yskal $temp_pho($i) $temp_pho($i1) \
					    $spur_pho($i) $spur_pho($i1) $ymm]
			    }
			}
		    }
		}
	    }
	    set i 0 
	    while {[gets $inp line] >=0} {
		set size [expr $size + [string length $line] + 1]
		gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
		set diff [expr $izahl - $i]
		if {$diff < 16} {
		    set num $diff
		} else {
		    set num 16
                }
		set anfang 0
		set ende 4
		for {set n 1} {$n <= $num} {incr n} {
		    incr i
		    set daten(1,$i) \
			[expr double([string range $line $anfang $ende])]
		    incr anfang 5
		    incr ende 5
		}
		update
		if {$diff < 16} {break}
	    }
	
	}

# Schleife ueber alle Zaehler:

	for {set k 1} {$k <= $countZahl} {incr k} {

# Name der Ausgabedatei bestimmen:

	    if {$countZahl == 1} {
		if {$conMode == 1 || $blockZahl == 1} {
		    set file ""
		    append file $conFile "." $extension
		} else {
		    set file ""
		    append file $conFile "_" $blockNr "." $extension
		}
	    } else {
		if {$conMode == 1 || $blockZahl == 1} {
		    set file ""
		    append file $conFile "_" $k "." $extension
		} else {
		    set file ""
		    append file $conFile "_"  $blockNr "_" $k "." $extension
		}
	    }

# Titel in Ausgabedatei schreiben:
 
	    if {$blockNr == 1 || $conMode == 2} {
		set fid [open $file "w"]
		for {set j 1} {$j <= $titelZahl} {incr j} {
		    puts $fid $titel($j)
		}
		if {$conType == 1} {puts $fid "TITELENDE     $filspe"}
		close $fid
	    }

# Blockanfang und Blockende bestimmen:

	    if {$fileSpec == "BEN" && $k == 2} {
		set blockanfang [expr $bla + $countDist]
		set blockende [expr $ble + $countDist]
	    } elseif {$fileSpec == "SCN" && $k == 1} {
		set blockanfang [expr $bla + $countDist]
		set blockende [expr $ble + $countDist]
	    } else {
		set blockanfang $bla
		set blockende $ble
	    }

# Ausgabedatei oeffnen und Kopf schreiben:

	    set fid [open $file a]
	    if {$conType == 1} {
		puts $fid "B"
		puts $fid [format "%12.3f%12.5f%12.6f%12.5f%12.4f%4d%8d" \
		    $zeit $blockanfang $step $blockende $temp $blockNr $izahl]
	    } else {
		puts $fid [format "%s %12.3f%12.5f%12.6f%12.5f%12.4f%4d%8d" \
		    $filspe $zeit $blockanfang $step $blockende $temp \
		    $blockNr $izahl]
	    }

# Daten im DUO-Format schreiben:

	    if {$conType == 1 || $conType == 2} {
		set j 1
		for {set n 1} {$n <= $izahl} {incr n} {
		    if {$fileSpec == "DAT"} {
			set yobs $daten(2,$n)
		    } else {
			set yobs $daten($k,$n)
			if {$fileSpec == "BEN" || $fileSpec == "SCN"} {
			    if {$conMoni == 1 && $k <= 2} {
				set yobs \
				[expr $yobs * $minMonitor / $daten(3,$n)]
			    }
			}
		    }
		    if {$j == 8 || $n == $izahl} {
			puts $fid [format "%6.0f. " $yobs]
			set j 1
		    } else {
			puts -nonewline $fid [format "%6.0f. " $yobs]
			set j [expr $j + 1]
		    }
		    update
		}

# Daten im Simref-Datenformat schreiben:

	    } elseif {$conType == 3} {
		for {set n 1} {$n <= $izahl} {incr n} {
		    if {$fileSpec == "DAT"} {
			set the $daten(1,$n)
			set yobs $daten(2,$n)
			set ysig $daten(3,$n)
			set ybac $daten(4,$n)
		    } else {
			set the [expr $bla + ($n-1) * $step]
			set yobs $daten($k,$n)
			if {$fileSpec == "BEN" || $fileSpec == "SCN"} {
			    if {$conMoni == 1 && $k <= 2} {
			     set yobs [expr $yobs * $minMonitor / $daten(3,$n)]
			    }
			}
			set ysig [expr sqrt($yobs)]
			set ybac 0.
		    }
		    puts $fid [format "%12.5f %14.0f. %14.2f %14.2f" \
			$the $yobs $ysig $ybac]
		    update
		}

# Daten im Simref Neutronendaten-Format schreiben:

	    } elseif {$conType == 4} {
		set j 1
		for {set n 1} {$n <= $izahl} {incr n} {
		    set gew 1
		    if {$fileSpec == "DAT"} {
			set yobs $daten(2,$n)
		    } else {
			set yobs $daten($k,$n)
			if {$fileSpec == "BEN" || $fileSpec == "SCN"} {
			    if {$conMoni == 1 && $k <= 2} {
			     set yobs [expr $yobs * $minMonitor / $daten(3,$n)]
			    }
			}
		    }
		    if {$j == 10 || $n == $izahl} {
			puts $fid [format "%2d%6.0f" $gew $yobs]
			set j 1
		    } else {
			puts -nonewline $fid [format "%2d%6.0f" $gew $yobs]
			set j [expr $j + 1]
		    }
		    update
		}
 	    }

# Ausgabedatei schliessen:

	    close $fid

	}
    }

# Datenfile schliessen:

    close $inp 

}

proc yskal {y1 y2 x1 x2 x} {

# yskal plaziert einen zu x gehoerenden wert yskal auf eine
# Gerade, die durch zwei Punkte (x1,y1) und (x2,y2) definiert ist
# oder yskal skaliert x auf die y-Skala.

# y1,y2: Minimum/Maximum der y-Skala
# x1,x2: Minimum/Maximum der x-Skala

      set skal [expr ($x-$x1) * ($y2-$y1) / double(($x2-$x1)) + $y1]

      return $skal

}

#==============================================================================
#                       Bloecke aufsummieren (Merge)
#==============================================================================

proc mergeWidget {} {
    global datFile mergFile

    set w .mergeWidget
    catch {destroy $w}
    toplevel $w
    wm title $w "Merge HASYLAB Data"

# Ausgabe-Datei:

    set mergFile ""
    append mergFile [file rootname $datFile(1)] ".sum"

    frame $w.file
    label $w.file.label -text "Output File:" -anchor w
    entry $w.file.entry -textvariable mergFile -width 35
    pack $w.file.label -side left -padx 4
    pack $w.file.entry -side left -padx 4 -expand yes -fill x
    pack $w.file -side top -fill x -pady 4

# Knoepfe:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command "destroy $w"
    button $w.buttons.merge -text "Merge" -width 7 -command { \
	busy_eval {mergeData $datFile(1) $mergFile}
    }
    button $w.buttons.browse -text "Browse..." -command {
	set types {
	    {"Merged Data Files" {.sum}}
	    {"All Files"              *}
	}
	set mergFile [tk_getOpenFile -filetypes $types -parent $w \
	    -initialfile $mergFile -title "Output File"]
    }
    pack $w.buttons.merge $w.buttons.browse $w.buttons.dismiss \
	    -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .mergeWidget
}

#==============================================================================
#                       Fortschrittsanzeige
#==============================================================================

proc gaugeWidget {titel} {

    set w .gauge
    catch {destroy .gauge}
    toplevel .gauge
    wm title .gauge $titel

    option add *Gauge.borderWidth 2 widgetDefault
    option add *Gauge.relief sunken widgetDefault
    option add *Gauge.length 200 widgetDefault
    option add *Gauge.color gray widgetDefault

    after idle {
	update idletasks
	set x_max [winfo screenwidth .gauge]
	set y_max [winfo screenheight .gauge]
	set x [expr ($x_max-[winfo reqwidth .gauge])/2]
	set y [expr ($y_max-[winfo reqheight .gauge])/2]
	wm geometry .gauge "+$x+$y"
    }

    gauge_create .gauge.g
    pack .gauge.g -expand yes -fill both -padx 10 -pady 10
    .gauge configure -cursor watch

    tkwait visibility .gauge
    grab set .gauge

}

proc gauge_create {win {color ""}} {
    frame $win -class Gauge

    set len [option get $win length Length]
    canvas $win.display -borderwidth 0 -background white \
	-highlightthickness 0 -width $len -height 20
    pack $win.display -expand yes

    if {$color == ""} {
	set color [option get $win color Color]
    }
    $win.display create rectangle 0 0 0 20 \
	-outline "" -fill $color -tags bar
    $win.display create text [expr 0.5*$len] 10 \
	-anchor c -text "0%" -tags value
    return $win

}

proc gauge_value {win val} {

    if {$val < 0 || $val > 100} {
	error "bad value \"$val\": should be 0-100"
    }
    set msg [format "%3.0f%%" $val]
    $win.display itemconfigure value -text $msg

    set w [expr 0.01*$val*[winfo width $win.display]]
    set h [winfo height $win.display]
    $win.display coords bar 0 0 $w $h

    update

}

#==============================================================================
#                         HASYLAB-Daten mergen
#==============================================================================

proc mergeData {datFile mergFile} {
    global blockZahl step blo_min blo_max titel titelZahl

# Initialisierung:

    set izahl_max [expr round( ($blo_max-$blo_min)/double($step) + 1 )]
    for {set n 1} {$n <= $izahl_max} {incr n} {
	set yobs($n) 0.
	set monitor($n) 0.
    }

# Datenfile oeffnen und Titel ueberlesen:

    gaugeWidget "Merge HASYLAB Data"
    set size 0
    set maxSize [file size $datFile(1)]
    set fid [open $datFile(1) r]
    set line ""
    while {[string range $line 0 16] != "titelende     HAS"} {
	gets $fid line
	set size [expr $size + [string length $line] + 1]
	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
    }

# Schleife ueber alle Bloecke.

    set monitor_max 0.
    for {set blockNr 1} {$blockNr <= $blockZahl} {incr blockNr} {

# Block einlesen:

	gets $fid line
	set size [expr $size + [string length $line] + 1]
	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	gets $fid line
	set size [expr $size + [string length $line] + 1]
	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]	
	scan $line "%f %f %f %f %d" temp bla step ble bnr
	set i 0 
	while {[gets $fid line] >=0} {
	    set size [expr $size + [string length $line] + 1]
	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    if {[scan $line "%f %f %f %f" y1 y2 y3 the] == 4} {
		set i [expr $i + 1]
		set y($i) $y1
		set mon($i) $y2
	    } else {
		break
	    }
	    update
	}
	set izahl $i
	set ble [expr $bla + ($izahl-1) * $step] 

# Bloecke aufaddieren:

	for {set n 1} {$n <= $izahl_max} {incr n} {
	    set the [expr $blo_min + ($n-1) * $step]
	    if {[expr abs($the - $bla)] < 0.00001} {
		for {set i 1} {$i <= $izahl} {incr i} {
		    set k [expr $n + $i - 1]
		    set yobs($k) [expr $yobs($k) + $y($i)]
		    set monitor($k) [expr $monitor($k) + $mon($i)]
		    if {$monitor($k) > $monitor_max} {
			set monitor_max $monitor($k)
		    }
		    update
		}
		break
            }
	    update
        }
	unset y mon
    
    }

# Ausgabedatei schliessen:

    close $fid

    destroy .gauge
    update

# Ausgabedatei oeffnen und Kopf schreiben:

    gaugeWidget "Saving Merged Data"
    set fid [open $mergFile "w"]
    for {set j 1} {$j <= $titelZahl} {incr j} {
	puts $fid $titel($j)
    }
    set bNr 1
    set zeit 0.
    set filspe "DAT" 
    puts $fid [format "%s %12.3f%12.5f%12.6f%12.5f%12.4f%4d%8d" \
	$filspe $zeit $blo_min $step $blo_max $temp \
	$bNr $izahl_max]

# Daten im Simref-Datenformat schreiben:

    for {set n 1} {$n <= $izahl_max} {incr n} {
	gauge_value .gauge.g [expr round(($n * 100) / $izahl_max)]
	set the [expr $blo_min + ($n - 1) * $step]
	if {$monitor($n) > 0.0} {
	    set monFaktor [expr $monitor_max / $monitor($n)]
	} else {
	    set monFaktor 1.
        }
	set obs [expr $yobs($n) * $monFaktor]
	set sig [expr sqrt($yobs($n)) * $monFaktor] 
	set bac 0.
	puts $fid [format "%12.5f %14.0f. %14.2f %14.2f" $the $obs $sig $bac]
	update
    }
    close $fid
    unset yobs monitor
    destroy .gauge

}

#==============================================================================
#                    Globale Parameter
#==============================================================================

proc dplot {} {
    global auge3D zScale3D zGrund3D zDach3D box3D line3D \
	edge3D face3D dar3D fileSpec blo_min blo_max temp_min temp_max \
	device xmin xmax tmin tmax

    set w .dplot
    catch {destroy $w}
    toplevel $w
    wm title $w "3D Plot"

    set xmin(data) $blo_min
    set xmax(data) $blo_max
    set tmin $temp_min
    set tmax $temp_max

# 2-theta Range:

    frame $w.xRange
    label $w.xRange.label -text "X Range (2-theta):" -anchor w
    entry $w.xRange.min -width 10 -textvariable xmin(data)
    bind $w.xRange.min <KeyPress> \
	    {if {![regexp "\[0-9\b\t.-\]" "%A"]} {bell;break}}
    label $w.xRange.bis -text "-" -width 2
    entry $w.xRange.max -width 10 -textvariable xmax(data)
    bind $w.xRange.max <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    button $w.xRange.reset -text "Maximize" -command { \
	    set xmin(data) $blo_min
	    set xmax(data) $blo_max
    }
    pack $w.xRange.label -side left -padx 4
    pack $w.xRange.reset $w.xRange.max $w.xRange.bis $w.xRange.min \
	    -side right -padx 4
    pack $w.xRange -side top -fill x -pady 4

# Temperature Range:

    frame $w.yRange
    label $w.yRange.label -text "Y Range (Temp.):" -anchor w
    entry $w.yRange.min -width 10 -textvariable tmin
    bind $w.yRange.min <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    label $w.yRange.bis -text "-" -width 2
    entry $w.yRange.max -width 10 -textvariable tmax
    bind $w.yRange.max <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    button $w.yRange.reset -text "Maximize" -command { \
	set tmin $temp_min 
	set tmax $temp_max
    }
    pack $w.yRange.label -side left -padx 4
    pack $w.yRange.reset $w.yRange.max $w.yRange.bis $w.yRange.min \
	    -side right -padx 4
    pack $w.yRange -side top -fill x -pady 4

    frame $w.sep1 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep1 -fill x -pady 4

# Koordinaten des Auges: x, y, z

    frame $w.auge
    label $w.auge.label -text "Coordinates of the Eye:" -anchor w
    entry $w.auge.x -textvariable auge3D(1) -width 10
    bind $w.auge.x <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    entry $w.auge.y -textvariable auge3D(2) -width 10
    bind $w.auge.y <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    entry $w.auge.z -textvariable auge3D(3) -width 10
    bind $w.auge.z <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    pack $w.auge.label -side left -padx 4
    pack $w.auge.z $w.auge.y $w.auge.x -side right -padx 4
    pack $w.auge -side top -fill x -pady 4

# Skalierungsfaktor fuer z-Werte:

    frame $w.scale
    label $w.scale.label -text "Scale Factor for Z-Values:" -anchor w
    entry $w.scale.entry -textvariable zScale3D -width 10
    bind $w.scale.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    pack $w.scale.label -side left -padx 4
    pack $w.scale.entry -side right -padx 4
    pack $w.scale -side top -fill x -pady 4

# z-Wert der Grundflaeche/Dachflaeche des Kastens:

    frame $w.zValue
    label $w.zValue.label -text "Z Range of the Box:" -anchor w
    entry $w.zValue.min -textvariable zGrund3D -width 10
    bind $w.zValue.min <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    entry $w.zValue.max -textvariable zDach3D -width 10
    bind $w.zValue.max <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    pack $w.zValue.label -side left -padx 4
    pack $w.zValue.max $w.zValue.min -side right -padx 4
    pack $w.zValue -side top -fill x -pady 4

# Kasten:

    frame $w.box
    label $w.box.label -text "Box:" -anchor w
    tk_optionMenu $w.box.option box3D \
	    "Visible" \
	    "Unvisible" \
            "Visible, lower part hatched" \
 	    "Visible, superscribed, hatched"
    $w.box.option config -anchor w
    pack $w.box.label -side left -padx 4
    pack $w.box.option -side right -padx 4
    pack $w.box -side top -fill x -pady 4

# Netzlinien:

    frame $w.grid
    label $w.grid.label -text "Grid:"  -anchor w
    tk_optionMenu $w.grid.option line3D \
	    "All Lines" \
	    "Only Lines in x Direction (2-theta)" \
            "Only lines in y Direction (Temperature)"
    $w.grid.option config -anchor w
    pack $w.grid.label -side left -padx 4
    pack $w.grid.option -side right -padx 4
    pack $w.grid -side top -fill x -pady 4

# Verdeckte Kanten:

    frame $w.edge
    label $w.edge.label -text "Edges:" -anchor w
    tk_optionMenu $w.edge.option edge3D \
	    "Draw all covered Edges" \
            "Suppress all covered Edges" \
	    "Draw only covered Edges of the Box"
    $w.edge.option config -anchor w
    pack $w.edge.label -side left -padx 4
    pack $w.edge.option -side right -padx 4
    pack $w.edge -side top -fill x -pady 4

# Unterseite der Flaeche:

    frame $w.face
    label $w.face.label -text "Lower Face of the Box:" -anchor w
    tk_optionMenu $w.face.option face3D \
	    "Visible" \
	    "Unvisible"
    $w.face.option config -anchor w
    pack $w.face.label -side left -padx 4
    pack $w.face.option -side right -padx 4
    pack $w.face -side top -fill x -pady 4

# Bloecke nacheinander im gleichen Abstand darstellen:

    frame $w.dar
    label $w.dar.label -text "Y Direction:" -anchor w
    tk_optionMenu $w.dar.option dar3D \
	    "Equidistant Blocks" \
	    "Temperature Scaled and Sorted Blocks"
    $w.dar.option config -anchor w
    pack $w.dar.label -side left -padx 4
    pack $w.dar.option -side right -padx 4
    pack $w.dar -side top -fill x -pady 4

# Knoepfe:

    frame $w.sep2 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep2 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command "destroy $w"
    button $w.buttons.plot -text "Plot" -width 7 -command { \
	    set device "Screen"
	    busy_eval {dreidePlot $datFile(1)}
    }
    button $w.buttons.save -text "Save" -width 7 \
	    -command "saveDreide \$datFile(1) .dplot"
    pack $w.buttons.plot $w.buttons.save $w.buttons.dismiss -side left \
	    -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .dplot
}

#==============================================================================
#                         W20-Profilfit
#==============================================================================

proc w20fit {file} {
    global refNr cycleNr

# W20-Inputfile abspeichern:

    saveW20File $file

# Fenster mit R-Werten anzeigen:

    showResiduals

# W20-Programm starten:

    set refNr 0
    set cycleNr -1
    set fid [open "| w20fit" "r+"]
    fconfigure $fid -buffering line
    puts $fid "$file"
    fileevent $fid readable "get_w20data $fid"

    .buttons.b1.w20plot configure -state "normal"
    .buttons.b2.save configure -state "normal"
    .menuBar.plot.m entryconfigure 2 -state "normal"
    .menuBar.w20.m entryconfigure 14 -state "normal"
    .menuBar.w20.m entryconfigure 4 -state "normal"

}

#==============================================================================
#                         Simref starten
#==============================================================================

proc simRefFit {file} {
    global parFile

# Eingabe- und Parameterdatei sichern:

    if {[winfo exists .editInpFile] == 1} {
        saveInpFile $file
	saveParFile $parFile
    }

# Fenster mit Simref-Ausgabe oeffnen:

    showOutput

# Simref starten:

    set fid [open "| simref26 proto.inp" "r+"]
    fconfigure $fid -buffering line
    puts $fid $file
    fileevent $fid readable "get_simOutput $fid"

# Menue und Knoepfe aktivieren:

    .buttons.b1.w20plot configure -state "normal"
    .buttons.b2.save configure -state "normal"

    .menuBar.plot.m entryconfigure 2 -state "normal"
    .menuBar.plot.m entryconfigure 6 -state "normal"
    .menuBar.plot.m entryconfigure 8 -state "normal"

}

#==============================================================================
#                     Simref-Ausgabe lesen
#==============================================================================

proc get_simOutput {fid} {
    global inpFile parFile bla_w20 ble_w20 xmin xmax \
	diagramType plotFile plotFileNr

    if {[gets $fid line] >= 0} {
	.showOutput.out.text insert end "$line\n"
    } else {
	close $fid
	busy_eval {
	    readPlotFile $plotFile($plotFileNr)
	    if {$xmin(W20) == "0." && $xmax(W20) == "1."} {
		set xmin(W20) $bla_w20
		set xmax(W20) $ble_w20
            }
#	    w20Minmax # ist nicht notwendig, da das in readPlotFile gemacht wird
	    readRefFile "[file rootname $plotFile($plotFileNr)].hkl" 
	    set diagramType W20
	    diagram_redraw $diagramType
	    if {[winfo exists .editInpFile] == 1} {
		.editInpFile.inp.text delete 1.0 end
		.editInpFile.par.text delete 1.0 end
		readInpFile $inpFile(Simref)
		readParFile $parFile
	    }
	}
    }

}

#==============================================================================
#                      Reflexlagen-Datei einlesen
#==============================================================================

proc readRefFile {file} {
    global twoThPos numRef refPhase phsZahl refAlpha

    if {[catch {set fid [open $file r]} err] != 0} {
	tk_messageBox -icon warning -type ok -title "Warning" \
	    -message "Cannot open file:\n$file"
	set numRef 0
    } else {
	gaugeWidget "Reading hkl File"
	set size 0
	set maxSize [file size $file]
	gets $fid line
	set size [expr $size + [string length $line] + 1]
	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	gets $fid line
	set size [expr $size + [string length $line] + 1]
	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	set numRef 0
	set phsZahl 1
	while {[gets $fid line] >=0} {
	    set size [expr $size + [string length $line] + 1]
	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    incr numRef
	    set twoThPos($numRef) [lindex $line 3]
	    set refAlpha($numRef) [lindex $line 2]
	    set refPhase($numRef) [lindex $line 1]
	    if {$refPhase($numRef) > $phsZahl} {
		set phsZahl $refPhase($numRef)
	    }
	}
	close $fid
	destroy .gauge
    }

}

#==============================================================================
#                Reflexlagen-Datei fuer Simref-Plot einlesen
#==============================================================================

proc readhklFile {file} {
    global refPhase phsZahl ref_Theta ref_Fwhm ref_Zahl ref_h ref_k ref_l refAlpha
 
    if {[catch {set fid [open $file r]} err] != 0} {
	tk_messageBox -icon warning -type ok -title "Warning" \
		-message "Cannot open file:\n$file"
	set ref_Zahl 0
    } else {
	gaugeWidget "Reading hkl File"
	set size 0
	set maxSize [file size $file]
	gets $fid line
	set size [expr $size + [string length $line] + 1]
	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	gets $fid line
	set size [expr $size + [string length $line] + 1]
	gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	set ref_Zahl 0
	set phsZahl 1
	while {[gets $fid line] >=0} {
	    set size [expr $size + [string length $line] + 1]
	    gauge_value .gauge.g [expr round($size * 100 / $maxSize)]
	    incr ref_Zahl
	    set refPhase($ref_Zahl) [lindex $line 1]
	    if {$refPhase($ref_Zahl) > $phsZahl} {
		set phsZahl $refPhase($ref_Zahl)
	    }
	    set refAlpha($ref_Zahl) [lindex $line 2]
	    set ref_Theta($ref_Zahl) [lindex $line 3]
	    set ref_Fwhm($ref_Zahl) [lindex $line 5]
            set ref_h($ref_Zahl) [expr int([lindex $line 6])]
	    set ref_k($ref_Zahl) [expr int([lindex $line 7])]
	    set ref_l($ref_Zahl) [expr int([lindex $line 8])]
#	    set ref_hkl($ref_Zahl) "$ref_h $ref_k $ref_l"
#	    set ref_hkl($ref_Zahl) "111"
	}
	destroy .gauge
	close $fid
    }

}

#==============================================================================
#                        W20-Ausgabe lesen
#==============================================================================

proc get_w20data {fid} {
    global cycleNr chisq relaxF position fwhm intensity eta1 eta2 asymPar \
	refNr back ratio rp rwp rexp gof nsigma num d_DW q_DW q4_DW \
	stddPos stddFwhm stddInt stddEta1 stddEta2 stddAsym stddBack \
	stddRatio ndata npar plotFile xmin xmax bla_w20 ble_w20 \
	diagramType ymin_all ymax_all

    if {[gets $fid line] >= 0} {
	set laenge [llength $line]
	if {[lindex $line 0] == "Cycle"} {
	    set cycleNr [expr $cycleNr + 1]
	    set chisq [lindex $line 4]
	    set relaxF [lindex $line 7]
	} elseif {[string first "Position (2-theta)" $line] >= 0 \
	    && $laenge == 7} {
	    set refNr [expr $refNr + 1]
	    if {$refNr > 0} {
		set position($refNr) [lindex $line 3]
		set stddPos($refNr) [lindex $line 4]
	    }
	} elseif {[string first "Width (FWHM)" $line] >= 0 && $laenge == 7} {
	    if {$refNr > 0} {
		set fwhm($refNr) [lindex $line 3]
		set stddFwhm($refNr) [lindex $line 4]
	    }
	} elseif {[string first "Integrated intensity" $line] >= 0 \
	    && $laenge == 7} {
	    if {$refNr > 0} {
		set intensity($refNr) [lindex $line 3]
		set stddInt($refNr) [lindex $line 4]
	    }
	} elseif {[string first "Eta_1" $line] >= 0 && $laenge == 6} {
	    if {$refNr > 0} {
		set eta1($refNr) [lindex $line 2]
		set stddEta1($refNr) [lindex $line 3]
	    }
	} elseif {[string first "Eta_2" $line] >= 0 && $laenge == 6} {
	    if {$refNr > 0} {
		set eta2($refNr) [lindex $line 2]
		set stddEta2($refNr) [lindex $line 3]
	    }
	} elseif {[string first "Asymmetry parameter" $line] >= 0 \
	    && $laenge == 7} {
	    if {$refNr > 0} {
		set asymPar($refNr) [lindex $line 3]
		set stddAsym($refNr) [lindex $line 4]
	    }
	} elseif {[string first "Background (constant)" $line] >= 0 \
	    && $laenge == 7} {
	    set back(1) [lindex $line 3]
	    set stddBack(1) [lindex $line 4]
	} elseif {[string first "Background (slope)" $line] >= 0 \
	    && $laenge == 7} {
	    set back(2) [lindex $line 3]
	    set stddBack(2) [lindex $line 4]
	} elseif {[string first "I(K_alpha2)/I(K_alpha1)" $line] >= 0 \
	    && $laenge == 6} {
	    set ratio [lindex $line 2]
	    set stddRatio [lindex $line 3]
	} elseif {[string first "Rp (R-pattern)" $line] >= 0} {
	    set rp [lindex $line 3]
	} elseif {[string first "Rwp (R-weighted pattern)" $line] >= 0} {
	    set rwp [lindex $line 4]
	} elseif {[string first "Rexp (R-expected)" $line] >= 0} {
	    set rexp [lindex $line 3]
	} elseif {[string first "S (Goodness-of-fit)" $line] >= 0} {
	    set gof [lindex $line 3]
	} elseif {[string first "N_sigma" $line] >= 0} {
	    set nsigma [lindex $line 2]
	} elseif {[string first "Number of data points N" $line] >= 0} {
	    set ndata [lindex $line 6]
	} elseif {[string first "Number of parameters P" $line] >= 0} {
	    set npar [lindex $line 5]
	} elseif {[string first "N-P" $line] >= 0} {
	    set num [lindex $line 2]
	} elseif {[lindex $line 0] == "d" && $laenge == 9} {
	    set d_DW [lindex $line 2]
	    set q_DW [lindex $line 5]
	    set q4_DW [lindex $line 8]
	}
    } else {
	close $fid
	busy_eval {
	    readPlotFile $plotFile(1)
	    set xmin(W20) $bla_w20
	    set xmax(W20) $ble_w20
	    w20Minmax
 	    readRefFile "[file rootname $plotFile(1)].hkl"
	    set diagramType W20
	    diagram_redraw $diagramType
	}
    }

}

#==============================================================================
#             Reflexlagen-Datei fuer W20 einlesen
#==============================================================================

proc positionData {} {
    global hklFile hkl_Zahl hkl_Theta hkl_Fwhm \
	refZahl position fwhm intensity eta1 eta2 asymPar \
	refPos refFwhm refInt refEta1 refEta2 refAsym \
	stddPos stddFwhm stddEta1 stddEta2 stddAsym

    set w .positionData
    catch {destroy $w}
    toplevel $w
    wm title $w "Reflection Data"

# Listbox der (x,y)-Werte:

    frame $w.titel
    label $w.titel.label -text "Positions (2-theta), FWHM:" -anchor w
    pack $w.titel.label -side left -padx 4
    pack $w.titel -side top -fill x -pady 4

    frame $w.frame
    pack $w.frame -side top -expand yes -fill y -fill x -padx 4 -pady 4

    scrollbar $w.frame.scroll -command "$w.frame.list yview"
    listbox $w.frame.list -yscroll "$w.frame.scroll set" -setgrid 1 -height 6
    pack $w.frame.scroll -side right -fill y
    pack $w.frame.list -side left -expand 1 -fill both

    for {set i 1} {$i <= $hkl_Zahl} {incr i} {
	$w.frame.list insert 0 "$hkl_Theta($i), $hkl_Fwhm($i)"
    }

# Anzahl der Werte:

    frame $w.anzahl
    label $w.anzahl.label -text "Number of Values:" -anchor w
    label $w.anzahl.wert -textvariable hkl_Zahl -anchor e -width 6 \
	    -relief sunken
    pack $w.anzahl.label -side left -padx 4
    pack $w.anzahl.wert -side left -padx 4
    pack $w.anzahl -side top -fill x -pady 4

# Datei mit Reflexlagen (*.hkl):

    frame $w.hklName
    label $w.hklName.label -text "File:" -anchor w
    entry $w.hklName.entry -textvariable hklFile -width 25 
    pack $w.hklName.label -side left -padx 4
    pack $w.hklName.entry -side right -padx 4
    pack $w.hklName -side top -fill x -pady 4

# Knoepfe:

    frame $w.sep2 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep2 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -command "destroy $w"
    button $w.buttons.open -text "Open..." -command "readRefData $w"
    button $w.buttons.accept -text "Accept" -command {
	set refZahl $hkl_Zahl
	for {set i 1} {$i <= $refZahl} {incr i} {
	    set position($i) $hkl_Theta($i)
	    set fwhm($i) $hkl_Fwhm($i)
	    set intensity($i) 100.
	    set eta1($i) 0.
	    set eta2($i) 0.
	    set asymPar($i) 0.
	    set refPos($i) 1
	    set refFwhm($i) 1
	    set refInt($i) 1
	    set refEta1($i) 0
	    set refEta2($i) 0
	    set refAsym($i) 0
	    set stddPos($i) 0.
	    set stddFwhm($i) 0.
	    set stddInt($i) 0.
	    set stddEta1($i) 0.
	    set stddEta2($i) 0.
	    set stddAsym($i) 0.
	}
	destroy .positiondata
    } 
    pack $w.buttons.open $w.buttons.accept $w.buttons.dismiss \
	-side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .positionData
}

#==============================================================================
#                    Reflexlagen-Datei fuer W20 einlesen
#==============================================================================

proc readRefData {w} {
    global hklFile thmin thmax hkl_Zahl hkl_Theta hkl_Fwhm

    set types {
	{"Reflection Data Files" {.hkl}}
	{"All Files"                  *}
    }
    set file [tk_getOpenFile -filetypes $types -parent $w \
	-initialfile $hklFile -title "Reflection Data File"]

    if {$file != ""} {
	set hklFile $file
	$w.frame.list delete 0 end
	set fid [open $hklFile r] 
	gets $fid
	gets $fid
	set hkl_Zahl 0
	while {[gets $fid line] >=0} {
	    set the [lindex $line 3]
	    set hwb [lindex $line 5]
	    if {$the >= $thmin && $the <= $thmax} {
		set hkl_Zahl [expr $hkl_Zahl + 1]
		$w.frame.list insert end "$the, $hwb"
		set hkl_Theta($hkl_Zahl) $the
		set hkl_Fwhm($hkl_Zahl) $hwb
 	    }
	}
    }

}

#==============================================================================
#                   Ergebnisfile lesen
#==============================================================================

proc saveResults {w} {
    global ergFile titel titelZahl refZahl rwp rexp gof nsigma num chisq \
	position fwhm intensity eta1 asymPar ratio back profFunc blockNr \
	stddPos stddFwhm stddInt stddEta1 stddAsym stddRatio stddBack temp \
	d_DW q_DW

    set types {
	{"Result Files" {.erg}}
	{"All Files"         *}
    }
    set file [tk_getSaveFile -filetypes $types -parent $w \
	-initialfile $ergFile -title "Result File"]

    if {$file != ""} {

	set ergFile $file

# Ergebnisfile einlesen:

	set number 0
	set cmd {set fid [open $ergFile r]}
	if {[catch $cmd err] == 0} {
	    while {[gets $fid line] >= 0} {
		if {[lindex $line 0] == "TITELENDE"} {
		    gets $fid
		    break
		}
	    }
	    while {[gets $fid line] >= 0} {
		set number [expr $number + 1]
	    }
	    close $fid
	}

# File oeffnen und Titel schreiben:

	if {$number > 0} {
	    set fid [open $ergFile a]
	} else {
	    set fid [open $ergFile w]
	    for {set i 1} {$i <= $titelZahl} {incr i} {
		puts $fid $titel($i)
	    }
	    puts $fid "TITELENDE     ERG"
	    puts $fid " BNr    Temp  P     Lage     Hwb      Int    Eta    Asymm  I2/I1      Ug    dUg      Rwp     Rexp      Res       Chisq   N-P     Nsig     d        Q"
	}

# Neue Parameter und R-Werte schreiben:

	switch $profFunc {
	    "Gaussian"                {set prof "G"}
	    "Lorentzian"              {set prof "L"} 
	    "Modified Lorentzian"     {set prof "M"}
	    "Intermediate Lorentzian" {set prof "I"}
	    "Edgeworth Serie"         {set prof "E"}
	    "Pseudo-Voigt"            {set prof "V"}
	    "Pearson VII"             {set prof "P"}
	    "Double Gaussian"         {set prof "D"}
	    default                   {set prof "G"}
	}
	for {set i 1} {$i <= $refZahl} {incr i} {
	    if {$i == 1} {
		puts $fid [format "%4d %7.2f  %s %8.4f %7.4f %8.2f %6.3f %8.4f %6.3f %7.1f %6.2f %8.2f %8.2f %8.2f %11.2f %5d %8.2f %8.4f %8.4f" \
		$blockNr $temp $prof \
		$position($i) $fwhm($i) $intensity($i) $eta1($i) $asymPar($i) \
		$ratio $back(1) $back(2) $rwp $rexp $gof $chisq $num $nsigma \
		$d_DW $q_DW]
		puts $fid [format \
            "                %8.4f %7.4f %8.2f %6.3f %8.4f %6.3f %7.1f %6.2f" \
		$stddPos($i) $stddFwhm($i) $stddInt($i) $stddEta1($i) \
		$stddAsym($i) $stddRatio $stddBack(1) $stddBack(2)]
	    } else {
                puts $fid [format \
	    "              %s %8.4f %7.4f %8.2f %6.3f %8.4f" $prof \
		$position($i) $fwhm($i) $intensity($i) $eta1($i) $asymPar($i)]
		puts $fid [format \
	    "                %8.4f %7.4f %8.2f %7.3f %8.4f" $stddPos($i) \
		$stddFwhm($i) $stddInt($i) $stddEta1($i) $stddAsym($i)]
	    }
	}
	close $fid

# Anzeige erneuern:
	
	.ergFileWidget.inhalt.text delete 1.0 end
	set cmd {set fid [open $ergFile r]}
	if {[catch $cmd err] == 0} {
	    while {[gets $fid line] >= 0} {
		.ergFileWidget.inhalt.text insert end "$line\n"
	    }
	    close $fid
	}
	
    }

}

#==============================================================================
#                    Ergebnis-File-Widget
#==============================================================================

proc ergFileWidget {} {
    global ergFile refZahl

    set w .ergFileWidget
    catch {destroy $w}
    toplevel $w
    wm title $w "Result File: [file tail $ergFile]"

# Inhalt des Ergebnisfiles anzeigen:

    set b $w.inhalt
    frame $b
    scrollbar $b.xsbar -orient horizontal \
	-command {.ergFileWidget.inhalt.text xview}
    scrollbar $b.ysbar -orient vertical \
	-command {.ergFileWidget.inhalt.text yview}
    text $b.text -width 40 -height 5 -wrap none \
	-xscrollcommand {.ergFileWidget.inhalt.xsbar set} \
	-yscrollcommand {.ergFileWidget.inhalt.ysbar set}
    .ergFileWidget.inhalt.text delete 1.0 end

    set cmd {set fid [open $ergFile r]}
    if {[catch $cmd err] == 0} {
	while {[gets $fid line] >= 0} {
	    .ergFileWidget.inhalt.text insert end "$line\n"
	}
	close $fid
    }

    pack $b.xsbar -side bottom -fill x
    pack $b.ysbar -side right -fill y
    pack $b.text -side left -expand yes -fill both
    pack $b  -expand yes -fill both -padx 4 -pady 4

# Anzahl der Eintraege:

    frame $w.anzahl
    label $w.anzahl.label -text "Number of Reflections:" -anchor w
    label $w.anzahl.wert -textvariable refZahl -anchor e -width 6 \
	    -relief sunken
    pack $w.anzahl.label -side left -padx 4
    pack $w.anzahl.wert -side left -padx 4
    pack $w.anzahl -side top -fill x -pady 4

# Buttons:

    frame $w.sep1 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep1 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -command "destroy $w"
    button $w.buttons.save -text "Save..." -command "saveResults $w"
    pack $w.buttons.save $w.buttons.dismiss \
	-side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .ergFileWidget
}

#==============================================================================
#                     R-Werte anzeigen
#==============================================================================

proc showResiduals {} {
    global chisq relaxF nsigma gof refZahl cycleNr rp rwp rexp \
	ndata npar num q_DW d_DW q4_DW

    set w .residuals
    catch {destroy $w}
    toplevel $w
    wm title $w "Residuals"

    after idle {
	update idletasks
        set xw [winfo screenwidth .residuals]
	set x [expr ($xw - [winfo reqwidth .residuals]-10)]
	wm geometry .residuals "+$x+0"
    }

##### Chisquare:

    frame $w.chisq
    label $w.chisq.label -text "Chisquare:" -anchor w -width 8
    label $w.chisq.wert -textvariable chisq -width 15 -relief sunken -anchor e
    pack $w.chisq.label -side left -padx 4
    pack $w.chisq.wert -side right -padx 4
    pack $w.chisq -side top -fill x -pady 4

##### Dyn. Under-Relax. Factor:

    frame $w.urf
    label $w.urf.label -text "Under-relaxation factor:" -anchor w
    label $w.urf.wert -textvariable relaxF -width 15 -relief sunken -anchor e
    pack $w.urf.label -side left -padx 4
    pack $w.urf.wert -side right -padx 4
    pack $w.urf -side top -fill x -pady 4

##### N_sigma:

    frame $w.nsigma
    label $w.nsigma.label -text "N_sigma:" -anchor w -width 8
    label $w.nsigma.wert -textvariable nsigma -width 15 \
	-relief sunken -anchor e
    pack $w.nsigma.label -side left -padx 4
    pack $w.nsigma.wert -side right -padx 4
    pack $w.nsigma -side top -fill x -pady 4

##### Goodness-of-fit:

    frame $w.gof
    label $w.gof.label -text "Goodness-of-fit:" -anchor w
    label $w.gof.wert -textvariable gof -width 15 -relief sunken -anchor e
    pack $w.gof.label -side left -padx 4
    pack $w.gof.wert -side right -padx 4
    pack $w.gof -side top -fill x -pady 4

    frame $w.sep2 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep2 -fill x -pady 4

##### Zahl der Reflexe und Anzahl der Zyklen:

    frame $w.reflexe
    label $w.reflexe.label -text "Number of Reflections:" -anchor w
    label $w.reflexe.wert -textvariable refZahl -width 6 \
	-anchor e -relief sunken
    pack $w.reflexe.label -side left -padx 4
    pack $w.reflexe.wert -side right -padx 4
    pack $w.reflexe -side top -fill x -pady 4

##### Anzahl der Rechenzyklen:

    frame $w.zyklen
    label $w.zyklen.label -text "Number of Cycles:" -anchor w
    label $w.zyklen.wert -textvariable cycleNr -width 6 \
	-relief sunken -anchor e
    pack $w.zyklen.label -side left -padx 4
    pack $w.zyklen.wert -side right -padx 4
    pack $w.zyklen -side top -fill x -pady 4

### Number of data points:

    frame $w.points
    label $w.points.label -text "Number of data points N:" -anchor w
    label $w.points.wert -textvariable ndata -relief sunken -anchor e -width 10
    pack $w.points.label -side left -padx 4
    pack $w.points.wert -side right -padx 4
    pack $w.points -side top -fill x -pady 4

### Number of parameters:

    frame $w.parameters
    label $w.parameters.label -text "Number of parameters P:" -anchor w
    label $w.parameters.wert -textvariable npar -relief sunken \
	-anchor e -width 7
    pack $w.parameters.label -side left -padx 4
    pack $w.parameters.wert -side right -padx 4
    pack $w.parameters -side top -fill x -pady 4

### Degrees of freedom:

    frame $w.freedom
    label $w.freedom.label -text "Degrees of freedom N-P:" -anchor w
    label $w.freedom.wert -textvariable num -relief sunken -anchor e -width 10
    pack $w.freedom.label -side left -padx 4
    pack $w.freedom.wert -side right -padx 4
    pack $w.freedom -side top -fill x -pady 4

    frame $w.sep3 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep3 -fill x -pady 4

##### R-Werte:

    frame $w.rp
    label $w.rp.label -text "R-pattern (Rp):" -anchor w
    label $w.rp.wert -textvariable rp -relief sunken -anchor e -width 10
    pack $w.rp.label -side left -padx 4
    pack  $w.rp.wert -side right -padx 4
    pack $w.rp -side top -fill x -pady 4

    frame $w.rwp
    label $w.rwp.label -text "R-weighted pattern (Rwp):" -anchor w
    label $w.rwp.wert -textvariable rwp -relief sunken -anchor e -width 10
    pack $w.rwp.label -side left -padx 4
    pack $w.rwp.wert -side right -padx 4
    pack $w.rwp -side top -fill x -pady 4

    frame $w.rexp
    label $w.rexp.label -text "R-expected (Rexp):" -anchor w
    label $w.rexp.wert -textvariable rexp -relief sunken -anchor e -width 10
    pack $w.rexp.label -side left -padx 4
    pack $w.rexp.wert -side right -padx 4
    pack $w.rexp -side top -fill x -pady 4

    frame $w.sep4 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep4 -fill x -pady 4

##### Durbin-Watson-Statistik:

    frame $w.dwstat_d
    label $w.dwstat_d.label -text "Durbin-Watson d:" -anchor w
    label $w.dwstat_d.wert -textvariable d_DW -relief sunken \
	-anchor e -width 10
    pack $w.dwstat_d.label -side left -padx 4
    pack $w.dwstat_d.wert -side right -padx 4
    pack $w.dwstat_d -side top -fill x -pady 4

    frame $w.dwstat_Q
    label $w.dwstat_Q.label -text "Durbin-Watson Q:" -anchor w
    label $w.dwstat_Q.wert -textvariable q_DW -relief sunken \
	-anchor e -width 10
    pack $w.dwstat_Q.label -side left -padx 4
    pack  $w.dwstat_Q.wert -side right -padx 4
    pack $w.dwstat_Q -side top -fill x -pady 4

    frame $w.dwstat_4Q
    label $w.dwstat_4Q.label -text "Durbin-Watson 4-Q:" -anchor w
    label $w.dwstat_4Q.wert -textvariable q4_DW -relief sunken \
	-anchor e -width 10
    pack $w.dwstat_4Q.label -side left -padx 4
    pack  $w.dwstat_4Q.wert -side right -padx 4
    pack $w.dwstat_4Q -side top -fill x -pady 4

### Knoepfe:

    frame $w.sep5 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep5 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.dismiss -text "Dismiss" -command "destroy $w"
    pack $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

}

#==============================================================================
#                        W20-Picasso-Plot-Eingabe
#==============================================================================

proc w20PlotPar {} {
    global w20symbolShape w20symbolColor w20symbolSize w20lineDraw \
	w20symbolFillColor w20lineWidth w20lineColor symbolList colorList

    set w .w20PlotPar
    catch {destroy $w}
    toplevel $w
    wm title $w "Symbols and Lines"

### Observed Values:

# Symbols:

    frame $w.symbol_obs
    label $w.symbol_obs.label1 -text "Symbols (obs):" -anchor w
    label $w.symbol_obs.label2 -text "Shape:" -anchor w
    label $w.symbol_obs.label3 -text "Color:" -anchor w
    
    [eval tk_optionMenu $w.symbol_obs.shape w20symbolShape(1) $symbolList] \
	configure -tearoff no
    $w.symbol_obs.shape config -anchor w
    [eval tk_optionMenu $w.symbol_obs.color w20symbolColor(1) $colorList] \
	configure -tearoff no
    $w.symbol_obs.color config -anchor w

    pack $w.symbol_obs.label1 -side left -padx 4
    pack $w.symbol_obs.color $w.symbol_obs.label3 -side right -padx 4
    pack $w.symbol_obs.shape $w.symbol_obs.label2 -side right -padx 4
    pack $w.symbol_obs -side top -fill x -pady 4

    frame $w.symbol_size
    label $w.symbol_size.label1 -text "Size:" -anchor w
    label $w.symbol_size.label2 -text "Fill Color:" -anchor w
    [eval tk_optionMenu $w.symbol_size.fill w20symbolFillColor(1) $colorList] \
	configure -tearoff no
    $w.symbol_size.fill config -anchor w

    entry $w.symbol_size.size -textvariable w20symbolSize(1) -width 8
    bind $w.symbol_size.size <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    pack $w.symbol_size.size $w.symbol_size.label1 -side right -padx 4
    pack $w.symbol_size.fill $w.symbol_size.label2 -side right -padx 4
    pack $w.symbol_size -side top -fill x -pady 4

    frame $w.sep2 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep2 -fill x -pady 4

### Lines:

# Observed Values:

    frame $w.line_obs

    checkbutton $w.line_obs.button -text "Line (obs):" \
	    -variable w20lineDraw(1) -relief flat

    label $w.line_obs.label2 -text "Width:" -anchor w
    label $w.line_obs.label3 -text "Color:" -anchor w

    [eval tk_optionMenu $w.line_obs.color w20lineColor(1) $colorList] \
	configure -tearoff no
    $w.line_obs.color config -anchor w

    entry $w.line_obs.width -textvariable w20lineWidth(1) -width 8
    bind $w.line_obs.width <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}

    pack $w.line_obs.button -side left -padx 4
    pack $w.line_obs.width $w.line_obs.label2 -side right -padx 4
    pack $w.line_obs.color $w.line_obs.label3 -side right -padx 4
    pack $w.line_obs -side top -fill x -pady 4

# Calculated Values:

    frame $w.line_calc

    checkbutton $w.line_calc.button -text "Line (calc):" \
	    -variable w20lineDraw(2) -relief flat

    label $w.line_calc.label2 -text "Width:" -anchor w
    label $w.line_calc.label3 -text "Color:" -anchor w

    [eval tk_optionMenu $w.line_calc.color w20lineColor(2) $colorList] \
	configure -tearoff no
    $w.line_calc.color config -anchor w

    entry $w.line_calc.width -textvariable w20lineWidth(2) -width 8
    bind $w.line_calc.width <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}

    pack $w.line_calc.button -side left -padx 4
    pack $w.line_calc.width $w.line_calc.label2 -side right -padx 4
    pack $w.line_calc.color $w.line_calc.label3 -side right -padx 4
    pack $w.line_calc -side top -fill x -pady 4

# Difference Values:

    frame $w.line_diff

    checkbutton $w.line_diff.button -text "Line (diff):" \
	    -variable w20lineDraw(3) -relief flat

    label $w.line_diff.label2 -text "Width:" -anchor w
    label $w.line_diff.label3 -text "Color:" -anchor w

    [eval tk_optionMenu $w.line_diff.color w20lineColor(3) $colorList] \
	configure -tearoff no
    $w.line_diff.color config -anchor w

    entry $w.line_diff.width -textvariable w20lineWidth(3) -width 8
    bind $w.line_diff.width <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}

    pack $w.line_diff.button -side left -padx 4
    pack $w.line_diff.width $w.line_diff.label2 -side right -padx 4
    pack $w.line_diff.color $w.line_diff.label3 -side right -padx 4
    pack $w.line_diff -side top -fill x -pady 4

# Background Values:

    frame $w.line_back

    checkbutton $w.line_back.button -text "Line (back):" \
	    -variable w20lineDraw(4) -relief flat

    label $w.line_back.label2 -text "Width:" -anchor w
    label $w.line_back.label3 -text "Color:" -anchor w

    [eval tk_optionMenu $w.line_back.color lineColor(4) $colorList] \
	configure -tearoff no
    $w.line_back.color config -anchor w

    entry $w.line_back.width -textvariable w20lineWidth(4) -width 8
    bind $w.line_back.width <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}

    pack $w.line_back.button -side left -padx 4
    pack $w.line_back.width $w.line_back.label2 -side right -padx 4
    pack $w.line_back.color $w.line_back.label3 -side right -padx 4
    pack $w.line_back -side top -fill x -pady 4

# Knoepfe:

    frame $w.sep3 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep3 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.apply -text "Apply" -width 7 -command {
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command {
	destroy .w20PlotPar
	busy_eval {diagram_redraw $diagramType}
    }
    pack $w.buttons.apply $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .w20PlotPar
}

#==============================================================================
#                  Plot-Parameter einstellen
#==============================================================================

proc plotPar {} {
    global fileSpec diagramType

    if {$diagramType == "W20"} {
	w20PlotPar
    } else {
	if {$fileSpec == "FOR" || $fileSpec == "ERG"} {
	    colPlotPar
	} else {
	    dataPlotPar
	}
    }

}

#==============================================================================
#                        W20-Picasso-Plot-Eingabe
#==============================================================================

proc dataPlotPar {} {
    global symbolShape symbolColor symbolSize lineDraw lineWidth lineColor \
	fileSpec symbolList colorList refPhaseColor hkl_size hklDraw refAlpha\
	colorListT posList

    set w .dataPlotPar
    catch {destroy $w}
    toplevel $w
    wm title $w "Symbols and Lines"

### Observed Values:

# Symbols:

    frame $w.symbol_obs
    label $w.symbol_obs.label1 -text "Symbols (obs):" -anchor w
    label $w.symbol_obs.label2 -text "Shape:" -anchor w
    label $w.symbol_obs.label3 -text "Color:" -anchor w

    [eval tk_optionMenu $w.symbol_obs.shape symbolShape(1) $symbolList] \
	configure -tearoff no
    $w.symbol_obs.shape config -anchor w
    [eval tk_optionMenu $w.symbol_obs.color symbolColor(1) $colorList] \
	configure -tearoff no
    $w.symbol_obs.color config -anchor w

    pack $w.symbol_obs.label1 -side left -padx 4
    pack $w.symbol_obs.color $w.symbol_obs.label3 -side right -padx 4
    pack $w.symbol_obs.shape $w.symbol_obs.label2 -side right -padx 4
    pack $w.symbol_obs -side top -fill x -pady 4

    frame $w.symbol_size
    label $w.symbol_size.label1 -text "Size:" -anchor w
    label $w.symbol_size.label2 -text "Fill Color:" -anchor w
    label $w.symbol_size.label3 -text "Rebmann:" -anchor w
    [eval tk_optionMenu $w.symbol_size.fill symbolFillColor(1) $colorList] \
	configure -tearoff no
    $w.symbol_size.fill config -anchor w

    entry $w.symbol_size.size -textvariable symbolSize(1) -width 8
    bind $w.symbol_size.size <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}
    entry $w.symbol_size.rebmann -textvariable rebfakt(1) -width 8
    bind $w.symbol_size.rebmann <KeyPress> \
	{if {![regexp "\[0-9\.\t\b\]" "%A"]} {bell;break}}
    pack $w.symbol_size.size $w.symbol_size.label1 -side right -padx 4
    pack $w.symbol_size.fill $w.symbol_size.label2 -side right -padx 4
    pack $w.symbol_size.rebmann $w.symbol_size.label3 -side right -padx 4
    pack $w.symbol_size -side top -fill x -pady 4

    frame $w.sep2 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep2 -fill x -pady 4

### Lines:

# Observed Values:

    frame $w.line_obs

    checkbutton $w.line_obs.button -text "Line (obs):" \
	    -variable lineDraw(1) -relief flat

    label $w.line_obs.label2 -text "Width:" -anchor w
    label $w.line_obs.label3 -text "Color:" -anchor w

    [eval tk_optionMenu $w.line_obs.color lineColor(1) $colorList] \
	configure -tearoff no
    $w.line_obs.color config -anchor w

    entry $w.line_obs.width -textvariable lineWidth(1) -width 8
    bind $w.line_obs.width <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}

    pack $w.line_obs.button -side left -padx 4
    pack $w.line_obs.width $w.line_obs.label2 -side right -padx 4
    pack $w.line_obs.color $w.line_obs.label3 -side right -padx 4
    pack $w.line_obs -side top -fill x -pady 4

    if {$fileSpec == "SIM"} {

# Calculated Values:

	frame $w.line_calc

	checkbutton $w.line_calc.button -text "Line (calc):" \
		-variable lineDraw(2) -relief flat

	label $w.line_calc.label2 -text "Width:" -anchor w
	label $w.line_calc.label3 -text "Color:" -anchor w

	[eval tk_optionMenu $w.line_calc.color lineColor(2) $colorList] \
	    configure -tearoff no
	$w.line_calc.color config -anchor w

	entry $w.line_calc.width -textvariable lineWidth(2) -width 8
	bind $w.line_calc.width <KeyPress> \
	    {if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}

	pack $w.line_calc.button -side left -padx 4
	pack $w.line_calc.width $w.line_calc.label2 -side right -padx 4
	pack $w.line_calc.color $w.line_calc.label3 -side right -padx 4
	pack $w.line_calc -side top -fill x -pady 4

# Difference Values:

	frame $w.line_diff

	checkbutton $w.line_diff.button -text "Line (diff):" \
		-variable lineDraw(3) -relief flat

	label $w.line_diff.label2 -text "Width:" -anchor w
	label $w.line_diff.label3 -text "Color:" -anchor w

	[eval tk_optionMenu $w.line_diff.color lineColor(3) $colorList] \
	    configure -tearoff no
	$w.line_diff.color config -anchor w

	entry $w.line_diff.width -textvariable lineWidth(3) -width 8
	bind $w.line_diff.width <KeyPress> \
	    {if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}

	pack $w.line_diff.button -side left -padx 4
	pack $w.line_diff.width $w.line_diff.label2 -side right -padx 4
	pack $w.line_diff.color $w.line_diff.label3 -side right -padx 4
	pack $w.line_diff -side top -fill x -pady 4

    }

# Background Values:

    if {$fileSpec == "SIM" || $fileSpec == "DAT"} {

	frame $w.line_back

	checkbutton $w.line_back.button -text "Line (back):" \
		-variable lineDraw(4) -relief flat

	label $w.line_back.label2 -text "Width:" -anchor w
	label $w.line_back.label3 -text "Color:" -anchor w

	[eval tk_optionMenu $w.line_back.color lineColor(4) $colorList] \
	    configure -tearoff no
	$w.line_back.color config -anchor w

	entry $w.line_back.width -textvariable lineWidth(4) -width 8
	bind $w.line_back.width <KeyPress> \
	    {if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}

	pack $w.line_back.button -side left -padx 4
	pack $w.line_back.width $w.line_back.label2 -side right -padx 4
	pack $w.line_back.color $w.line_back.label3 -side right -padx 4
	pack $w.line_back -side top -fill x -pady 4

    }

# Standard Deviations:

    if {$fileSpec == "DAT"} {

	frame $w.line_sig

	checkbutton $w.line_sig.button -text "Error Bars (sigma):" \
		-variable lineDraw(5) -relief flat

	label $w.line_sig.label2 -text "Width:" -anchor w
	label $w.line_sig.label3 -text "Color:" -anchor w

	[eval tk_optionMenu $w.line_sig.color lineColor(5) $colorList] \
	    configure -tearoff no
	$w.line_sig.color config -anchor w

	entry $w.line_sig.width -textvariable lineWidth(5) -width 8
	bind $w.line_sig.width <KeyPress> \
	    {if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}

	pack $w.line_sig.button -side left -padx 4
	pack $w.line_sig.width $w.line_sig.label2 -side right -padx 4
	pack $w.line_sig.color $w.line_sig.label3 -side right -padx 4
	pack $w.line_sig -side top -fill x -pady 4

    }

# Reflexmarkierungen

    frame $w.hkl

    checkbutton $w.hkl.button -text "HKL-values" \
	    -variable hklDraw -relief flat
    checkbutton $w.hkl.alpha1 -text "Alpha1" \
	    -variable hkl_alpha1 -relief flat
    checkbutton $w.hkl.alpha2 -text "Alpha2" \
	    -variable hkl_alpha2 -relief flat
    label $w.hkl.label2 -text "Size:" -anchor w
    label $w.hkl.label3 -text "Color:" -anchor w

    [eval tk_optionMenu $w.hkl.lage hkl_pos $posList] configure -tearoff no
    $w.hkl.lage config  -anchor w

    [eval tk_optionMenu $w.hkl.colorA refPhaseColor(1) $colorListT] \
	configure -tearoff no
    $w.hkl.colorA config -anchor w

    [eval tk_optionMenu $w.hkl.colorB refPhaseColor(2) $colorListT] \
	    configure -tearoff no
    $w.hkl.colorB config -anchor w
    
    [eval tk_optionMenu $w.hkl.colorC refPhaseColor(3) $colorListT] \
	    configure -tearoff no
    $w.hkl.colorC config -anchor w
    
    [eval tk_optionMenu $w.hkl.colorD refPhaseColor(4) $colorListT] \
	    configure -tearoff no
    $w.hkl.colorD config -anchor w
    
    [eval tk_optionMenu $w.hkl.colorE refPhaseColor(5) $colorListT] \
	    configure -tearoff no
    $w.hkl.colorE config -anchor w
    
    entry $w.hkl.width -textvariable hkl_size -width 8
    bind $w.hkl.width <KeyPress> \
	{if {![regexp "\[0-9\t\b\]" "%A"]} {bell;break}}

    pack $w.hkl.button -side left -padx 4
    pack $w.hkl.alpha1 -side left -padx 4
    pack $w.hkl.alpha2 -side left -padx 4
    pack $w.hkl.width $w.line_obs.label2 -side right -padx 4
    pack $w.hkl.colorE $w.line_obs.label3 -side right -padx 4
    pack $w.hkl.colorD $w.line_obs.label3 -side right -padx 4
    pack $w.hkl.colorC $w.line_obs.label3 -side right -padx 4
    pack $w.hkl.colorB $w.line_obs.label3 -side right -padx 4
    pack $w.hkl.colorA $w.line_obs.label3 -side right -padx 4
    pack $w.hkl.lage $w.line_obs.label3 -side right -padx 4
    pack $w.hkl -side top -fill x -pady 4    



# Knoepfe:

    frame $w.sep3 -height 2 -borderwidth 1 -relief sunken
    pack $w.sep3 -fill x -pady 4

    frame $w.buttons
    button $w.buttons.apply -text "Apply" -width 7 -command {
	font configure "hkl_font" -size $hkl_size
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command {
	font configure "hkl_font" -size $hkl_size
	destroy .dataPlotPar
	busy_eval {diagram_redraw $diagramType}
    }
    pack $w.buttons.apply $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .dataPlotPar
}

#==============================================================================
#                      Postscript-Ausgabe
#==============================================================================

proc draw_print {} {
    global dinA4
    set x0 0
    set y0 0
    set x1 [winfo width .diagram]
    set y1 [winfo height .diagram]
#    foreach {x0 y0 x1 y1} [.diagram bbox all] {}
#    set w [expr $x1-$x0]
#    set h [expr $y1-$y0]
    if {[expr double($x1) / double($y1)] >= $dinA4} {
	set w [expr $y1 * $dinA4]
	set h $y1
    } else {
	set w $x1
	set h [expr double($x1) / $dinA4]
    }
    return [.diagram postscript -x $x0 -y $y0 -width $w -height $h \
	-rotate yes -pageheight 210m]
}

#==============================================================================
#           Eingabe der Blocknummer und der Zaehlernummer
#==============================================================================

proc selBlockPar {} {
    global blockNr blockZahl countNr countZahl countDist fileSpec \
	datFile diagramType xmin xmax thmin thmax

    set w .selBlockPar
    catch {destroy $w}
    toplevel $w
    wm title $w "Block"

    set b $blockNr
    set c $countNr
    set d $countDist

# BlockNummer:

    set blockList {}
    for {set i 1} {$i <= $blockZahl} {incr i} {
	lappend blockList $i
    }
    frame $w.blockNr
    label $w.blockNr.label -text "Block Number:" -anchor w
    [eval tk_optionMenu $w.blockNr.werte blockNr $blockList] \
	    configure -tearoff no
    $w.blockNr.werte config -anchor w -width 5
    pack $w.blockNr.label -side left -padx 4
    pack $w.blockNr.werte -side right -padx 4
    pack $w.blockNr -side top -fill x -pady 4

# Zaehlernummer:

    set countList {}
    for {set i 1} {$i <= $countZahl} {incr i} {
	lappend countList $i
    }
    frame $w.countNr
    label $w.countNr.label -text "Counter:" -anchor w
    [eval tk_optionMenu $w.countNr.werte countNr $countList] \
	    configure -tearoff no
    $w.countNr.werte config -anchor w -width 5
    pack $w.countNr.label -side left -padx 4
    pack $w.countNr.werte -side right -padx 4
    pack $w.countNr -side top -fill x -pady 4

# Abstand der Zaehler:

    if {$fileSpec == "SCN" || $fileSpec == "SCN"} {
	frame $w.countAbst
	label $w.countAbst.label -text "Counter Distance:" -width 15 -anchor w
	entry $w.countAbst.entry -textvariable countDist -width 8
	bind $w.countAbst.entry <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
	pack $w.countAbst.label $w.countAbst.entry -side left -padx 4
	pack $w.countAbst -side top -fill x -pady 4
    }

# Knoepfe:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.ok -text "Ok" -width 7 -command {
	    destroy .selBlockPar
	    busy_eval {
		readDataFile $datFile(1)
		xMinmax $fileSpec
		yMinmax $fileSpec
		set thmin $xmin(data)
		set thmax $xmax(data)
		set diagramType data
		diagram_redraw $diagramType
	    }
    }
    button $w.buttons.cancel -text "Cancel" -width 7 \
	    -command "set blockNr \"$b\";
		      set countNr \"$c\";
		      set countDist \"$d\";
		      destroy $w"
    pack $w.buttons.ok $w.buttons.cancel -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .selBlockPar
}

#==============================================================================
#                    X- und Y-Bereich setzen
#==============================================================================

proc xyRange {} {
    global diagramType fileSpec xmin xmax ymin ymax thmin thmax

    set w .xyRange
    catch {destroy $w}
    toplevel $w
    wm title $w "X and Y Scaling"

    set x1 $xmin($diagramType)
    set x2 $xmax($diagramType)
    set y1 $ymin($diagramType)
    set y2 $ymax($diagramType)

# X Range:

    frame $w.xRange
    label $w.xRange.label -text "X Range:" -anchor w
    entry $w.xRange.min -width 10 -textvariable xmin($diagramType)
    bind $w.xRange.min <KeyPress> \
	    {if {![regexp "\[0-9\b\t.-\]" "%A"]} {bell;break}}
    label $w.xRange.bis -text "-" -width 2
    entry $w.xRange.max -width 10 -textvariable xmax($diagramType)
    bind $w.xRange.max <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    button $w.xRange.reset -text "Maximize" -command { \
	if {$diagramType == "data"} {
	    busy_eval {
		xMinmax $fileSpec
		yMinmax $fileSpec
	    }
	} else {
	    set xmin($diagramType) $bla_w20
	    set xmax($diagramType) $ble_w20
	    busy_eval {w20Minmax}
	}
    }
    pack $w.xRange.label -side left -padx 4
    pack $w.xRange.reset $w.xRange.max $w.xRange.bis $w.xRange.min \
	    -side right -padx 4
    pack $w.xRange -side top -fill x -pady 4

# Y Range:

    frame $w.yRange
    label $w.yRange.label -text "Y Range:" -anchor w
    entry $w.yRange.min -width 10 -textvariable ymin($diagramType)
    bind $w.yRange.min <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    label $w.yRange.bis -text "-" -width 2
    entry $w.yRange.max -width 10 -textvariable ymax($diagramType)
    bind $w.yRange.max <KeyPress> \
	    {if {![regexp "\[0-9\t\b.-\]" "%A"]} {bell;break}}
    button $w.yRange.reset -text "Maximize" -command {
	if {$diagramType == "data"} {
	    busy_eval {yMinmax $fileSpec}
	} else {
	    busy_eval {w20Minmax}
	}
    }
    pack $w.yRange.label -side left -padx 4
    pack $w.yRange.reset $w.yRange.max $w.yRange.bis $w.yRange.min \
	    -side right -padx 4
    pack $w.yRange -side top -fill x -pady 4

# Separator:

    frame $w.sep -height 2 -borderwidth 1 -relief sunken
    pack $w.sep -fill x -pady 4

    frame $w.buttons
    button $w.buttons.apply -text "Apply" -width 7 -command {
	set thmin $xmin($diagramType)
	set thmax $xmax($diagramType)
	busy_eval {diagram_redraw $diagramType}
    }
    button $w.buttons.dismiss -text "Dismiss" -width 7 -command {
	set thmin $xmin($diagramType)
	set thmax $xmax($diagramType)
	destroy .xyRange
	busy_eval {diagram_redraw $diagramType}
    }
    pack $w.buttons.apply $w.buttons.dismiss -side left -expand yes -padx 4
    pack $w.buttons -side bottom -fill x -pady 4

    grab set .xyRange
}

